<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * ConvertKit API Class
 *
 */
class SliceWP_ConvertKit_API {

	/**
	 * API Key
	 *
	 */
	private $api_key;

	/**
	 * The arguments used for API requests
	 *
	 */
	private $request_args;

	/**
	 * Last API error
	 *
	 */
	private $last_error;


	/**
	 * Constructor
	 * 
	 */
	public function __construct() {}


	/**
	 * Init the ConvertKit API
	 * 
	 * @param string $api_key
	 * 
	 * @return bool
	 * 
	 */
	public function init( $api_key ) {

		if ( empty( $api_key ) ) {
			return false;
		}

		// Set the API key
		$this->api_key = $api_key;

		// Set the request args
		$this->request_args = array(
			'user-agent'	=> 'SliceWP; ' . home_url(),
			'timeout'		=> 30,
			'headers'		=> array(
				'Content-Type'	=> 'application/json',
			),
		);

		// Check if ConvertKit connection works
		return $this->ping() ? true : false;

	}


	/**
	 * Pings the ConvertKit API
	 * 
	 * @return bool
	 * 
	 */
	public function ping() {

		// Check for API request args
		if ( ! $this->request_args ) {
			return false;
		}

		// Make an API request to see if everything is configured correct
		$request_url = 'https://api.convertkit.com/v3/tags?api_key=' . $this->api_key;
		$response 	 = wp_safe_remote_get( $request_url, $this->request_args );

		// Check for WP Error
		if ( is_wp_error( $response ) ) {

			$this->last_error = $response->get_error_message();
			return false;

		}

		$response_body = json_decode( wp_remote_retrieve_body( $response ) );

		// Check for body
		if ( empty( $response_body ) ) {

			$this->last_error = __( 'Empty response received.' , 'slicewp' );
			return false;

		}

		// Check for errors
		if ( isset( $response_body->error ) ) {

			$this->last_error = $response_body->message;
			return false;

		}

		return true;

	}


	/**
	 * Gets the ConvertKit tags
	 * 
	 * @return array|null
	 * 
	 */
	public function get_tags() {

		// Check for API request args
		if ( ! $this->request_args ) {
			return null;
		}

		// Get the ConvertKit tags
		$request_url = 'https://api.convertkit.com/v3/tags?api_key=' . $this->api_key;
		$response 	 = wp_safe_remote_get( $request_url, $this->request_args );

		// Check for WP Error
		if ( is_wp_error( $response ) ) {

			$this->last_error = $response->get_error_message();
			return null;

		}

		$tags_api = json_decode( wp_remote_retrieve_body( $response ) );
		
		// Check for body
		if ( empty( $tags_api ) ) {

			$this->last_error = __( 'Empty response received.' , 'slicewp' );
			return null;

		}

		// Check for errors
		if ( isset( $tags_api->error ) ) {

			$this->last_error = $tags_api->message;
			return null;

		}

		$tags = array();

		// We don't have any tags
		if ( empty( $tags_api->tags ) ) {
			return $tags;
		}

		foreach ( $tags_api->tags as $key => $tag ) {

			$tags[ $tag->id ] = $tag->name;

		}

		return $tags;

	}


	/**
	 * Saves the ConvertKit API data in settings
	 * 
	 */
	public function save_api_data() {

		// Get the saved API data
		$api_data = slicewp_get_option( 'convertkit_api_data', array() );

		// Get the tags from ConvertKit
		$api_tags = $this->get_tags();

		// Save the groups
		if ( ! is_null( $api_tags ) ) {
			$api_data['tags'] = $api_tags;
		}

		// Update the API data
		slicewp_update_option( 'convertkit_api_data', $api_data );

		// Save the API key in settings
		$settings = slicewp_get_option( 'settings', array() );
		$settings['convertkit_api_key'] = $this->api_key;

		// Update general settings
		slicewp_update_option( 'settings', $settings );

	}


	/**
	 * Outputs the ConvertKit tags
	 * 
	 */
	public function output_tags() {

		// Get tags data
		$api_data = slicewp_get_option( 'convertkit_api_data', array() );
		$tags 	  = ( ! empty( $api_data['tags'] ) ? $api_data['tags'] : '' );

		// Get the selected tags
		$selected_tags = slicewp_get_setting( 'convertkit_tags', array() );

		?>

		<!-- Tags -->
		<div id="slicewp-convertkit-tags" class="slicewp-field-wrapper slicewp-field-wrapper-inline">

			<div class="slicewp-field-label-wrapper">
				<label for="slicewp-convertkit-tags">
					<?php echo __( 'Tags', 'slicewp' ); ?> *
				</label>
			</div>

			<?php if ( ! empty( $tags ) ): ?>

				<select id="slicewp-convertkit-tags-select" name="settings[convertkit_tags][]" class="slicewp-select2-convertkit-tags" multiple placeholder="<?php echo __( 'Select tags...', 'slicewp' ); ?>">

					<?php

						if ( ! empty( $tags ) ) {

							foreach ( $tags as $key => $tag ) {
								
								echo '<option value="' . $key . '"' . selected( 1, in_array( $key, $selected_tags )  ) . '>' . $tag . '</option>';

							}

						}

					?>

				</select>

			<?php else: ?>

				<div class="slicewp-field-notice slicewp-field-notice-warning" style="display: block;">
					<p><?php echo __( "We couldn't find any tags for your mailing list. At least one tag needs to be configured so we can subscribe affiliates.", 'slicewp' ); ?></p>
					<p><?php echo __( 'Create a tag for your mailing list in ConvertKit and then click the above Connect button.', 'slicewp' ); ?></p>
				</div>

			<?php endif; ?>

		</div><!-- / Tags -->

	<?php

	}


	/**
	 * Adds the user in ConvertKit
	 * 
	 * @param array $data
	 * 
	 */
	public function add_subscriber( $data ) {

		// Check for email
		if ( empty( $data['email'] ) ) {
			return;
		}

		// Check for API request args
		if ( ! $this->request_args ) {
			return;
		}

		// Get ConvertKit tags
		$tags = slicewp_get_setting( 'convertkit_tags' );

		if ( empty( $tags ) ) {
			return;
		}

		// Prepare the API call data
		$data = array(
			'api_key'		=> $this->api_key,
			'email'			=> sanitize_email( $data['email'] ),
			'first_name'	=> ! empty( $data['first_name'] ) ? sanitize_text_field( $data['first_name'] ) : '',
			'fields'		=> array(
								'last_name' => ! empty ( $data['last_name'] ) ? sanitize_text_field( $data['last_name'] ) : ''
			),
			'tags'			=> implode( ',', $tags )
		);

		$request_args			= $this->request_args;
		$request_args['method']	= 'POST';
		$request_args['body']	= json_encode( $data );

		// Prepare API request
		$request_url = 'https://api.convertkit.com/v3/tags/' . $tags[0] . '/subscribe';
		$response 	 = wp_safe_remote_get( $request_url, $request_args );

		// Check for WP Error
		if ( is_wp_error( $response ) ) {

			slicewp_add_log( 'ConvertKit API: add_subscriber() failed. Error: ' . $response->get_error_message() );
			return;

		}

		$subscription = json_decode( wp_remote_retrieve_body( $response ) );

		// Check for body
		if ( empty( $subscription ) ) {

			slicewp_add_log( 'ConvertKit API: add_subscriber() failed. Error: Empty response received.' );
			return null;

		}
		
		// Check for errors
		if ( isset( $subscription->error ) ) {

			slicewp_add_log( 'ConvertKit API: add_subscriber() failed. Error: ' . $subscription->message );

		}

	}


	/**
	 * Returns the last API error
	 * 
	 * @return string
	 * 
	 */
	public function get_last_error() {

		return $this->last_error;

	}

}


/**
 * Register the ConvertKit API as a service.
 *
 * @param array
 *
 * @return array
 *
 */
function slicewp_register_service_convertkit_api( $services ) {

	$services['convertkit'] = new SliceWP_ConvertKit_API();

	return $services;

}
add_filter( 'slicewp_register_service', 'slicewp_register_service_convertkit_api' );