<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Handles the editing of the custom affiliate fields
 *
 */
class SliceWP_Affiliate_Fields_Editor {

	/**
	 * The fields data saved in the DB.
	 *
	 * @access protected
	 * @var    array
	 *
	 */
	protected $fields;

	/**
	 * Constructor
	 *
	 */
	public function __construct() {

		// Set up fields
		$this->fields = slicewp_get_option( 'affiliate_fields', array() );

		// Add new field AJAX callback
		add_action( 'wp_ajax_slicewp_affiliate_fields_editor_add_new_field', array( $this, 'add_new_field' ) );

		// Localize field types
		add_action( 'admin_enqueue_scripts', array( $this, 'localize_field_types' ), 100 );

		// Save affiliate fields settings
		add_action( 'slicewp_admin_action_save_settings', array( $this, 'admin_action_save_settings' ), 40 );

	}


	/**
	 * Returns an array with the available field type groups
	 *
	 * @return array
	 *
	 */
	public function get_field_types_groups() {

		$groups = array(
			'predefined' => __( 'Predefined', 'slicewp' ),
			'standard'	 => __( 'Standard', 'slicewp' ),
			'location'   => __( 'Location', 'slicewp' )
		);

		return $groups;

	}


	/**
	 * Returns an array of field types that can be added to the editor
	 *
	 * @return array
	 *
	 */
	public function get_field_types() {

		$field_types = array(

			/**
			 * Predefined group fields
			 *
			 */
			'user_login' => array(
				'nicename'	  	   => __( 'Username', 'slicewp' ),
				'group'		  	   => 'predefined',
				'key'		  	   => 'user_login',
				'form_unique' 	   => true,
				'default_required' => true,
				'settings'	  	   => array( 'label', 'description', 'key', 'form', 'placeholder', 'description_placement', 'field_class', 'wrapper_class' )
			),
			'user_email' => array(
				'nicename' 	  	=> __( 'Email Address', 'slicewp' ),
				'group'		  	=> 'predefined',
				'key'		    => 'user_email',
				'form_unique' 	=> true,
				'form_required' => true,
				'settings'	    => array( 'label', 'description', 'key', 'form', 'placeholder', 'description_placement', 'field_class', 'wrapper_class' )
			),
			'first_name' => array(
				'nicename'	  => __( 'First Name', 'slicewp' ),
				'group'		  => 'predefined',
				'key'		  => 'first_name',
				'form_unique' => true,
				'settings'	  => array( 'label', 'description', 'is_required', 'key', 'form', 'placeholder', 'description_placement', 'field_class', 'wrapper_class' )
			),
			'last_name' => array(
				'nicename'	  => __( 'Last Name', 'slicewp' ),
				'group'		  => 'predefined',
				'key'		  => 'last_name',
				'form_unique' => true,
				'settings'	  => array( 'label', 'description', 'is_required', 'key', 'form', 'placeholder', 'description_placement', 'field_class', 'wrapper_class' )
			),
			'password' => array(
				'nicename'	  	=> __( 'Password', 'slicewp' ),
				'group'		    => 'predefined',
				'key'		    => 'password',
				'form_unique' 	=> true,
				'form_required' => true,
				'settings'	    => array( 'label', 'description', 'key', 'form', 'placeholder', 'description_placement', 'field_class', 'wrapper_class' )
			),
			'password_confirm' => array(
				'nicename'	  	   => __( 'Password Confirmation', 'slicewp' ),
				'group'		  	   => 'predefined',
				'key'		  	   => 'password_confirm',
				'form_unique' 	   => true,
				'default_required' => true,
				'settings'	  	   => array( 'label', 'description', 'key', 'form', 'placeholder', 'description_placement', 'field_class', 'wrapper_class' )
			),
			'payment_email'	=> array(
				'nicename' 	  => __( 'Payment Email', 'slicewp' ),
				'group'		  => 'predefined',
				'key'		  => 'payment_email',
				'form_unique' => true,
				'settings'	  => array( 'label', 'description', 'is_required', 'key', 'form', 'placeholder', 'description_placement', 'field_class', 'wrapper_class', 'admin_label' )
			),
			'website' => array(
				'nicename'	  => __( 'Website', 'slicewp' ),
				'group'		  => 'predefined',
				'key'		  => 'website',
				'form_unique' => true,
				'settings'	  => array( 'label', 'description', 'is_required', 'key', 'form', 'placeholder', 'description_placement', 'field_class', 'wrapper_class', 'admin_label' )
			),

			/**
			 * Standard group fields
			 *
			 */
			'text' => array(
				'nicename'		=> __( 'Single Line Text', 'slicewp' ),
				'group'			=> 'standard',
				'form_unique' 	=> false,
				'settings'	  	=> array( 'label', 'description', 'is_required', 'key', 'form', 'placeholder', 'description_placement', 'field_class', 'wrapper_class', 'admin_label' )
			),
			'textarea' => array(
				'nicename'		=> __( 'Textarea', 'slicewp' ),
				'group'			=> 'standard',
				'form_unique' 	=> false,
				'settings'	  	=> array( 'label', 'description', 'is_required', 'key', 'form', 'placeholder', 'description_placement', 'field_class', 'wrapper_class', 'admin_label' )
			),
			'select' => array(
				'nicename'		=> __( 'Select Drop-down', 'slicewp' ),
				'group'			=> 'standard',
				'form_unique' 	=> false,
				'settings'	  	=> array( 'label', 'description', 'is_required', 'options', 'key', 'form', 'description_placement', 'field_class', 'wrapper_class', 'admin_label' )
			),
			'checkbox' => array(
				'nicename'		=> __( 'Checkboxes', 'slicewp' ),
				'group'			=> 'standard',
				'form_unique' 	=> false,
				'settings'	  	=> array( 'label', 'description', 'is_required', 'options', 'key', 'form', 'description_placement', 'field_class', 'wrapper_class', 'admin_label' )
			),
			'radio' => array(
				'nicename'		=> __( 'Radio Buttons', 'slicewp' ),
				'group'			=> 'standard',
				'form_unique' 	=> false,
				'settings'	  	=> array( 'label', 'description', 'is_required', 'options', 'key', 'form', 'description_placement', 'field_class', 'wrapper_class', 'admin_label' )
			),
			'file' => array(
				'nicename'		=> __( 'File Upload', 'slicewp' ),
				'group'			=> 'standard',
				'form_unique'	=> false,
				'settings'		=> array( 'label', 'description', 'is_required', 'is_multiple', 'allowed_file_extensions', 'maximum_file_size', 'key', 'form', 'description_placement', 'wrapper_class', 'admin_label' )
			),
			'heading' => array(
				'nicename'		=> __( 'Heading', 'slicewp' ),
				'group'			=> 'standard',
				'form_unique' 	=> false,
				'settings'	  	=> array( 'heading_default_value', 'description', 'heading_level', 'form', 'wrapper_class', 'admin_heading_default_value', 'hide_on_admin' )
			),

			/**
			 * Location group fields
			 *
			 */
			'country' => array(
				'nicename'		=> __( 'Country', 'slicewp' ),
				'group'			=> 'location',
				'form_unique' 	=> false,
				'settings'	  	=> array( 'label', 'description', 'is_required', 'default_value', 'key', 'form', 'description_placement', 'field_class', 'wrapper_class', 'admin_label' )
			),
			'state' => array(
				'nicename'		=> __( 'State', 'slicewp' ),
				'group'			=> 'location',
				'form_unique' 	=> false,
				'settings'	  	=> array( 'label', 'description', 'is_required', 'state_default_country', 'key', 'form', 'description_placement', 'field_class', 'wrapper_class', 'admin_label' )
			)

		);

		return $field_types;

	}


	/**
	 * Checks whether the given field type is in the existing form fields
	 *
	 * @param string $field_type
	 *
	 * @return bool
	 *
	 */
	protected function is_field_type_in_fields( $field_type ) {

		foreach ( $this->fields as $field ) {

			if ( $field['type'] == $field_type ) {
				return true;
			}

		}

		return false;

	}


	/**
	 * Outputs the entire HTML for the editor
	 *
	 */
	public function output_editor() {

		?>

			<div id="slicewp-affiliate-fields-editor" class="slicewp-card">

				<?php
					$this->output_editor_header();
					$this->output_editor_fields();
				?>

			</div>

			<!-- Save Settings Button -->
			<input type="submit" class="slicewp-form-submit slicewp-button-primary" value="<?php echo __( 'Save Settings', 'slicewp' ); ?>" />

		<?php

	}


	/**
	 * Outputs the editor header, where the add new field selector exists.
	 *
	 */
	public function output_editor_header() {

		?>

		<div class="slicewp-card-header">

			<!-- The selector for inserting a new field -->
			<div id="slicewp-affiliate-fields-editor-select-new-wrapper" class="slicewp-field-wrapper slicewp-last">

				<select id="slicewp-affiliate-fields-editor-select-new" class="slicewp-select2">
					<option value=""><?php echo __( 'Select a new field to add...', 'slicewp' ); ?></option>

					<?php foreach( $this->get_field_types_groups() as $group_slug => $group_name ): ?>

						<optgroup label="<?php echo sprintf( __( '%s Fields', 'slicewp-' ), esc_attr( $group_name ) ); ?>">

							<?php foreach( $this->get_field_types() as $field_type => $field_type_data ): ?>

								<?php if( $field_type_data['group'] != $group_slug ) continue; ?>

								<option value="<?php echo esc_attr( $field_type ); ?>" <?php echo ( $field_type_data['form_unique'] && $this->is_field_type_in_fields( $field_type ) ? 'disabled' : '' ); ?>><?php echo esc_attr( $field_type_data['nicename'] ); ?></option>

							<?php endforeach; ?>

						</optgroup>

					<?php endforeach; ?>

				</select>

				<a id="slicewp-affiliate-fields-editor-select-add-new" class="slicewp-button-secondary slicewp-disabled" disabled href="#">
					<span><?php echo __( 'Add Field', 'slicewp' ); ?></span>
					<div class="spinner"><!-- --></div>
				</a>

			</div>

		</div>

		<?php

	}


	/**
	 * Outputs the sortable fields and their settings panel
	 *
	 */
	public function output_editor_fields() {

		?>

			<div id="slicewp-affiliate-fields-editor-fields-settings-wrapper" class="slicewp-card-inner">

				<?php

					foreach ( $this->fields as $key => $field ) {
						
						if ( empty( $field['type'] ) ) {
							continue;
						}

						$this->output_editor_single_field( $field, $key );

					}

				?>

			</div>

			<input id="slicewp-affiliate-fields-editor-fields-last-key" type="hidden" value="<?php echo count( $this->fields ); ?>" />

		<?php

	}


	/**
	 * Outputs the settings panel for a single field
	 *
	 * @param array $field
	 * @param mixed $key
	 *
	 */
	protected function output_editor_single_field( $field, $key ) {

		$field_types  = $this->get_field_types();
		$is_new_field = ( empty( $field['settings'] ) ? true : false );

		?>

			<div class="slicewp-affiliate-fields-editor-field-wrapper slicewp-expandable-item" data-type="<?php esc_attr_e( $field['type'] ); ?>">

				<!-- Field Header -->
				<div class="slicewp-affiliate-fields-editor-field-header">
					<div class="slicewp-sort-handle"><!-- --></div>

					<div class="slicewp-affiliate-fields-editor-field-name"><?php esc_html_e( ! empty( $field['settings']['label'] ) ? $field['settings']['label'] : ( ! empty( $field['settings']['default_value'] ) ? $field['settings']['default_value'] : $field_types[$field['type']]['nicename'] ) ); ?><?php echo ( ! empty( $field['settings']['is_required'] ) || ! empty( $field_types[$field['type']]['form_required'] ) || ! empty( $field_types[$field['type']]['default_required'] ) ? '<span>*</span>' : '' ); ?></div>
					<div class="slicewp-affiliate-fields-editor-field-type"><?php esc_html_e( $field_types[$field['type']]['nicename'] ); ?></div>

					<div class="slicewp-affiliate-fields-editor-field-actions">
						<a href="#" class="slicewp-edit-field slicewp-expand-item"><?php _e( 'Edit Field', 'slicewp' ); ?><?php echo slicewp_get_svg( 'outline-chevron-down' ); ?></a>
						<a href="#" class="slicewp-trash <?php echo ( ! empty( $field_types[$field['type']]['form_required'] ) ? 'slicewp-disabled' : '' ); ?>"><?php _e( 'Remove', 'slicewp' ); ?></a>
					</div>
				</div>

				<!-- Field Settings -->
				<div class="slicewp-affiliate-fields-editor-field-body" style="display: none;">

					<!-- Field Settings Panels Nav Tab -->
					<div class="slicewp-affiliate-fields-field-nav-tab">
						<a href="#" class="slicewp-active" data-tab="general"><?php _e( 'General', 'slicewp' ); ?></a>
						<a href="#" data-tab="appearance"><?php _e( 'Appearance', 'slicewp' ); ?></a>

						<?php if( count( array_intersect( array( 'admin_label', 'admin_heading_default_value', 'hide_on_admin' ), $field_types[$field['type']]['settings'] ) ) !== 0 ): ?>
							<a href="#" data-tab="advanced"><?php _e( 'Advanced', 'slicewp' ); ?></a>
						<?php endif; ?>
					</div>

					<!-- General Tab -->
					<div class="slicewp-affiliate-fields-field-tab" data-tab="general">

						<!-- Heading Default Value -->
						<?php if( in_array( 'heading_default_value', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-label-<?php echo absint( $key ); ?>">
										<?php echo __( 'Default Value', 'slicewp' ); ?>
									</label>
								</div>

								<input id="slicewp-field-setting-label-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][default_value]" type="text" value="<?php esc_attr_e( ( ! empty( $field['settings']['default_value'] ) ? $field['settings']['default_value'] : $field_types[$field['type']]['nicename'] ) ); ?>" />

							</div>

						<?php endif; ?>
						<!-- / Heading Default Value -->

						<!-- Field Label -->
						<?php if( in_array( 'label', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-label-<?php echo absint( $key ); ?>">
										<?php echo __( 'Field Label', 'slicewp' ); ?>
									</label>
								</div>

								<input id="slicewp-field-setting-label-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][label]" type="text" value="<?php esc_attr_e( ( isset( $field['settings']['label'] ) ? $field['settings']['label'] : $field_types[$field['type']]['nicename'] ) ); ?>" />

							</div>

						<?php endif; ?>
						<!-- / Field Label -->

						<!-- Field Description -->
						<?php if( in_array( 'description', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-description-<?php echo absint( $key ); ?>">
										<?php echo __( 'Field Description', 'slicewp' ); ?>
									</label>
								</div>

								<textarea id="slicewp-field-setting-description-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][description]"><?php echo esc_textarea( ( ! empty( $field['settings']['description'] ) ? $field['settings']['description'] : '' ) ); ?></textarea>

							</div>

						<?php endif; ?>
						<!-- / Field Description -->

						<!-- Field Required -->
						<?php if( in_array( 'is_required', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-is-required-<?php echo absint( $key ); ?>">
										<?php echo __( 'Required', 'slicewp' ); ?>
									</label>
								</div>

								<div class="slicewp-switch">

									<input id="slicewp-field-setting-is-required-<?php echo absint( $key ); ?>" class="slicewp-toggle slicewp-toggle-round" name="affiliate_fields[<?php echo absint($key); ?>][settings][is_required]" type="checkbox" value="1" <?php echo ( ! empty( $field['settings']['is_required'] ) ? 'checked' : '' ); ?> />
									<label for="slicewp-field-setting-is-required-<?php echo absint( $key ); ?>"></label>

								</div>

								<label for="slicewp-field-setting-is-required-<?php echo absint( $key ); ?>"><?php _e( 'Enable to make this field required', 'slicewp' ); ?></label>

							</div>

						<?php endif; ?>
						<!-- / Field Required -->

						<!-- Field Options -->
						<?php if ( in_array( 'options', $field_types[$field['type']]['settings'] ) ): ?>

							<?php

								// Set defaults in case the field doesn't have any options set
								if( ! isset( $field['settings']['options'] ) || ! is_array( $field['settings']['options'] ) ) {
									$field['settings']['options'] = array(
										array(
											'label'	  => __( 'Option 1', 'slicewp' ),
											'value'	  => 'option_1'
										),
										array(
											'label'	  => __( 'Option 2', 'slicewp' ),
											'value'	  => 'option_2'
										),
										array(
											'label'	  => __( 'Option 3', 'slicewp' ),
											'value'	  => 'option_3'
										)
									);

								}

							?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">
								
								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-options-<?php echo absint( $key ); ?>">
										<?php echo __( 'Options', 'slicewp' ); ?>
									</label>
								</div>

								<div class="slicewp-field-settings-options">

									<?php foreach ( $field['settings']['options'] as $option_key => $option ): ?>

										<div class="slicewp-field-settings-option">
											<div class="slicewp-sort-handle-option"><!-- --></div>
											<input type="<?php echo ( $field['type'] == 'checkbox' ? 'checkbox' : 'radio' ); ?>" name="affiliate_fields[<?php echo absint( $key ); ?>][settings][options][<?php echo absint( $option_key ); ?>][default]" value="1" <?php echo ( ! empty( $field['settings']['options'][$option_key]['default'] ) ? 'checked' : '' ); ?> />
											<input type="text" name="affiliate_fields[<?php echo absint( $key ); ?>][settings][options][<?php echo absint( $option_key ); ?>][label]" placeholder="<?php _e( 'Option label...', 'slicewp' ); ?>" value="<?php ( ! empty( $field['settings']['options'][$option_key]['label'] ) ? esc_attr_e( $field['settings']['options'][$option_key]['label'] ) : '' ); ?>" class="slicewp-field-settings-option-label" />
											<input type="text" name="affiliate_fields[<?php echo absint( $key ); ?>][settings][options][<?php echo absint( $option_key ); ?>][value]" placeholder="<?php _e( 'Option value...', 'slicewp' ); ?>" value="<?php ( ! empty( $field['settings']['options'][$option_key]['value'] ) ? esc_attr_e( $field['settings']['options'][$option_key]['value'] ) : '' ); ?>" class="slicewp-field-settings-option-value" />
											<a href="#" class="slicewp-field-settings-option-add"><svg xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke="currentColor"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M12 9v3m0 0v3m0-3h3m-3 0H9m12 0a9 9 0 11-18 0 9 9 0 0118 0z" /></svg></a>
											<a href="#" class="slicewp-field-settings-option-remove"><svg xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke="currentColor"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M15 12H9m12 0a9 9 0 11-18 0 9 9 0 0118 0z" /></svg></a>
										</div>

									<?php endforeach; ?>

								</div>

								<div class="slicewp-field-notice slicewp-field-notice-error slicewp-field-notice-key-char-check"><p><?php echo sprintf( __( "%sImportant note%s: Keys can only contain lower case alphanumeric characters, underscores and hyphens.", 'slicewp' ), '<strong>', '</strong>' ); ?></p></div>

								<input type="hidden" value="<?php echo ( ! empty( $option_key ) ? ( $option_key + 1 ) : count( $field['settings']['options'] ) ); ?>" />

								<!-- Option Placeholder -->
								<div class="slicewp-field-settings-option slicewp-field-settings-option-placeholder">
									<div class="slicewp-sort-handle-option"><!-- --></div>
									<input type="<?php echo ( $field['type'] == 'checkbox' ? 'checkbox' : 'radio' ); ?>" data-name="affiliate_fields[<?php echo absint( $key ); ?>][settings][options][#option_key][default]" value="1" />
									<input type="text" placeholder="<?php _e( 'Option label...', 'slicewp' ); ?>" data-name="affiliate_fields[<?php echo absint( $key ); ?>][settings][options][#option_key][label]" />
									<input type="text" placeholder="<?php _e( 'Option value...', 'slicewp' ); ?>" data-name="affiliate_fields[<?php echo absint( $key ); ?>][settings][options][#option_key][value]" />
									<a href="#" class="slicewp-field-settings-option-add"><svg xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke="currentColor"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M12 9v3m0 0v3m0-3h3m-3 0H9m12 0a9 9 0 11-18 0 9 9 0 0118 0z" /></svg></a>
									<a href="#" class="slicewp-field-settings-option-remove"><svg xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke="currentColor"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M15 12H9m12 0a9 9 0 11-18 0 9 9 0 0118 0z" /></svg></a>
								</div>
								<!-- / Option Placeholder -->

							</div>

						<?php endif; ?>
						<!-- / Field Options -->

						<!-- Heading Level -->
						<?php if ( in_array( 'heading_level', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-heading-level-<?php echo absint( $key ); ?>">
										<?php echo __( 'Heading Level', 'slicewp' ); ?>
									</label>
								</div>

								<select id="slicewp-field-setting-heading-level-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][heading_level]" class="slicewp-select2">
									<option value="h2" <?php echo ( ! empty( $field['settings']['heading_level'] ) && $field['settings']['heading_level'] == 'h2' ? 'selected' : '' ); ?>><?php esc_attr_e( 'H2' ); ?></option>
									<option value="h3" <?php echo ( ! empty( $field['settings']['heading_level'] ) && $field['settings']['heading_level'] == 'h3' ? 'selected' : '' ); ?>><?php esc_attr_e( 'H3' ); ?></option>
									<option value="h4" <?php echo ( ! empty( $field['settings']['heading_level'] ) && $field['settings']['heading_level'] == 'h4' ? 'selected' : '' ); ?>><?php esc_attr_e( 'H4' ); ?></option>
									<option value="h5" <?php echo ( ! empty( $field['settings']['heading_level'] ) && $field['settings']['heading_level'] == 'h5' ? 'selected' : '' ); ?>><?php esc_attr_e( 'H5' ); ?></option>
									<option value="h6" <?php echo ( ! empty( $field['settings']['heading_level'] ) && $field['settings']['heading_level'] == 'h6' ? 'selected' : '' ); ?>><?php esc_attr_e( 'H6' ); ?></option>
								</select>

							</div>

						<?php endif; ?>
						<!-- / Heading Level -->

						<!-- Default Country Value -->
						<?php if ( $field['type'] == 'country' && in_array( 'default_value', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-default-value-<?php echo absint( $key ); ?>">
										<?php echo __( 'Default Country', 'slicewp' ); ?>
									</label>
								</div>

								<select id="slicewp-field-setting-default-value-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][default_value]" class="slicewp-select2">
									<option value=""><?php _e( 'None', 'slicewp' ); ?></option>
									<?php foreach( slicewp_get_countries() as $code => $name ): ?>
										<option value="<?php esc_attr_e( $code ); ?>" <?php echo ( ! empty( $field['settings']['default_value'] ) && $field['settings']['default_value'] == $code ? "selected" : '' ); ?>><?php esc_html_e( $name ); ?></option>
									<?php endforeach; ?>
								</select>

							</div>

						<?php endif; ?>
						<!-- / Default Country Value -->

						<!-- Default State Country -->
						<?php if ( in_array( 'state_default_country', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-state-default-country-<?php echo absint( $key ); ?>">
										<?php echo __( 'Default Country', 'slicewp' ); ?>
									</label>
								</div>

								<select id="slicewp-field-setting-state-default-country-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][state_default_country]" class="slicewp-select2">
									<option value=""><?php _e( 'None', 'slicewp' ); ?></option>
									<?php foreach( slicewp_get_countries() as $code => $name ): ?>
										<option value="<?php esc_attr_e( $code ); ?>" <?php echo ( ! empty( $field['settings']['state_default_country'] ) && $field['settings']['state_default_country'] == $code ? "selected" : '' ); ?>><?php esc_html_e( $name ); ?></option>
									<?php endforeach; ?>
								</select>

							</div>

						<?php endif; ?>
						<!-- / Default State Country -->

						<!-- Field is Multiple -->
						<?php if ( in_array( 'is_multiple', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-is-multiple-<?php echo absint( $key ); ?>">
										<?php echo __( 'Multiple Files', 'slicewp' ); ?>
									</label>
								</div>

								<div class="slicewp-switch">

									<input id="slicewp-field-setting-is-multiple-<?php echo absint( $key ); ?>" class="slicewp-toggle slicewp-toggle-round" name="affiliate_fields[<?php echo absint($key); ?>][settings][is_multiple]" type="checkbox" value="1" <?php echo ( ! empty( $field['settings']['is_multiple'] ) ? 'checked' : '' ); ?> />
									<label for="slicewp-field-setting-is-multiple-<?php echo absint( $key ); ?>"></label>

								</div>

								<label for="slicewp-field-setting-is-multiple-<?php echo absint( $key ); ?>"><?php _e( 'Enable multi-file upload', 'slicewp' ); ?></label>

							</div>

						<?php endif; ?>
						<!-- Field is Multiple -->

						<!-- Field Allowed File Extensions -->
						<?php if ( in_array( 'maximum_file_size', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-allowed-file-extensions-<?php echo absint( $key ); ?>">
										<?php echo __( 'Allowed File Extensions', 'slicewp' ); ?>
										<?php echo slicewp_output_tooltip( __( 'Enter the allowed file extensions for file uploads, separated with commas (i.e. jpg, gif, png, pdf). This will limit the type of files a user may upload.', 'slicewp' ) ); ?>
									</label>
								</div>

								<input id="slicewp-field-setting-allowed-file-extensions-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][allowed_file_extensions]" type="text" placeholder="<?php echo __( 'i.e jpg, gif, png, pdf', 'slicewp' ); ?>" value="<?php esc_attr_e( ( isset( $field['settings']['allowed_file_extensions'] ) ? $field['settings']['allowed_file_extensions'] : '' ) ); ?>" />

							</div>

						<?php endif; ?>
						<!-- / Field Allowed File Extensions -->

						<!-- Field Maximum File Size -->
						<?php if ( in_array( 'maximum_file_size', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-maximum-file-size-<?php echo absint( $key ); ?>">
										<?php echo __( 'Maximum File Size', 'slicewp' ); ?>
									</label>
								</div>

								<input id="slicewp-field-setting-maximum-file-size-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][maximum_file_size]" type="number" placeholder="<?php echo absint( wp_max_upload_size() / 1024 / 1024 ); ?>" max="<?php echo absint( wp_max_upload_size() / 1024 / 1024 ); ?>" value="<?php esc_attr_e( ( isset( $field['settings']['maximum_file_size'] ) ? $field['settings']['maximum_file_size'] : '' ) ); ?>" />
								<p class="description"><?php echo sprintf( __( 'Maximum allowed on this server: %s', 'slicewp' ), size_format( wp_max_upload_size() ) ); ?></p>

							</div>

						<?php endif; ?>
						<!-- / Field Maximum File Size -->

						<!-- Field Key -->
						<?php if ( in_array( 'key', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline slicewp-field-wrapper-affiliate-field-key">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-key-<?php echo absint( $key ); ?>">
										<?php echo __( 'Field Key', 'slicewp' ); ?>
									</label>
								</div>

								<?php if ( empty( $field_types[$field['type']]['key'] ) ): ?>

									<div class="slicewp-field-setting-key-wrapper">
										<input id="slicewp-field-setting-key-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][key]" type="text" value="<?php esc_attr_e( ( isset( $field['settings']['key'] ) ? $field['settings']['key'] : $field['type'] . '_' . $this->generate_random_key( 5 ) ) ); ?>" <?php echo ( isset( $field['settings']['key'] ) ? 'readonly' : ''); ?> maxlength="50" />
										
										<?php if ( isset( $field['settings']['key'] ) ): ?>
											<a href="#"><span class="dashicons dashicons-lock"></span><?php _e( 'Unlock', 'slicewp' ); ?></a>
										<?php endif; ?>
									</div>

									<div class="slicewp-field-notice slicewp-field-notice-error slicewp-field-notice-key-char-check"><p><?php echo sprintf( __( "%sImportant note%s: Keys can only contain lower case alphanumeric characters, underscores and hyphens.", 'slicewp' ), '<strong>', '</strong>' ); ?></p></div>
									<div class="slicewp-field-notice slicewp-field-notice-error slicewp-field-notice-key-empty"><p><?php echo sprintf( __( "%sImportant note%s: Field keys cannot be empty. Please add a value for the field key.", 'slicewp' ), '<strong>', '</strong>' ); ?></p></div>
									<div class="slicewp-field-notice slicewp-field-notice-warning"><p><?php echo sprintf( __( "%sImportant note%s: If you already have affiliates, it's recommended to %snot%s modify the field's key. You will lose the affiliate information saved for this field.", 'slicewp' ), '<strong>', '</strong>', '<strong>', '</strong>' ); ?></p></div>

								<?php else: ?>

									<input id="slicewp-field-setting-key-<?php echo absint( $key ); ?>" type="text" value="<?php esc_attr_e( $field_types[$field['type']]['key'] ); ?>" disabled />

								<?php endif; ?>

							</div>

						<?php endif; ?>
						<!-- / Field Key -->

						<!-- Field Form Display -->
						<?php if ( in_array( 'form', $field_types[$field['type']]['settings'] ) ): ?>
							
							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline slicewp-field-wrapper-affiliate-field-form slicewp-last">

								<div class="slicewp-field-label-wrapper">
									<label>
										<?php echo __( 'Show in form', 'slicewp' ); ?>
									</label>
								</div>

								<?php if ( ! in_array( $field['type'], array( 'user_login', 'user_email', 'first_name', 'last_name', 'password', 'password_confirm' ) ) ): ?>
								
									<div style="margin-bottom: 10px;">
										<div class="slicewp-switch">

											<input id="slicewp-field-setting-form-affiliate-registration-<?php echo absint( $key ); ?>" class="slicewp-toggle slicewp-toggle-round" name="affiliate_fields[<?php echo absint($key); ?>][settings][form][]" type="checkbox" value="affiliate_registration" <?php echo ( $is_new_field || ( ! empty( $field['settings']['form'] ) && in_array( 'affiliate_registration', $field['settings']['form'] ) ) ? 'checked' : '' ); ?> />
											<label for="slicewp-field-setting-form-affiliate-registration-<?php echo absint( $key ); ?>"></label>

										</div>

										<label for="slicewp-field-setting-form-affiliate-registration-<?php echo absint( $key ); ?>"><?php _e( 'Affiliate Registration', 'slicewp' ); ?></label>
									</div>

									<div>
										<div class="slicewp-switch">

											<input id="slicewp-field-setting-form-affiliate-account-<?php echo absint( $key ); ?>" class="slicewp-toggle slicewp-toggle-round" name="affiliate_fields[<?php echo absint($key); ?>][settings][form][]" type="checkbox" value="affiliate_account" <?php echo ( $is_new_field || ( ! empty( $field['settings']['form'] ) && in_array( 'affiliate_account', $field['settings']['form'] ) ) ? 'checked' : '' ); ?> />
											<label for="slicewp-field-setting-form-affiliate-account-<?php echo absint( $key ); ?>"></label>

										</div>

										<label for="slicewp-field-setting-form-affiliate-account-<?php echo absint( $key ); ?>"><?php _e( 'Affiliate Account Settings', 'slicewp' ); ?></label>
									</div>

								<?php else: ?>

									<div class="slicewp-field-notice slicewp-field-notice-warning"><p><?php _e( 'This field will appear only in the affiliate registration form and only for logged-out users.', 'slicewp' ); ?></p></div>

								<?php endif; ?>

							</div>

						<?php endif; ?>
						<!-- / Field Form Display -->

					</div>
					<!-- / General Tab -->
					
					<!-- Appearance Tab -->
					<div class="slicewp-affiliate-fields-field-tab" data-tab="appearance" style="display: none;">

						<!-- Field Placeholder -->
						<?php if ( in_array( 'placeholder', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-placeholder-<?php echo absint( $key ); ?>">
										<?php echo __( 'Placeholder', 'slicewp' ); ?>
									</label>
								</div>

								<input id="slicewp-field-setting-placeholder-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][placeholder]" type="text" value="<?php esc_attr_e( ( ! empty( $field['settings']['placeholder'] ) ? $field['settings']['placeholder'] : '' ) ); ?>" />

							</div>

						<?php endif; ?>
						<!-- / Field Placeholder -->

						<!-- Field Description Placement -->
						<?php if ( in_array( 'description_placement', $field_types[$field['type']]['settings'] ) ): ?>

							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-description-placement-<?php echo absint( $key ); ?>">
										<?php echo __( 'Description Placement', 'slicewp' ); ?>
									</label>
								</div>

								<select id="slicewp-field-setting-description-placement-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][description_placement]" class="slicewp-select2">
									<option value="after" <?php echo ( ! empty( $field['settings']['description_placement'] ) && $field['settings']['description_placement'] == 'after' ? 'selected' : '' ); ?>><?php echo __( 'After input', 'slicewp' ); ?></option>
									<option value="before" <?php echo ( ! empty( $field['settings']['description_placement'] ) && $field['settings']['description_placement'] == 'before' ? 'selected' : '' ); ?>><?php echo __( 'Before input', 'slicewp' ); ?></option>
								</select>

							</div>

						<?php endif; ?>
						<!-- / Field Description Placement -->

						<!-- Field Input Classes -->
						<?php if ( in_array( 'field_class', $field_types[$field['type']]['settings'] ) ): ?>
							
							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-input-class-<?php echo absint( $key ); ?>">
										<?php echo __( 'Input CSS Classes', 'slicewp' ); ?>
									</label>
								</div>

								<input id="slicewp-field-setting-input-class-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][field_class]" type="text" value="<?php esc_attr_e( ( ! empty( $field['settings']['field_class'] ) ? $field['settings']['field_class'] : '' ) ); ?>" />

							</div>

						<?php endif; ?>
						<!-- / Field Input Classes -->

						<!-- Field Wrapper Classes -->
						<?php if ( in_array( 'wrapper_class', $field_types[$field['type']]['settings'] ) ): ?>
							
							<div class="slicewp-field-wrapper slicewp-field-wrapper-inline slicewp-last">

								<div class="slicewp-field-label-wrapper">
									<label for="slicewp-field-setting-wrapper-class-<?php echo absint( $key ); ?>">
										<?php echo __( 'Wrapper CSS Classes', 'slicewp' ); ?>
									</label>
								</div>

								<input id="slicewp-field-setting-wrapper-class-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][wrapper_class]" type="text" value="<?php esc_attr_e( ( ! empty( $field['settings']['wrapper_class'] ) ? $field['settings']['wrapper_class'] : '' ) ); ?>" />

							</div>

						<?php endif; ?>
						<!-- / Field Wrapper Classes -->

					</div>
					<!-- / Appearance Tab -->

					<!-- Advanced Tab -->
					<?php if ( count( array_intersect( array( 'admin_label', 'admin_heading_default_value', 'hide_on_admin' ), $field_types[$field['type']]['settings'] ) ) !== 0 ): ?>

						<div class="slicewp-affiliate-fields-field-tab" data-tab="advanced" style="display: none;">

							<!-- Field Admin Label -->
							<?php if ( in_array( 'admin_label', $field_types[$field['type']]['settings'] ) ): ?>

								<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

									<div class="slicewp-field-label-wrapper">
										<label for="slicewp-field-setting-admin-label-<?php echo absint( $key ); ?>">
											<?php echo __( 'Field Admin Label', 'slicewp' ); ?>
										</label>
									</div>

									<input id="slicewp-field-setting-admin-label-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][admin_label]" type="text" value="<?php esc_attr_e( ( isset( $field['settings']['admin_label'] ) ? $field['settings']['admin_label'] : '' ) ); ?>" />

								</div>

							<?php endif; ?>
							<!-- / Field Admin Label -->

							<!-- Admin Default Value -->
							<?php if ( in_array( 'admin_heading_default_value', $field_types[$field['type']]['settings'] ) ): ?>

								<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

									<div class="slicewp-field-label-wrapper">
										<label for="slicewp-field-setting-admin-default-value-<?php echo absint( $key ); ?>">
											<?php echo __( 'Admin Default Value', 'slicewp' ); ?>
										</label>
									</div>

									<input id="slicewp-field-setting-admin-default-value-<?php echo absint( $key ); ?>" name="affiliate_fields[<?php echo absint($key); ?>][settings][admin_default_value]" type="text" value="<?php esc_attr_e( ( isset( $field['settings']['admin_default_value'] ) ? $field['settings']['admin_default_value'] : '' ) ); ?>" />

								</div>

							<?php endif; ?>
							<!-- / Admin Default Value -->

							<!-- Field Hide on Admin -->
							<?php if ( in_array( 'hide_on_admin', $field_types[$field['type']]['settings'] ) ): ?>

								<div class="slicewp-field-wrapper slicewp-field-wrapper-inline">

									<div class="slicewp-field-label-wrapper">
										<label for="slicewp-field-setting-hide-on-admin-<?php echo absint( $key ); ?>">
											<?php echo __( 'Hide on Admin Pages', 'slicewp' ); ?>
										</label>
									</div>

									<div class="slicewp-switch">

										<input id="slicewp-field-setting-hide-on-admin-<?php echo absint( $key ); ?>" class="slicewp-toggle slicewp-toggle-round" name="affiliate_fields[<?php echo absint($key); ?>][settings][hide_on_admin]" type="checkbox" value="1" <?php echo ( ! empty( $field['settings']['hide_on_admin'] ) ? 'checked' : '' ); ?> />
										<label for="slicewp-field-setting-hide-on-admin-<?php echo absint( $key ); ?>"></label>

									</div>

									<label for="slicewp-field-setting-hide-on-admin-<?php echo absint( $key ); ?>"><?php _e( 'Enable to hide this field on admin dashboard pages.', 'slicewp' ); ?></label>

								</div>

							<?php endif; ?>
							<!-- / Field Hide on Admin -->

						</div>

					<?php endif; ?>
					<!-- / Advanced Tab -->

				</div>

				<!-- Hidden inputs -->
				<input type="hidden" name="affiliate_fields[<?php echo absint($key); ?>][type]" value="<?php esc_attr_e( ( ! empty( $field['type'] ) ? $field['type'] : '' ) ); ?>" />

			</div>

		<?php

	}


	/**
	 * AJAX callback to add a new field to the editor, based on what the user selected
	 *
	 */
	public function add_new_field() {

		if( empty( $_POST['slicewp_token'] ) || ! wp_verify_nonce( $_POST['slicewp_token'], 'slicewp_save_settings' ) )
			wp_die(0);

		if( empty( $_POST['field_type'] ) || empty( $_POST['fields_last_key'] ) )
			wp_die(0);

		if( ! in_array( $_POST['field_type'], array_keys( $this->get_field_types() ) ) )
			wp_die(0);

		$this->output_editor_single_field( array( 'type' => sanitize_text_field( $_POST['field_type'] ) ), absint( $_POST['fields_last_key'] ) );
		wp_die();

	}


	/**
	 * Localizez the add-ons's main JS script and adds the field types data
	 *
	 */
	public function localize_field_types() {

		wp_localize_script( 'slicewp-custom-affiliate-fields-script', 'slicewp_affiliate_fields_editor_field_types', $this->get_field_types() );

	}


	/**
	 * Validates and handles the saving of the main plugin settings
	 *
	 */
	public function admin_action_save_settings() {

		// Verify for nonce
		if( empty( $_POST['slicewp_token'] ) || ! wp_verify_nonce( $_POST['slicewp_token'], 'slicewp_save_settings' ) )
			return;

		if( empty( $_POST['affiliate_fields'] ) )
			return;

		// Save the settings
		$_POST = stripslashes_deep( $_POST );

		// Sanitize fields
		$affiliate_fields = _slicewp_array_wp_kses_post( $_POST['affiliate_fields'] );

		// Reset fields array
		$affiliate_fields = array_values( $affiliate_fields );

		// Reset and clean field options
		foreach( $affiliate_fields as $key => $field ) {

			if( empty( $field['settings']['options'] ) || ! is_array( $field['settings']['options'] ) )
				continue;

			// Remove empty options
			foreach( $field['settings']['options'] as $option_key => $option ) {

				if( empty( $option['default'] ) && empty( $option['label'] ) && empty( $option['value'] ) )
					unset( $field['settings']['options'][$option_key] );

			}

			// Reset options array
			$field['settings']['options'] = array_values( $field['settings']['options'] );

			// Add the field settings back into main array
			$affiliate_fields[$key] = $field;

		}

		slicewp_update_option( 'affiliate_fields', $affiliate_fields );

	}


	/**
	 * Returns a random generated key
	 *
	 * @param int $length
	 *
	 * @return string
	 *
	 */
	protected function generate_random_key( $length = 20 ) {

	    $chars         = '0123456789';
	    $chars_length  = strlen( $chars );
	    $random_string = '';

	    for( $i = 0; $i < $length; $i++ ) {

	        $random_string .= $chars[ rand( 0, $chars_length - 1 ) ];

	    }

	    return $random_string;

	}

}


/**
 * Register the Affiliate Fields Editor as a service
 *
 * @param array
 *
 * @return array
 *
 */
function slicewp_register_service_affiliate_fields_editor( $services ) {

	$services['affiliate_fields_editor'] = 'SliceWP_Affiliate_Fields_Editor';

	return $services;

}
add_filter( 'slicewp_register_service', 'slicewp_register_service_affiliate_fields_editor' );