<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Deregister default affiliate fields if custom affiliate fields are set.
 *
 * If we don't have any affiliate fields we need a back-up, so we'll not deregister 
 * the custom affiliate fields from core.
 *
 */
function slicewp_maybe_deregister_default_affiliate_fields() {

	if ( function_exists( 'slicewp' ) ) {

		// Get custom affiliate fields
		$custom_affiliate_fields = slicewp_get_option( 'affiliate_fields', array() );

		if ( ! empty( $custom_affiliate_fields ) ) {

			remove_filter( 'slicewp_register_affiliate_fields', 'slicewp_register_default_affiliate_fields' );

		}

	}

}
add_action( 'plugins_loaded', 'slicewp_maybe_deregister_default_affiliate_fields' );


/**
 * Register the custom affiliate fields based on what the user configured in the settings page.
 *
 * @param array $fields
 *
 * @return array
 *
 */
function slicewp_register_custom_affiliate_fields( $fields ) {

	$custom_affiliate_fields = slicewp_get_option( 'affiliate_fields', array() );

	if ( empty( $custom_affiliate_fields ) || ! is_array( $custom_affiliate_fields ) ) {
		return $fields;
	}

	// Get field types.
	$field_types = slicewp()->services['affiliate_fields_editor']->get_field_types();

	// Go through each field and prepare it for the fields API.
	foreach ( $custom_affiliate_fields as $field ) {

		if ( empty( $field['type'] ) ) {
			continue;
		}

		// Exclude headings on admin if they're marked as hidden.
		if ( $field['type'] == 'heading' && is_admin() && ! empty( $field['settings']['hide_on_admin'] ) ) {
			continue;
		}

		// Set the field's name.
		$name = ( ! empty( $field['settings']['key'] ) ? $field['settings']['key'] : ( ! empty( $field_types[$field['type']]['key'] ) ? $field_types[$field['type']]['key'] : $field['type'] ) );

		// Set the field's type.
		switch ( $field['type'] ) {

			case 'user_login':
			case 'first_name':
			case 'last_name':
				$type = 'text';
				break;

			case 'user_email':
			case 'payment_email':
				$type = 'email';
				break;

			case 'password':
			case 'password_confirm':
				$type = 'password';
				break;

			case 'website':
				$type = 'url';
				break;

			default:
				$type = $field['type'];
				break;

		}

		// Set label.
		$label = ( ! empty( $field['settings']['label'] ) ? $field['settings']['label'] : null );

		if ( is_admin() ) {
			$label = ( ! empty( $field['settings']['admin_label'] ) ? $field['settings']['admin_label'] : $label );
		}

		// Set default value.
		$default_value = ( ! empty( $field['settings']['default_value'] ) ? $field['settings']['default_value'] : null );

		if ( is_admin() && $field['type'] == 'heading' ) {
			$default_value = ( ! empty( $field['settings']['admin_default_value'] ) ? $field['settings']['admin_default_value'] : $default_value );
		}

		// Set is_required value.
		$is_required = ( in_array( $field['type'], array( 'user_login', 'user_email', 'password', 'password_confirm' ) ) ? 1 : ( ! empty( $field['settings']['is_required'] ) ? 1 : 0 ) );

		if ( is_admin() ) {
			$is_required = ( ! empty( $field['settings']['hide_on_admin'] ) ? 0 : $is_required );
		}

		// Set options.
		$options = array();

		if ( ! empty( $field['settings']['options'] ) && is_array( $field['settings']['options'] ) ) {

			$default_value = array();

			foreach ( $field['settings']['options'] as $option ) {

				$options[$option['value']] = $option['label'];

				if ( ! empty( $option['default'] ) ) {
					$default_value[] = $option['value'];
				}

			}

			if ( $field['type'] != 'checkbox' ) {
				$default_value = implode( '', $default_value );
			}

		}

		// Set output conditionals.
		if ( in_array( $field['type'], array( 'user_login', 'user_email', 'first_name', 'last_name', 'password', 'password_confirm' ) ) ) {

			$output_conditionals = array( 'form' => array( 'affiliate_registration' ) );

		} else {

			$output_conditionals = ( ! empty( $field['settings']['form'] ) ? array( 'form' => $field['settings']['form'] ) : array() );

		}


		// Prepare field data.
		$field_data = array(
			'name' 					=> $name,
			'id'					=> ( strpos( $name, 'slicewp-' ) === 0 ? $name : 'slicewp-' . $name ),
			'type' 					=> $type,
			'label' 				=> $label,
			'description' 			=> ( ! empty( $field['settings']['description'] ) ? $field['settings']['description'] : null ),
			'description_placement' => ( ! empty( $field['settings']['description_placement'] ) ? $field['settings']['description_placement'] : '' ),
			'placeholder' 			=> ( ! empty( $field['settings']['placeholder'] ) ? $field['settings']['placeholder'] : '' ),
			'field_class' 			=> ( ! empty( $field['settings']['field_class'] ) ? array_map( 'trim', explode( ' ', $field['settings']['field_class'] ) ) : array() ),
			'wrapper_class' 		=> ( ! empty( $field['settings']['wrapper_class'] ) ? array_map( 'trim', explode( ' ', $field['settings']['wrapper_class'] ) ) : array() ),
			'is_required'			=> $is_required,
			'options'				=> $options,
			'default_value'			=> ( ! is_null( $default_value ) ? $default_value : null ),
			'output_conditionals'	=> $output_conditionals
		);

		// "Heading" field data.
		if ( 'heading' == $field['type'] ) {

			if ( ! empty( $field['settings']['heading_level'] ) ) {
				$field_data['level'] = str_replace( 'h', '', $field['settings']['heading_level'] );
			}

		}

		// "File" field data.
		if ( 'file' == $field['type'] ) {

			if ( ! empty( $field['settings']['is_multiple'] ) ) {
				$field_data['is_multiple'] = true;
			}

			if ( ! empty( $field['settings']['allowed_file_extensions'] ) ) {
				$field_data['allowed_mime_types'] = slicewp_extensions_to_mime_types( array_map( 'trim', explode( ',', $field['settings']['allowed_file_extensions'] ) ) );
			}

			if ( ! empty( $field['settings']['maximum_file_size'] ) ) {
				$field_data['maximum_file_size'] = absint( $field['settings']['maximum_file_size'] ) * 1024 * 1024;
			}

		}

		// Append field
		$fields[] = $field_data;

	}

	return $fields;

}
add_filter( 'slicewp_register_affiliate_fields', 'slicewp_register_custom_affiliate_fields' );


/**
 * Registers the custom affiliate fields as columns in the affiliates CSV export file.
 * 
 * @param array $cols
 * 
 * @return array
 * 
 */
function slicewp_export_csv_cols_affiliates_custom_affiliate_fields( $cols ) {

	foreach ( slicewp_get_affiliate_fields() as $field ) {

		$field = slicewp_create_form_field_object( $field );

		if ( empty( $field->get( 'name' ) ) ) {
			continue;
		}

		if ( in_array( $field->get( 'name' ), array( 'user_login', 'user_email', 'first_name', 'last_name', 'password', 'password_confirm', 'payment_email', 'website' ) ) ) {
			continue;
		}

		$cols['meta_' . $field->get( 'name' )] = $field->get( 'label' );

	}

	return $cols;

}
add_filter( 'slicewp_export_csv_cols_affiliates', 'slicewp_export_csv_cols_affiliates_custom_affiliate_fields', 50 );


/**
 * Appends the custom affiliate fields data to the export affiliate data.
 * 
 * @param array 			$data
 * @param SliceWP_Affiliate $affiliate
 * 
 * @return array
 *
 */
function slicewp_export_get_data_item_affiliates_custom_affiliate_fields( $data, $affiliate ) {

	foreach ( slicewp_get_affiliate_fields() as $field ) {

		$field = slicewp_create_form_field_object( $field );

		if ( empty( $field->get( 'name' ) ) ) {
			continue;
		}

		if ( in_array( $field->get( 'name' ), array( 'user_login', 'user_email', 'first_name', 'last_name', 'password', 'password_confirm', 'payment_email', 'website' ) ) ) {
			continue;
		}

		$meta_value   = slicewp_get_affiliate_meta( $affiliate->get( 'id' ), $field->get( 'name' ), true );
		$export_value = '';

		// Fields where the export value is the same as the meta value.
		if ( in_array( $field->get( 'type' ), array( 'text', 'textarea' ) ) ) {
			$export_value = $meta_value;
		}

		// Select and radio fields.
		if ( in_array( $field->get( 'type' ), array( 'select', 'radio' ) ) ) {
			$export_value = ( ! empty( $field->get( 'options' )[$meta_value] ) ? $field->get( 'options' )[$meta_value] : $meta_value );
		}

		// Checkbox field.
		if ( $field->get( 'type' ) == 'checkbox' ) {

			if ( is_array( $meta_value ) && ! empty( $meta_value ) ) {

				$_export_value = array();

				foreach ( $meta_value as $value ) {

					$_export_value[] = ( ! empty( $field->get( 'options' )[$value] ) ? $field->get( 'options' )[$value] : $value );
					
				}

				$export_value = implode( ', ', $_export_value );

			}

		}

		// Country field.
		if ( $field->get( 'type' ) == 'country' ) {

			$countries = slicewp_get_countries();

			$export_value = ( ! empty( $meta_value ) && ! empty( $countries[$meta_value] ) ? $countries[$meta_value] : '' );

		}

		// State field.
		if ( $field->get( 'type' ) == 'state' ) {

			// Default to the meta value.
			// This should be replaced with the name of the state if we find a state.
			$export_value = $meta_value;

			// Get affiliate fields
			$_affiliate_fields = slicewp_get_affiliate_fields();

			// Set the internal pointer of the array to the current key
			while ( key( $_affiliate_fields ) !== array_search( $field->get( 'name' ), array_column( $_affiliate_fields, 'name' ) ) ) next( $_affiliate_fields );

			// Get previous field.
			$previous_field = slicewp_create_form_field_object( prev( $_affiliate_fields ) );

			// We need to make this validation only if the previous field is a "country" field
			if ( ! is_null( $previous_field ) && $previous_field->get( 'type' ) == 'country' ) {

				$country 		= slicewp_get_affiliate_meta( $affiliate->get( 'id' ), $previous_field->get( 'name' ), true );
				$country_states = slicewp_get_country_states( $country );

				if ( ! empty( $country ) && ! is_null( $country_states ) ) {
					$export_value = $country_states[$meta_value];
				}

			}

		}

		// File field.
		if ( $field->get( 'type' ) == 'file' ) {

			if ( is_array( $meta_value ) && ! empty( $meta_value ) ) {
			
				foreach ( $meta_value as $key => $attachment_id ) {

					$export_value .= wp_get_attachment_url( absint( $attachment_id ) ) . ( $key < count( $meta_value ) - 1 ? "\r\n" : '' );

				}

			}

		}

		$data['meta_' . $field->get( 'name' )] = $export_value;

	}

	return $data;

}
add_filter( 'slicewp_export_get_data_item_affiliates', 'slicewp_export_get_data_item_affiliates_custom_affiliate_fields', 50, 2 );