<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * "Custom Affiliate Fields" add-on class.
 * 
 */
class SliceWP_Add_On_Custom_Affiliate_Fields extends SliceWP_Add_On {

	/**
	 * Construct.
	 * 
	 */
	public function __construct() {

		// Slug.
		$this->slug = 'custom_affiliate_fields';

		// Name.
		$this->name = 'Custom Affiliate Fields';

		// Description.
		$this->description = 'Add custom fields to your affiliate registration and affiliate account settings forms.';

		// Icon URL.
		$this->icon_url = $this->get_dir_url() . 'assets/img/add-on-icon-custom-affiliate-fields.png';

		// Documentation URL.
		$this->documentation_url = 'https://slicewp.com/products/custom-affiliate-fields/';

		// Settings URL.
		$this->settings_url = add_query_arg( array( 'page' => 'slicewp-settings', 'tab' => 'affiliate_fields' ), admin_url( 'admin.php' ) );

		// Author information.
		$this->author 	  = 'SliceWP';
		$this->author_url = 'https://slicewp.com/';
		
		parent::__construct();

		add_action( 'slicewp_activated_add_on', array( $this, 'set_default_affiliate_fields_settings' ) );

	}


	/**
	 * Include files.
	 * 
	 */
	protected function include_files() {

		// If the legacy add-on is active, don't load any files.
		if ( _slicewp_is_legacy_single_add_on_active( $this->slug ) ) {
			return;
		}

		$files = array(
			'base/functions.php',
			'admin/functions.php',
			'admin/class-affiliate-fields-editor.php'
		);

		foreach ( $files as $file ) {

			include plugin_dir_path( __FILE__ ) . 'includes/' . $file;

		}

	}


	/**
	 * Sets the affiliate fields settings, if they are not set already.
	 *
	 * Will add the same fields as the core's registration form.
	 * 
	 * @param string $add_on_slug
	 *
	 */
	public function set_default_affiliate_fields_settings( $add_on_slug ) {

		if ( $add_on_slug != $this->slug ) {
			return;
		}

		$affiliate_fields = slicewp_get_option( 'affiliate_fields', array() );

		if ( ! empty( $affiliate_fields ) ) {
			return;
		}

		// Username field.
		$affiliate_fields[] = array(
			'settings' => array(
				'label' => __( 'Username', 'slicewp' )
			),
			'type' => 'user_login'
		);

		// First Name field.
		$affiliate_fields[] = array(
			'settings' => array(
				'label' 	  => __( 'First Name', 'slicewp' ),
				'is_required' => true
			),
			'type' => 'first_name'
		);

		// Last Name field.
		$affiliate_fields[] = array(
			'settings' => array(
				'label' 	  => __( 'Last Name', 'slicewp' ),
				'is_required' => true
			),
			'type' => 'last_name'
		);

		// Email field.
		$affiliate_fields[] = array(
			'settings' => array(
				'label' => __( 'Email', 'slicewp' )
			),
			'type' => 'user_email'
		);

		// Password field.
		$affiliate_fields[] = array(
			'settings' => array(
				'label' => __( 'Password', 'slicewp' )
			),
			'type' => 'password'
		);

		// Password Confirmation field.
		$affiliate_fields[] = array(
			'settings' => array(
				'label' => __( 'Password Confirmation', 'slicewp' )
			),
			'type' => 'password_confirm'
		);

		// Payment Email field.
		$affiliate_fields[] = array(
			'settings' => array(
				'label'    	  => __( 'Payment Email', 'slicewp' ),
				'is_required' => ( slicewp_get_setting( 'required_field_payment_email', false ) ? true : false ),
				'form'		  => array( 'affiliate_registration', 'affiliate_account' )
			),
			'type' => 'payment_email'
		);

		// Website field.
		$affiliate_fields[] = array(
			'settings' => array(
				'label' 	  => __( 'Website', 'slicewp' ),
				'is_required' => ( slicewp_get_setting( 'required_field_website', false ) ? true : false ),
				'form'		  => array( 'affiliate_registration', 'affiliate_account' )
			),
			'type' => 'website'
		);

		// Promotional methods.
		$affiliate_fields[] = array(
			'settings' => array(
				'label' 	  => __( 'How will you promote us?', 'slicewp' ),
				'is_required' => ( slicewp_get_setting( 'required_field_promotional_methods', false ) ? true : false ),
				'key'		  => 'promotional_methods',
				'form'		  => array( 'affiliate_registration' )
			),
			'type' => 'textarea'
		);

		// Update the affiliate fields.
		slicewp_update_option( 'affiliate_fields', $affiliate_fields );

	}


	/**
     * Registers and enqueues scripts and styles on the admin side.
     * 
     */
    public function enqueue_admin_scripts() {

        // Load admin scripts and styles only on our pages.
        if ( empty( $_GET['page'] ) || ! is_string( $_GET['page'] ) || strpos( $_GET['page'], 'slicewp' ) === false ) {
            return;
        }

        // Add-on styles.
		wp_register_style( 'slicewp-' . esc_attr( str_replace( '_', '-', $this->slug ) ) . '-style', $this->get_dir_url() . 'assets/css/style-admin.css', array(), $this->get_parent_plugin_version() );
		wp_enqueue_style( 'slicewp-' . esc_attr( str_replace( '_', '-', $this->slug ) ) . '-style' );

		// Add-on script.
		wp_register_script( 'slicewp-' . esc_attr( str_replace( '_', '-', $this->slug ) ) . '-script', $this->get_dir_url() . 'assets/js/script-admin.js', array( 'jquery', 'slicewp-script', 'jquery-ui-sortable', 'wp-color-picker', 'jquery-ui-datepicker', 'jquery-ui-autocomplete' ), $this->get_parent_plugin_version() );
		wp_enqueue_script( 'slicewp-' . esc_attr( str_replace( '_', '-', $this->slug ) ) . '-script' );

    }

}


/**
 * Registers the Custom Affiliate Fields add-on.
 * 
 * @param array
 * 
 * @return array
 * 
 */
function slicewp_register_add_on_custom_affiliate_fields( $add_ons ) {

	$add_ons['custom_affiliate_fields'] = 'SliceWP_Add_On_Custom_Affiliate_Fields';

	return $add_ons;

}
add_filter( 'slicewp_register_add_on', 'slicewp_register_add_on_custom_affiliate_fields', 20 );