<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Handles the settings fields and meta-data saving on SliceWP's users side
 *
 */
class SliceWP_Users_Custom_Affiliate_Slug {

	/**
	 * Initializer.
	 *
	 */
	public static function init() {

		// Add "custom_slug" field to the affiliate account settings.
		add_filter( 'slicewp_register_affiliate_fields', array( __CLASS__, 'register_affiliate_fields' ), 15 );

		// Validate the affiliate specific custom affiliate slug settings.
		add_action( 'slicewp_user_action_update_affiliate_settings', array( __CLASS__, 'validate_metadata' ), 25 );

		// Adds the custom slug affiliate link to the affiliate account.
		add_action( 'slicewp_affiliate_account_tab_affiliate_links_top', array( __CLASS__, 'include_affiliate_link_view' ) );

	}


	/**
	 * Add "custom_slug" field to the affiliate account settings.
	 *
	 * @param array $fields
	 *
	 * @return array
	 *
	 */
	public static function register_affiliate_fields( $fields ) {

		if ( is_admin() ) {
			return $fields;
		}

		if ( ! is_user_logged_in() ) {
			return $fields;
		}

		$allow_slug_sitewide  = slicewp_get_setting( 'custom_affiliate_slug_creation', false );
		$allow_slug_affiliate = slicewp_get_affiliate_meta( slicewp_get_current_affiliate_id(), 'custom_slug_creation', true );

		if ( 'disabled' == $allow_slug_affiliate ) {
			return $fields;
		}

		if ( ! $allow_slug_sitewide && empty( $allow_slug_affiliate ) ) {
			return $fields;
		}

		$fields[] = array(
			'name'  			  => 'custom_slug',
			'id'				  => 'slicewp-custom-slug',
			'type'				  => 'text',
			'label' 			  => __( 'Your Custom Slug', 'slicewp' ),
			'output_conditionals' => array( 'form' => array( 'affiliate_account' ) )
		);

		return $fields;

	}


	/**
	 * Validates the affiliate custom affiliate slug metadata.
	 *
	 */
	public static function validate_metadata() {

		if ( empty( $_POST['custom_slug'] ) ) {
			return;
		}

		$custom_slug = sanitize_text_field( $_POST['custom_slug'] );

		// Make sure the custom slug isn't numeric.
		if ( is_numeric( $custom_slug ) ) {

			slicewp_form_errors()->add( 'custom_slug', __( 'The custom slug cannot have a numeric value.', 'slicewp' ) );

		}

		// Make sure the custom slug contains only alphanumeric values.
		if ( ! ( ctype_alpha( $custom_slug ) || ctype_alnum( $custom_slug ) ) ) {

			slicewp_form_errors()->add( 'custom_slug', __( 'The custom slug contains invalid characters. Please make sure the slug contains only alphanumeric characters and no spaces.', 'slicewp' ) );

		}

		// Make sure the custom slug is unique.
		$affiliate_id = slicewp_get_current_affiliate_id();
		$_affiliate   = slicewp_get_affiliate_by_custom_slug( $custom_slug );

		if ( ! is_null( $_affiliate ) && $_affiliate->get( 'id' ) != $affiliate_id ) {

			slicewp_form_errors()->add( 'custom_slug', __( 'The custom slug is already set for another affiliate.', 'slicewp' ) );

		}

	}


	/**
	 * Adds the custom slug affiliate link to the affiliate account.
	 *
	 */
	public static function include_affiliate_link_view() {

		if ( 'custom_slug' == slicewp_get_setting( 'affiliate_referral_link_format', '' ) ) {
			return;
		}

		if ( 'custom_slug' != slicewp_get_setting( 'affiliate_referral_link_format', '' ) && empty( slicewp_get_setting( 'custom_affiliate_slug_referral_link_display', '' ) ) ) {
			return;
		}

		if ( empty( slicewp_get_affiliate_meta( slicewp_get_current_affiliate_id(), 'custom_slug', true ) ) ) {
			return;
		}

		include 'views/view-custom-affiliate-slug-referral-link.php';

	}
	
}

SliceWP_Users_Custom_Affiliate_Slug::init();