<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Handles batch export for commissions
 *
 */
class SliceWP_Batch_Export_Commissions extends SliceWP_Batch_Export {

	/**
	 * The export type.
	 *
	 * @access protected 
	 * @var    string
	 *
	 */
	protected $export_type = 'commissions';


	/**
	 * Returns the columns for the CSV export file.
	 *
	 * @return array
	 *
	 */
	protected function csv_cols() {

		$cols = array(
			'commission_id'    	   => __( 'Commission ID', 'slicewp' ),
			'affiliate_id'		   => __( 'Affiliate ID', 'slicewp' ),
			'affiliate_email'	   => __( 'Affiliate Email', 'slicewp' ),
			'affiliate_first_name' => __( 'Affiliate First Name', 'slicewp' ),
			'affiliate_last_name'  => __( 'Affiliate Last Name', 'slicewp' ),
			'amount'			   => __( 'Amount', 'slicewp' ),
			'currency'			   => __( 'Currency', 'slicewp' ),
			'reference'			   => __( 'Reference', 'slicewp' ),
			'type'				   => __( 'Type', 'slicewp' ),
			'status'          	   => __( 'Status', 'slicewp' ),
			'date' 	 			   => __( 'Date', 'slicewp' )
		);

		return $cols;

	}


	/**
	 * Returns the total number of commissions that should be processed.
	 *
	 * @return int
	 *
	 */
	protected function fetch_items_total() {

		// Get the start date from the filter, or set it to the default value if not present
		$date_min = ( ! empty( $this->filters['date_min'] ) ? new DateTime( $this->filters['date_min'] . ' 00:00:00' ) : '' );

		// Get the end date from the filter, or set it to the default value if not present
		$date_max = ( ! empty( $this->filters['date_max'] ) ? new DateTime( $this->filters['date_max'] . ' 23:59:59') : '' );		

		$args = array(
			'status'   	   => ( ! empty( $this->filters['status'] ) ? sanitize_text_field( $this->filters['status'] ) : '' ),
			'affiliate_id' => ( ! empty( $this->filters['affiliate_id'] ) ? absint( $this->filters['affiliate_id']  ) : '' ),
			'date_min' 	   => ( ! empty( $date_min ) ? get_gmt_from_date( $date_min->format( 'Y-m-d H:i:s' ) ) : '' ),
			'date_max' 	   => ( ! empty( $date_max ) ? get_gmt_from_date( $date_max->format( 'Y-m-d H:i:s' ) ) : '' ),
			'number'   	   => -1
		);

		return absint( slicewp_get_commissions( $args, true ) );

	}


	/**
	 * Returns an array with the commission data for the current batch step.
	 *
	 * @return array
	 *
	 */
	protected function get_data() {

		$data = array();

		$commission_types 	 = slicewp_get_commission_types();
		$commission_statuses = slicewp_get_commission_available_statuses();

		// Get the start date from the filter, or set it to the default value if not present
		$date_min = ( ! empty( $this->filters['date_min'] ) ? new DateTime( $this->filters['date_min'] . ' 00:00:00' ) : '' );

		// Get the end date from the filter, or set it to the default value if not present
		$date_max = ( ! empty( $this->filters['date_max'] ) ? new DateTime( $this->filters['date_max'] . ' 23:59:59') : '' );

		$commissions_args = array(
			'status'   	   => ( ! empty( $this->filters['status'] ) ? sanitize_text_field( $this->filters['status'] ) : '' ),
			'affiliate_id' => ( ! empty( $this->filters['affiliate_id'] ) ? absint( $this->filters['affiliate_id']  ) : '' ),
			'date_min' 	   => ( ! empty( $date_min ) ? get_gmt_from_date( $date_min->format( 'Y-m-d H:i:s' ) ) : '' ),
			'date_max' 	   => ( ! empty( $date_max ) ? get_gmt_from_date( $date_max->format( 'Y-m-d H:i:s' ) ) : '' ),
			'number'   	   => $this->per_step,
			'offset'   	   => $this->get_offset()
		);

		$commissions = slicewp_get_commissions( $commissions_args );

		if ( ! empty( $commissions ) ) {

			foreach ( $commissions as $commission ) {

				$affiliate = slicewp_get_affiliate( $commission->get( 'affiliate_id' ) );

				if ( ! $affiliate ) {
					continue;
				}

				$user = get_user_by( 'id', $affiliate->get( 'user_id' ) );

				if ( ! $user ) {
					continue;
				}

				$data[] = apply_filters( 'slicewp_export_get_data_item_' . $this->export_type, array(
					'commission_id'		   => $commission->get( 'id' ),
					'affiliate_id'    	   => $affiliate->get( 'id' ),
					'affiliate_email'      => $user->user_email,
					'affiliate_first_name' => $user->first_name,
					'affiliate_last_name'  => $user->last_name,
					'amount'			   => $commission->get( 'amount' ),
					'currency'			   => $commission->get( 'currency' ),
					'reference'			   => $commission->get( 'reference' ),
					'type'				   => ( ! empty( $commission_types[$commission->get( 'type' )]['label'] ) ? $commission_types[$commission->get( 'type' )]['label'] : $commission->get( 'type' ) ),
					'status'          	   => ( ! empty( $commission_statuses[$commission->get( 'status' )] ) ? $commission_statuses[$commission->get( 'status' )] : $commission->get( 'status' ) ),
					'date' 	 			   => slicewp_date_i18n( esc_attr( $commission->get( 'date_created' ) ) )
				), $commission );

			}

		}

		return $this->filter_data( $data );

	}


	/**
	 * Returns a message based on the provided slug.
	 *
	 * @param string $slug
	 *
	 * @return string
	 *
	 */
	public function get_message( $slug = '' ) {

		if ( 'complete' == $slug ) {

			return sprintf( __( '%s commissions exported.', 'slicewp' ), '<strong>' . absint( $this->items_total ) . '</strong>' );

		}

		return '';

	}

}