<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Validates and handles the adding of the new lead commission in the database
 *
 */
function slicewp_admin_action_add_lead_commission() {

	// Verify for nonce
	if ( empty( $_POST['slicewp_token'] ) || ! wp_verify_nonce( $_POST['slicewp_token'], 'slicewp_add_lead_commission' ) )
		return;

	// Verify for affiliate ID
	if ( empty( $_POST['affiliate_id'] ) ) {

		slicewp_admin_notices()->register_notice( 'commission_affiliate_id_missing', '<p>' . __( 'Please select the affiliate you wish to assign this commission to.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'commission_affiliate_id_missing' );

		return;

	}

	// Verify for commission status
	if ( empty( $_POST['status'] ) ) {

		slicewp_admin_notices()->register_notice( 'commission_status_missing', '<p>' . __( 'Please select the status of the new commission.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'commission_status_missing' );

		return;

	}

	$statuses = slicewp_get_commission_available_statuses();

	// Verify if the commission status is valid
	if ( ! in_array( $_POST['status'], array_keys( $statuses ) ) ) {

		slicewp_admin_notices()->register_notice( 'commission_status_invalid', '<p>' . __( 'The selected status in not allowed.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'commission_status_invalid' );

		return;

	}

	$_POST = stripslashes_deep( $_POST );

	// Get the lead data
	$lead = slicewp_get_lead( absint( $_GET['lead_id'] ) );
	
	if ( empty( $lead ) )
		return;

	// Verify if we already have a commission for this lead
	$commission_args = array(
		'origin'	=> sanitize_text_field( $lead->get( 'origin' ) ),
		'reference'	=> absint( $lead->get( 'entry_id' ) )
	);

	$commissions = slicewp_get_commissions( $commission_args );

	if ( ! empty( $commissions ) ) {

		slicewp_admin_notices()->register_notice( 'commission_already_exists', '<p>' . __( 'A commission already exists for this lead.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'commission_already_exists' );

		return;

	}

	// Process the customer
	$customer_args = array(
		'email' 		=> sanitize_email( $lead->get( 'email' ) ),
		'first_name' 	=> sanitize_text_field( $lead->get( 'first_name' ) ),
		'last_name' 	=> sanitize_text_field( $lead->get( 'last_name' ) ),
	);

	$customer_id = slicewp_process_customer( $customer_args );
	
	if ( $customer_id )
		slicewp_add_log( sprintf( '%s: Customer #%s has been successfully processed.', strtoupper( $_POST['origin'] ), $customer_id ) );
	else
		slicewp_add_log( sprintf( '%s: Customer could not be processed due to an unexpected error.', strtoupper( $_POST['origin'] ) ) );


	// Prepare commission data to be inserted
	$commission_data = array(
		'affiliate_id'  => absint( $_POST['affiliate_id'] ),
		'visit_id'		=> ( ! is_null( $_POST['visit_id'] ) ? absint( $_POST['visit_id'] ) : 0 ),
		'date_created'  => slicewp_mysql_gmdate(),
		'date_modified' => slicewp_mysql_gmdate(),
		'type'			=> 'lead',
		'status'		=> sanitize_text_field( $_POST['status'] ),
		'reference'		=> ( ! empty( $_POST['reference'] ) ? sanitize_text_field( $_POST['reference'] ) : '' ),
		'origin'		=> ( ! empty( $_POST['origin'] ) ? sanitize_text_field( $_POST['origin'] ) : '' ),
		'customer_id'	=> absint( $customer_id ),
		'amount'		=> slicewp_sanitize_amount( ( ! empty( $_POST['amount'] ) ? sanitize_text_field( $_POST['amount'] ) : '0' ) ),
		'currency'		=> slicewp_get_setting( 'active_currency', 'USD' )
	);

	// Insert commission into the database
	$commission_id = slicewp_insert_commission( $commission_data );

	// If the commission could not be inserted show a message to the user
	if ( ! $commission_id ) {

		slicewp_admin_notices()->register_notice( 'commission_insert_false', '<p>' . __( 'Something went wrong. Could not add the commission. Please try again.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'commission_insert_false' );

		return;

	}

	// Update the visit with the commission id
	slicewp_update_visit( absint( $_POST['visit_id'] ), array( 'date_modified' => slicewp_mysql_gmdate(), 'commission_id' => absint( $commission_id ) ) );
	
	// Update the lead with the commission id
	$lead_args = array(
		'commission_id' => absint( $commission_id ),
		'date_modified' => slicewp_mysql_gmdate()
	);

	slicewp_update_lead( $lead->get( 'id' ), $lead_args );

	// Redirect to the edit page of the commission with a success message
	wp_redirect( add_query_arg( array( 'page' => 'slicewp-leads', 'slicewp_message' => 'commission_insert_success' ), admin_url( 'admin.php' ) ) );
	exit;

}
add_action( 'slicewp_admin_action_add_lead_commission', 'slicewp_admin_action_add_lead_commission', 50 );


/**
 * Validates and handles the deleting of a lead from the database
 *
 */
function slicewp_admin_action_delete_lead() {

	// Verify for nonce
	if ( empty( $_GET['slicewp_token'] ) || ! wp_verify_nonce( $_GET['slicewp_token'], 'slicewp_delete_lead' ) )
		return;

	// Verify for lead id
	if ( empty( $_GET['lead_id'] ) )
		return;

	// Verify for lead's existance
	$lead_id	= absint( $_GET['lead_id'] );
	$lead		= slicewp_get_lead( $lead_id );

	if ( is_null( $lead ) )
		return;

	// Delete the lead
	$deleted = slicewp_delete_lead( $lead_id );

	if ( ! $deleted ) {

		slicewp_admin_notices()->register_notice( 'lead_delete_false', '<p>' . __( 'Something went wrong. Could not delete the lead. Please try again.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'lead_delete_false' );

		return;

	}

	// Redirect to the current page
	wp_redirect( add_query_arg( array( 'page' => 'slicewp-leads', 'slicewp_message' => 'lead_delete_success' ), admin_url( 'admin.php' ) ) );
	exit;

}
add_action( 'slicewp_admin_action_delete_lead', 'slicewp_admin_action_delete_lead', 50 );
