<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * The class that defines the Add Lead Commission action
 *
 */
final class SliceWP_Ninja_Forms_Action_Add_Lead_Commission extends NF_Abstracts_Action {

	/**
	 * @var string
	 *
	 */
	protected $_name  = 'slicewp_add_lead_commission';
	
	/**
	 * @var array
	 *
	 */
	protected $_tags = array( 'slicewp', 'commission', 'affiliate' );

	/**
	 * @var string
	 *
	 */
	protected $_timing = 'late';

	/**
	 * @var int
	 *
	 */
	protected $_priority = 15;


	/**
	 * Constructor.
	 *
	 */
	public function __construct() {

		parent::__construct();

		$this->_nicename = __( 'Add Lead Commission', 'slicewp' );

		$this->_settings[ 'slicewp_customer_email' ] = array(
			'name'				=> 'slicewp_customer_email',
			'label'				=> __( 'Customer Email', 'slicewp' ),
			'type'				=> 'textbox',
			'width'				=> 'full',
			'value'				=> '',
			'group'				=> 'primary',
			'use_merge_tags'	=> array(
				'exclude' => array(
					'user'
				)
			)
		);
		
		$this->_settings[ 'slicewp_customer_first_name' ] = array(
			'name'				=> 'slicewp_customer_first_name',
			'label'				=> __( 'Customer First Name', 'slicewp' ),
			'type'				=> 'textbox',
			'width'				=> 'one-half',
			'value'				=> '',
			'group'				=> 'primary',
			'use_merge_tags'	=> array(
				'exclude' => array(
					'user'
				)
			)
		);

		$this->_settings[ 'slicewp_customer_last_name' ] = array(
			'name'				=> 'slicewp_customer_last_name',
			'label'				=> __( 'Customer Last Name', 'slicewp' ),
			'type'				=> 'textbox',
			'width'				=> 'one-half',
			'value'				=> '',
			'group'				=> 'primary',
			'use_merge_tags'	=> array(
				'exclude' => array(
					'user'
				)
			)
		);

	}


	/**
	 * Save.
	 *
	 */
	public function save( $action_settings ) {}


	/**
	 * Process the form entry.
	 * 
	 */
	public function process( $action_settings, $form_id, $data ) {

        if ( isset( $data['settings']['is_preview'] ) && $data['settings']['is_preview'] ) {
			return $data;
		}

		$sub_data = array(
			'form_id'			=> $form_id,
			'entry_id'			=> $data['actions']['save']['sub_id'],
			'customer_email'	=> ! empty( $action_settings[ 'slicewp_customer_email' ] ) ? $action_settings[ 'slicewp_customer_email' ] : '',
			'first_name'		=> ! empty( $action_settings[ 'slicewp_customer_first_name' ] ) ? $action_settings[ 'slicewp_customer_first_name' ] : '',
			'last_name'			=> ! empty( $action_settings[ 'slicewp_customer_last_name' ] ) ? $action_settings[ 'slicewp_customer_last_name' ] : ''
		);

		// Create the commission
		$commission_id = $this->add_lead_commission( $sub_data );

		$sub_data['commission_id'] = ! empty( $commission_id ) ? $commission_id : '';

		$data['extra']['slicewp']['commission'] = $sub_data;

		return $data;

	}


	/**
	 * Adds the lead commission
	 * 
	 * @param $sub_data array
	 * 
	 * @return int
	 * 
	 */
	public function add_lead_commission( $sub_data ) {

		// Get and check to see if referrer exists
		$affiliate_id = slicewp_get_referrer_affiliate_id();
		$visit_id	  = slicewp_get_referrer_visit_id();

		/**
		 * Filters the referrer affiliate ID for Ninja Forms.
		 * This is mainly used by add-ons for different functionality.
		 *
		 * @param int $affiliate_id
		 * @param array $sub_data
		 *
		 */
		$affiliate_id = apply_filters( 'slicewp_referrer_affiliate_id_nfo', $affiliate_id, $sub_data );

		if ( empty( $affiliate_id ) ) {
			return 0;
		}

		// Verify if the affiliate is valid.
		if ( ! slicewp_is_affiliate_valid( $affiliate_id ) ) {

			slicewp_add_log( 'NFO: Lead commission was not created because the affiliate is not valid.' );
			return 0;

		}

		// Check to see if a commission for this lead has been registered.
		$commissions = slicewp_get_commissions( array( 'reference' => $sub_data['entry_id'], 'origin' => 'nfo' ) );

		if ( ! empty( $commissions ) ) {

			slicewp_add_log( 'NFO: Lead commission was not created because another commission for the reference and origin already exists.' );
			return 0;

		}

		// Check to see if the affiliate made the lead.
		if ( empty( slicewp_get_setting( 'affiliate_own_commissions' ) ) ) {

			$affiliate = slicewp_get_affiliate( $affiliate_id );

			if ( is_user_logged_in() && get_current_user_id() == $affiliate->get( 'user_id' ) ) {

				slicewp_add_log( 'NFO: Lead commission was not created because the customer is also the affiliate.' );
				return 0;

			}

			if ( ! empty( $sub_data['customer_email'] ) ) {

				if ( slicewp_affiliate_has_email( $affiliate_id, $customer_email ) ) {

					slicewp_add_log( 'NFO: Lead commission was not created because the customer is also the affiliate.' );
					return 0;

				}

			}

		}

		// Process the customer.
		$customer_args = array(
			'email'   	 	=> ( ! empty( $sub_data['customer_email'] ) ? $sub_data['customer_email'] : '' ),
			'user_id' 	 	=> ( is_user_logged_in() ? get_current_user_id() : '' ),
			'first_name'	=> ( ! empty( $sub_data['first_name'] ) ? $sub_data['first_name'] : '' ),
			'last_name'		=> ( ! empty( $sub_data['last_name']  ) ? $sub_data['last_name'] : '' )
		);

		$customer_id = slicewp_process_customer( $customer_args );

		if ( $customer_id ) {
			slicewp_add_log( sprintf( 'NFO: Customer #%s has been successfully processed.', $customer_id ) );
		} else {
			slicewp_add_log( 'NFO: Customer could not be processed due to an unexpected error.' );
		}

		// Calculate commission amount.
		$args = array(
			'origin'		=> 'nfo',
			'type'			=> 'lead',
			'affiliate_id'	=> $affiliate_id,
			'customer_id'	=> $customer_id
		);

		$commission_amount = slicewp_calculate_commission_amount( 0, $args );

		// Prepare commission data.
		$commission_data = array(
			'affiliate_id'	=> absint( $affiliate_id ),
			'visit_id'		=> ( ! is_null( $visit_id ) ? absint( $visit_id ) : 0 ),
			'date_created'	=> slicewp_mysql_gmdate(),
			'date_modified'	=> slicewp_mysql_gmdate(),
			'type'			=> 'lead',
			'status'		=> 'unpaid',
			'reference'		=> absint( $sub_data['entry_id'] ),
			'customer_id'	=> absint( $customer_id ),
			'origin'		=> 'nfo',
			'amount'		=> slicewp_sanitize_amount( $commission_amount ),
			'currency'		=> slicewp_get_setting( 'active_currency', 'USD' )
		);

		// Insert the commission.
		$commission_id = slicewp_insert_commission( $commission_data );

		if ( ! empty( $commission_id ) ) {

			// Update the visit with the newly inserted commission_id
			if ( ! is_null( $visit_id ) ) {

				slicewp_update_visit( $visit_id, array( 'date_modified' => slicewp_mysql_gmdate(), 'commission_id' => $commission_id ) );

			}

			// Get the lead created for this entry
			$leads_args = array(
				'origin'   => 'nfo',
				'entry_id' => absint( $sub_data['entry_id'] )
			);

			$leads = slicewp_get_leads( $leads_args );

			if ( ! empty( $leads ) ) {

				// Update the lead with the commission id
				slicewp_update_lead( $leads[0]->get( 'id' ), array( 'commission_id' => $commission_id ) );

			}

			slicewp_add_log( sprintf( 'NFO: Lead commission #%s has been successfully inserted.', $commission_id ) );
			return $commission_id;

		} else {

			slicewp_add_log( 'NFO: Lead commission could not be inserted due to an unexpected error.' );
			return 0;

		}

	}

}