<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * The class that defines the Register Lead action
 *
 */
final class SliceWP_Ninja_Forms_Action_Register_Lead extends NF_Abstracts_Action {

	/**
	 * @var string
	 *
	 */
	protected $_name  = 'slicewp_register_lead';
	
	/**
	 * @var array
	 *
	 */
	protected $_tags = array( 'slicewp', 'lead', 'affiliate' );

	/**
	 * @var string
	 *
	 */
	protected $_timing = 'late';

	/**
	 * @var int
	 *
	 */
	protected $_priority = 10;


	/**
	 * Constructor.
	 *
	 */
	public function __construct() {

		parent::__construct();

		$this->_nicename = __( 'Register Lead', 'slicewp' );

		$this->_settings[ 'slicewp_customer_email' ] = array(
			'name'				=> 'slicewp_customer_email',
			'label'				=> __( 'Customer Email', 'slicewp' ),
			'type'				=> 'textbox',
			'width'				=> 'full',
			'value'				=> '',
			'group'				=> 'primary',
			'use_merge_tags'	=> array(
				'exclude' => array(
					'user'
				)
			)
		);

		$this->_settings[ 'slicewp_customer_first_name' ] = array(
			'name'				=> 'slicewp_customer_first_name',
			'label'				=> __( 'Customer First Name', 'slicewp' ),
			'type'				=> 'textbox',
			'width'				=> 'one-half',
			'value'				=> '',
			'group'				=> 'primary',
			'use_merge_tags'	=> array(
				'exclude' => array(
					'user'
				)
			)
		);

		$this->_settings[ 'slicewp_customer_last_name' ] = array(
			'name'				=> 'slicewp_customer_last_name',
			'label'				=> __( 'Customer Last Name', 'slicewp' ),
			'type'				=> 'textbox',
			'width'				=> 'one-half',
			'value'				=> '',
			'group'				=> 'primary',
			'use_merge_tags'	=> array(
				'exclude' => array(
					'user'
				)
			)
		);

	}

	
	/**
	 * Save.
	 *
	 */
	public function save( $action_settings ) {}


	/**
	 * Process the form entry.
	 * 
	 */
	public function process( $action_settings, $form_id, $data ) {
		
        if ( isset( $data['settings']['is_preview'] ) && $data['settings']['is_preview'] ) {
			return $data;
		}

		$sub_data = array(
			'form_id'			=> $form_id,
			'entry_id'			=> $data['actions']['save']['sub_id'],
			'customer_email'	=> ( ! empty( $action_settings[ 'slicewp_customer_email' ] ) ? $action_settings[ 'slicewp_customer_email' ] : '' ),
			'first_name'		=> ( ! empty( $action_settings[ 'slicewp_customer_first_name' ] ) ? $action_settings[ 'slicewp_customer_first_name' ] : '' ),
			'last_name'			=> ( ! empty( $action_settings[ 'slicewp_customer_last_name' ] ) ? $action_settings[ 'slicewp_customer_last_name' ] : '' )
		);

		// Register the lead
		$lead_id = $this->register_lead( $sub_data );

		$sub_data['lead_id'] = ! empty( $lead_id ) ? $lead_id : '';

		$data['extra']['slicewp']['lead'] = $sub_data;

		return $data;

	}


	/**
	 * Registers the lead
	 * 
	 * @param $sub_data array
	 * 
	 * @return int
	 * 
	 */
	public function register_lead( $sub_data ) {

		// Get and check to see if referrer exists
		$affiliate_id = slicewp_get_referrer_affiliate_id();
		$visit_id	  = slicewp_get_referrer_visit_id();

		/**
		 * Filters the referrer affiliate ID for Ninja Forms.
		 * This is mainly used by add-ons for different functionality.
		 *
		 * @param int $affiliate_id
		 * @param array $sub_data
		 *
		 */
		$affiliate_id = apply_filters( 'slicewp_referrer_affiliate_id_nfo', $affiliate_id, $sub_data );

		if ( empty( $affiliate_id ) ) {
			return 0;
		}

		// Verify if the affiliate is valid
		if ( ! slicewp_is_affiliate_valid( $affiliate_id ) ) {

			slicewp_add_log( 'NFO: Lead was not created because the affiliate is not valid.' );
			return 0;

		}

		// Check to see if a lead for this lead has been registered
		$leads = slicewp_get_leads( array( 'form_id' => $sub_data['form_id'], 'entry_id' => $sub_data['entry_id'], 'origin' => 'nfo' ) );

		if ( ! empty( $leads ) ) {

			slicewp_add_log( 'NFO: Lead was not created because another lead for this entry already exists.' );
			return 0;

		}

		// Check to see if the affiliate made the lead
		if ( empty( slicewp_get_setting( 'affiliate_own_commissions' ) ) ) {

			$affiliate = slicewp_get_affiliate( $affiliate_id );

			if ( is_user_logged_in() && get_current_user_id() == $affiliate->get( 'user_id' ) ) {

				slicewp_add_log( 'NFO: Lead was not created because the customer is also the affiliate.' );
				return 0;

			}

			if ( ! empty( $sub_data['customer_email'] ) ) {

				if ( slicewp_affiliate_has_email( $affiliate_id, $customer_email ) ) {

					slicewp_add_log( 'NFO: Lead was not created because the customer is also the affiliate.' );
					return 0;

				}

			}

		}

		// Prepare lead data
		$lead_data = array(
			'affiliate_id'		=> absint( $affiliate_id ),
			'visit_id'			=> ( ! is_null( $visit_id ) ? absint( $visit_id ) : 0 ),
			'origin'			=> 'nfo',
			'form_id'			=> absint( $sub_data['form_id'] ),
			'entry_id'			=> absint( $sub_data['entry_id'] ),
			'date_created'		=> slicewp_mysql_gmdate(),
			'date_modified'		=> slicewp_mysql_gmdate(),
			'email'				=> ! empty( $sub_data['customer_email'] ) ? sanitize_text_field( $sub_data['customer_email'] ) : '',
			'first_name'		=> ! empty( $sub_data['first_name'] ) ? sanitize_text_field( $sub_data['first_name'] ) : '',
			'last_name'			=> ! empty( $sub_data['last_name'] ) ? sanitize_text_field( $sub_data['last_name'] ) : ''
		);

		// Insert the lead
		$lead_id = slicewp_insert_lead( $lead_data );

		if ( ! empty( $lead_id ) ) {

			slicewp_add_log( sprintf( 'NFO: Lead #%s has been successfully inserted.', $lead_id ) );			
			return $lead_id;

		} else {

			slicewp_add_log( 'NFO: Lead could not be inserted due to an unexpected error.' );
			return 0;

		}

	}

}