<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * WS Form "Register lead" action.
 * 
 */
class SliceWP_WS_Form_Action_Register_Lead extends WS_Form_Action {

    public $id = 'slicewp_register_lead';
    public $pro_required = false;
    public $label;
    public $label_action;
    public $multiple = false;
    public $events;
    public $configured = true;
    public $priority = 185;
    public $can_repost = false;
    public $form_add = false;
    public $woocommerce_bypass = true;

    // Config.
    public $lead_first_name;
    public $lead_last_name;
    public $lead_email_address;


    /**
     * Constructor.
     * 
     */
    public function __construct() {

        // Set label.
        $this->label = __( 'Register Lead', 'slicewp' );

        // Set label for actions pull down.
        $this->label_action = __( 'Register Lead', 'slicewp' );

        // Events.
        $this->events = array( 'submit' );

        // Register config filters.
        add_filter( 'wsf_config_meta_keys', array( $this, 'config_meta_keys' ), 10, 2 );

        // Register action.
        parent::register( $this );

    }


    /**
     * Get settings.
     * 
     */
    public function get_action_settings() {

        $settings = array(
            'meta_keys'	=> array(
                'action_' . $this->id . '_lead_email_address',
                'action_' . $this->id . '_lead_first_name',
                'action_' . $this->id . '_lead_last_name'
            )
        );

        // Wrap settings so they will work with sidebar_html function in admin.js.
        $settings = parent::get_settings_wrapper( $settings );

        // Add labels.
        $settings->label = $this->label;
        $settings->label_action = $this->label_action;

        // Add multiple.
        $settings->multiple = $this->multiple;

        // Add events.
        $settings->events = $this->events;

        // Add can_repost.
        $settings->can_repost = $this->can_repost;

        // Apply filter.
        $settings = apply_filters( 'wsf_action_email_settings', $settings );

        return $settings;
    }

    
    /**
     * Meta keys for this action
     * 
     */
    public function config_meta_keys( $meta_keys = array(), $form_id = 0 ) {

        // Build config_meta_keys
        $config_meta_keys = array(

            // Lead Email Address.
            'action_' . $this->id . '_lead_email_address' => array(
                'label'						=>	__( 'Lead Email', 'slicewp' ),
                'type'						=>	'text',
                'help'						=>	__( "The lead's email address.", 'slicewp' ),
                'default'					=>	'',
                'select_list'				=>	true
            ),

            // Lead first name.
            'action_' . $this->id . '_lead_first_name' => array(
                'label'						=>	__( 'Lead First Name', 'slicewp' ),
                'type'						=>	'text',
                'help'						=>	__( "The lead's first name.", 'slicewp' ),
                'default'					=>	'',
                'select_list'				=>	true
            ),

            // Lead Last Name.
            'action_' . $this->id . '_lead_last_name' => array(
                'label'						=>	__( 'Lead Last Name', 'slicewp' ),
                'type'						=>	'text',
                'help'						=>	__( "The lead's first name.", 'slicewp' ),
                'default'					=>	'',
                'select_list'				=>	true
            )

        );

        // Merge
        $meta_keys = array_merge( $meta_keys, $config_meta_keys );

        return $meta_keys;

    }


    /**
     * Runs the action.
     * 
     */
    public function post( $form, $submit, $config ) {

        // Load the config.
        self::load_config( $config );

        $submit_parse = clone $submit;

        // Get submitted values.
        $lead_email_address = sanitize_email( WS_Form_Common::parse_variables_process( trim( $this->lead_email_address ), $form, $submit_parse, 'text/plain' ) );
        $lead_first_name    = sanitize_text_field( WS_Form_Common::parse_variables_process( trim( $this->lead_first_name ), $form, $submit_parse, 'text/plain' ) );
        $lead_last_name     = sanitize_text_field( WS_Form_Common::parse_variables_process( trim( $this->lead_last_name ), $form, $submit_parse, 'text/plain' ) );

        // Get and check to see if referrer exists.
		$affiliate_id = slicewp_get_referrer_affiliate_id();
		$visit_id	  = slicewp_get_referrer_visit_id();

        /**
		 * Filters the referrer affiliate ID for WS Form.
		 * This is mainly used by add-ons for different functionality.
		 *
		 * @param int $affiliate_id
		 * @param array $sub_data
		 *
		 */
		$affiliate_id = apply_filters( 'slicewp_referrer_affiliate_id_wsf', $affiliate_id, $submit );

		if ( empty( $affiliate_id ) ) {
            return;
        }

        // Verify if the affiliate is valid.
		if ( ! slicewp_is_affiliate_valid( $affiliate_id ) ) {

			slicewp_add_log( 'WS Form: Lead was not created because the affiliate is not valid.' );
			return;

		}

        // Check to see if a lead for this lead has been registered.
		$leads = slicewp_get_leads( array( 'form_id' => $submit->form_id, 'entry_id' => $submit->id, 'origin' => 'wsf' ) );

		if ( ! empty( $leads ) ) {

			slicewp_add_log( 'WS Form: Lead was not created because another lead for this entry already exists.' );
			return;

		}

        // Check to see if the affiliate made the lead.
		if ( empty( slicewp_get_setting( 'affiliate_own_commissions' ) ) ) {

			$affiliate = slicewp_get_affiliate( $affiliate_id );

			if ( is_user_logged_in() && get_current_user_id() == $affiliate->get( 'user_id' ) ) {

				slicewp_add_log( 'WS Form: Lead was not created because the lead is also the affiliate.' );
				return;

			}

			if ( ! empty( $lead_email_address ) ) {

				if ( slicewp_affiliate_has_email( $affiliate_id, $lead_email_address ) ) {

					slicewp_add_log( 'WS Form: Lead was not created because the lead is also the affiliate.' );
					return;

				}

			}

		}

        // Prepare lead data.
        $lead_data = array(
            'affiliate_id'		=> absint( $affiliate_id ),
            'visit_id'			=> ( ! is_null( $visit_id ) ? absint( $visit_id ) : 0 ),
            'origin'			=> 'wsf',
            'form_id'			=> absint( $submit->form_id ),
            'entry_id'			=> absint( $submit->id ),
            'date_created'		=> slicewp_mysql_gmdate(),
            'date_modified'		=> slicewp_mysql_gmdate(),
            'email'				=> $lead_email_address,
            'first_name'		=> $lead_first_name,
            'last_name'			=> $lead_last_name
        );

        // Insert the lead.
        $lead_id = slicewp_insert_lead( $lead_data );

        if ( ! empty( $lead_id ) ) {

            slicewp_add_log( sprintf( 'WS Form: Lead #%s has been successfully inserted.', $lead_id ) );

    
        } else {
    
            slicewp_add_log( 'WS Form: Lead could not be inserted due to an unexpected error.' );
    
        }

    }


    /**
     * Loads the config as object parameters.
     * 
     */
    public function load_config( $config ) {

        // Get configuration.
        $this->lead_first_name    = parent::get_config( $config, 'action_' . $this->id . '_lead_first_name' );
        $this->lead_last_name     = parent::get_config( $config, 'action_' . $this->id . '_lead_last_name' );
        $this->lead_email_address = parent::get_config( $config, 'action_' . $this->id . '_lead_email_address' );

    }
    
}

new SliceWP_WS_Form_Action_Register_Lead();