<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Sanitizes the given rate, preparing it for the database.
 * 
 * @access private
 * 
 * @param float $rate
 * 
 * @return float
 * 
 */
function _slicewp_sanitize_currency_exchange_rate( $rate ) {

    if ( $rate >= 1 ) {

        $rate = number_format( $rate, 3, '.', '' );

    } else {

        $rate = number_format( $rate, 20, '.', '' );

        $fractional_part = explode( '.', $rate )[1];
        $leading_zeros   = strlen( $fractional_part ) - strlen( ltrim( $fractional_part, '0' ) );

        $rate = number_format( $rate, ( $leading_zeros + 3 ), '.', '' );
        
    }

    return $rate;

}


/**
 * Hooks into "slicewp_maybe_convert_amount_exchange_rate" to change the exchange rate between the given currencies.
 * 
 * @param float  $rate
 * @param string $currency_from
 * @param string $currency_to
 * 
 * @return float
 * 
 */
function slicewp_multi_currency_maybe_convert_amount_exchange_rate( $rate, $currency_from, $currency_to ) {

    if ( trim( $currency_from ) == trim( $currency_to ) ) {
        return $rate;
    }

    if ( trim( $currency_to ) != slicewp_get_setting( 'active_currency', 'USD' ) ) {
        return $rate;
    }

    $settings       = slicewp_get_option( 'multi_currency', array() );
    $exchange_rates = ( ! empty( $settings['currency_exchange_rates'] ) && is_array( $settings['currency_exchange_rates'] ) ? $settings['currency_exchange_rates'] : array() );

    if ( empty( $exchange_rates ) ) {
        return $rate;
    }

    foreach ( $exchange_rates as $exchange_rate ) {

        if ( $exchange_rate['currency'] != $currency_from ) {
            continue;
        }

        $rate = (float)$exchange_rate['exchange_rate'];
        break;

    }

    return $rate;

}
add_filter( 'slicewp_maybe_convert_amount_exchange_rate', 'slicewp_multi_currency_maybe_convert_amount_exchange_rate', 20, 3 );


/**
 * Callback set for the cron hook to update the exchange rates.
 * 
 */
function slicewp_multi_currency_update_exchange_rates() {

    $settings = slicewp_get_option( 'multi_currency', array() );

    // Check that we have rates.
    $rates = ( ! empty( $settings['currency_exchange_rates'] ) ? $settings['currency_exchange_rates'] : array() );

    if ( empty( $rates ) ) {
        return;
    }

    // Check that the ExchangeRate-API API key is set.
    if ( empty( $settings['exchangerate_api_api_key'] ) ) {
        return;
    }

    // Make the call to ExchangeRate-API.
    $response = wp_remote_get( 'https://v6.exchangerate-api.com/v6/' . sanitize_text_field( $settings['exchangerate_api_api_key'] ) . '/latest/' . slicewp_get_setting( 'active_currency', 'USD' ) );

    // Bail if the call is unsuccessful.
    if ( is_wp_error( $response ) ) {
	    return;
    }

    $data = json_decode( wp_remote_retrieve_body( $response ), true );

    // Bail if we don't have any rates to update.
    if ( ! isset( $data['result'] ) || $data['result'] !== 'success' || empty( $data['conversion_rates'] ) ) {
        return;
    }

    // Update the currency values.
    foreach ( $rates as $key => $rate ) {

        if ( ! in_array( $rate['currency'], array_keys( $data['conversion_rates'] ) ) ) {
            continue;
        }

        $rates[$key]['exchange_rate'] = _slicewp_sanitize_currency_exchange_rate( 1 / $data['conversion_rates'][$rate['currency']] );

    }

    // Update the settings.
    $settings['currency_exchange_rates'] = $rates;

    slicewp_update_option( 'multi_currency', $settings );

    // Update the last updated timestamp.
    $last_updated = slicewp_get_option( 'last_updated', array() );
    $last_updated['currency_exchange_rates'] = slicewp_mysql_gmdate();
    slicewp_update_option( 'last_updated', $last_updated );

}
add_action( 'slicewp_multi_currency_update_exchange_rates', 'slicewp_multi_currency_update_exchange_rates' );