<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Registers the "inherit" commission type.
 *
 * @param array $commission_types
 *
 * @return array
 *
 */
function slicewp_register_commission_type_inherit( $commission_types ) {

	$commission_types['inherit'] = array(
		'label' 	 => __( 'Inherit', 'slicewp' ),
		'rate_types' => array( 'percentage', 'fixed_amount' )
	);

	return $commission_types;

}
add_filter( 'slicewp_register_commission_types', 'slicewp_register_commission_type_inherit' );


/**
 * Returns an array of SliceWP_Affiliate objects representing the ancestors of the given affiliate.
 * 
 * @param int|SliceWP_Affiliate $affiliate
 * @param int                   $depth
 * 
 * @return array
 * 
 */
function slicewp_get_affiliate_ancestors( $affiliate, $depth = 0 ) {
    
    // Get given affiliate.
    $affiliate = slicewp_get_affiliate( $affiliate );

    if ( is_null( $affiliate ) || empty( $affiliate->get( 'parent_id' ) ) || $affiliate->get( 'parent_id' ) == $affiliate->get( 'id' ) ) {
        return array();
    }

    $ancestors  = array();
    $current_id = $affiliate->get( 'parent_id' );
    $index      = 0;

    // Go through each ancestor and add it to the list.
    while ( $ancestor = slicewp_get_affiliate( $current_id ) ) {

        if ( is_null( $ancestor ) ) {
            break;
        }

        if ( in_array( $ancestor->get( 'id' ), array_column( array_map( 'slicewp_object_to_array', $ancestors ), 'id' ) ) ) {
            break;
        }

        if ( $depth > 0 && $index >= $depth ) {
            break;
        }

        $ancestors[] = $ancestor;

        if ( empty( $ancestor->get( 'parent_id' ) ) ) {
            break;
        }

        if ( $ancestor->get( 'parent_id' ) == $affiliate->get( 'id' ) ) {
            break;
        }

        $current_id = $ancestor->get( 'parent_id' );
        $index++;

    }

    return $ancestors;

}


/**
 * Returns an array of SliceWP_Affiliate objects representing the children of the given affiliate.
 * 
 * @param int|SliceWP_Affiliate $affiliate
 * @param bool                  $count
 * 
 * @return array
 * 
 */
function slicewp_get_affiliate_children( $affiliate, $count = false ) {

    $affiliate_id = ( is_object( $affiliate ) ? $affiliate->get( 'id' ) : $affiliate );

    $args = array(
        'number'    => -1,
        'parent_id' => $affiliate_id,
        'order'     => 'ASC'
    );

    return slicewp_get_affiliates( $args, $count );

}


/**
 * Saves the referring affiliate ID as the parent_id for the affiliate that
 * just registered an account.
 * 
 * @param int $affiliate_id
 * 
 */
function slicewp_on_register_affiliate_set_parent_id( $affiliate_id ) {

    // Bail if the multi-level affiliates feature is disabled.
    if ( empty( slicewp_get_setting( 'multi_level_affiliates' ) ) ) {
        return;
    }

    // Get the referrer affiliate ID.
    $referrer_affiliate_id = slicewp_get_referrer_affiliate_id();

    if ( empty( $referrer_affiliate_id ) ) {
        return;
    }

    if ( ! slicewp_is_affiliate_active( $referrer_affiliate_id ) ) {
        return;
    }

    // Save the parent affiliate ID for the newly registered affiliate.
    slicewp_update_affiliate( $affiliate_id, array( 'parent_id' => $referrer_affiliate_id, 'date_modified' => slicewp_mysql_gmdate() ) );

}
add_action( 'slicewp_register_affiliate', 'slicewp_on_register_affiliate_set_parent_id' );


/**
 * Inserts inherited commissions when a new commission is inserted.
 * 
 * @param int   $commission_id
 * @param array $commission_data
 * 
 */
function slicewp_on_commission_insert_reward_inherited_commissions( $commission_id, $commission_data ) {

    /**
     * @todo Maybe add an option in the "Add Commission" admin page to select whether to create
     *       inherited commissions or not.
     * 
     */
    if ( slicewp_verify_request_action( 'add_commission' ) ) {
        return;
    }

    if ( empty( $commission_data['type'] ) || $commission_data['type'] == 'inherit' ) {
        return;
    }

    if ( empty( $commission_data['origin'] ) ) {
        return;
    }

    if ( empty( $commission_data['reference'] ) ) {
        return;
    }

    if ( empty( $commission_data['affiliate_id'] ) ) {
        return;
    }

    /**
     * Filter to maybe block the creation of inherited commissions.
     * 
     * @param bool  true
     * @param int   $commission_id
     * @param array $commission_data
     * 
     */
    if ( false === apply_filters( 'slicewp_should_reward_inherited_commissions', true, $commission_id, $commission_data ) ) {

        return;

    }

    // Bail if the multi-level affiliates feature is disabled.
    if ( empty( slicewp_get_setting( 'multi_level_affiliates' ) ) ) {
        return;
    }

    $levels = slicewp_get_setting( 'affiliate_levels', array() );

    // Bail if no affiliate levels have been set.
    if ( empty( $levels ) ) {
        return;
    }

    $affiliate_ancestors = slicewp_get_affiliate_ancestors( absint( $commission_data['affiliate_id'] ), count( $levels ) );

    if ( empty( $affiliate_ancestors ) ) {
        return;
    }

    foreach ( $affiliate_ancestors as $index => $ancestor ) {

        // Don't add commissions for non-active affiliates.
        if ( 'active' !== $ancestor->get( 'status' ) ) {
            continue;
        }

        // Calculate commission amount.
        $args = array(
            'origin'           => $commission_data['origin'],
            'type'             => 'inherit',
            'affiliate_level'  => ( $index + 1 )
        );

        $child_commission_amount = slicewp_calculate_commission_amount( $commission_data['amount'], $args );

        // Prepare commission data.
        $child_commission_data = array(
            'affiliate_id'     => $ancestor->get( 'id' ),
            'origin'           => $commission_data['origin'],
            'reference'        => $commission_data['reference'],
            'reference_amount' => 0,
            'status'           => ( ! empty( $commission_data['status'] ) ? sanitize_text_field( $commission_data['status'] ) : 'pending' ),
            'currency'         => ( ! empty( $commission_data['currency'] ) ? sanitize_text_field( $commission_data['currency'] ) : slicewp_get_setting( 'active_currency', 'USD' ) ),
            'date_created'     => slicewp_mysql_gmdate(),
		    'date_modified'    => slicewp_mysql_gmdate(),
            'visit_id'         => ( ! empty( $commission_data['visit_id'] ) ? absint( $commission_data['visit_id'] ) : 0 ),
            'customer_id'      => ( ! empty( $commission_data['customer_id'] ) ? absint( $commission_data['customer_id'] ) : 0 ),
            'type'             => 'inherit',
            'amount'           => slicewp_sanitize_amount( $child_commission_amount ),
            'parent_id'        => absint( $commission_id )
        );

        // Insert commission.
        slicewp_insert_commission( $child_commission_data );

    }

}
add_action( 'slicewp_insert_commission', 'slicewp_on_commission_insert_reward_inherited_commissions', 25, 2 );


/**
 * Calculates the commission amount for the "inherit" commission type, based on the passed affiliate level.
 * 
 * @param float $commission_amount
 * @param float $base_amount
 * @param array $args
 * 
 * @return float
 * 
 */
function slicewp_calculate_commission_amount_inherit( $commission_amount, $base_amount, $args ) {

    if ( $args['type'] != 'inherit' ) {
        return $commission_amount;
    }

    if ( ! isset( $args['affiliate_level'] ) ) {
        return $commission_amount;
    }

    // Get affiliate levels.
    $affiliate_levels = slicewp_get_setting( 'affiliate_levels', array() );

    $level = $args['affiliate_level'] - 1;

    if ( empty( $affiliate_levels[$level] ) ) {
        return $commission_amount;
    }

    $rate 	   = ( isset( $affiliate_levels[$level]['commission_rate'] ) ? $affiliate_levels[$level]['commission_rate'] : '' );
    $rate_type = ( isset( $affiliate_levels[$level]['commission_rate_type'] ) ? $affiliate_levels[$level]['commission_rate_type'] : '' );

    if ( ! is_numeric( $rate ) || empty( $rate_type ) ) {
        return $commission_amount;
    }

    $commission_amount = ( $rate_type == 'percentage' ? round( ( $base_amount * $rate / 100 ), 2 ) : $rate );

    return $commission_amount;

}
add_filter( 'slicewp_calculate_commission_amount', 'slicewp_calculate_commission_amount_inherit', 20, 3 );


/**
 * Registers the "parent_id" column in the affiliates CSV export file.
 *
 * @param array $cols
 *
 * @return $cols
 *
 */
function slicewp_export_csv_cols_affiliates_parent_id( $cols ) {

	$cols['parent_id'] = __( 'Parent Affiliate ID', 'slicewp' );

	return $cols;

}
add_filter( 'slicewp_export_csv_cols_affiliates', 'slicewp_export_csv_cols_affiliates_parent_id', 25 );


/**
 * Adds the value for the "parent_id" column in the affiliates CSV export item data.
 *
 * @param array 			$item_data
 * @param SliceWP_Affiliate $affiliate
 *
 * @return array
 *
 */
function slicewp_export_get_data_item_affiliates_parent_id( $item_data, $affiliate ) {

	$item_data['parent_id'] = $affiliate->get( 'parent_id' );

	return $item_data;

}
add_filter( 'slicewp_export_get_data_item_affiliates', 'slicewp_export_get_data_item_affiliates_parent_id', 25, 2 );