<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Register the payout requests email notifications that is sent to administrators.
 *
 * @param array $email_notifications
 *
 * @return array
 *
 */
function slicewp_email_notifications_admin_payment_request( $email_notifications = array() ) {

	// Prepare notification data.
	$notification = array(
		'name' 		  => __( 'Payment Requested', 'slicewp' ),
		'description' => __( 'The administrator will receive an email when an affiliate requests a payout.', 'slicewp' ),
		'recipient'   => 'admin',
        'merge_tags'  => array()
    );

    // Add merge tags.
	$merge_tags = new SliceWP_Merge_Tags();

	foreach ( $merge_tags->get_tags() as $tag_slug => $tag_data ) {

		if ( empty( $tag_data['category'] ) || in_array( $tag_data['category'], array( 'affiliate', 'payment', 'general' ) ) ) {
			$notification['merge_tags'][] = $tag_slug;
		}

	}

    // Register notification.
    $email_notifications['admin_payment_request'] = $notification;
	
	return $email_notifications;

}
add_filter( 'slicewp_available_email_notification', 'slicewp_email_notifications_admin_payment_request', 25 );


/**
 * Register the payout requests email notifications that is sent to affiliates.
 *
 * @param array $email_notifications
 *
 * @return array
 *
 */
function slicewp_email_notifications_payment_request_affiliate( $email_notifications = array() ) {

    // Prepare notification data.
	$notification = array(
		'name' 		  => __( 'Payment Requested', 'slicewp' ),
		'description' => __( 'The administrator will receive an email when an affiliate requests a payout.', 'slicewp' ),
		'recipient'   => 'affiliate',
        'merge_tags'  => array()
    );

    // Add merge tags.
	$merge_tags = new SliceWP_Merge_Tags();

	foreach ( $merge_tags->get_tags() as $tag_slug => $tag_data ) {

		if ( empty( $tag_data['category'] ) || in_array( $tag_data['category'], array( 'affiliate', 'payment', 'general' ) ) ) {
			$notification['merge_tags'][] = $tag_slug;
		}

	}

    // Register notification.
    $email_notifications['affiliate_payment_request'] = $notification;
	
	return $email_notifications;

}
add_filter( 'slicewp_available_email_notification', 'slicewp_email_notifications_payment_request_affiliate', 50 );


/**
 * Send an email notification to the admininstrators when a new "requested" payment is created.
 *
 * @param int   $payment_id
 * @param array $payment_data
 *
 */
function slicewp_send_email_notification_admin_payment_request( $payment_id = 0, $payment_data = array() ) {

	if ( empty( $payment_id ) ) {
        return;
    }

    if ( empty( $payment_data['status'] ) || $payment_data['status'] != 'requested' ) {
        return;
    }

	// Get the affiliate ID attached to the payment.
	$affiliate_id = absint( $payment_data['affiliate_id'] );

	// Verify if email notification sending is enabled.
	$notification_settings = slicewp_get_email_notification_settings( 'admin_payment_request' );

	if ( empty( $notification_settings['enabled'] ) ) {
        return;
    }
	
	// Verify if the email notification subject and content are filled in.
	if ( empty( $notification_settings['subject'] ) || empty( $notification_settings['content'] ) ) {
        return;
    }
	
	// Verify that admin emails are not empty.
	$admin_emails = slicewp_get_setting( 'admin_emails' );

	if ( empty( $admin_emails ) ) {
        return;
    }

	// Put the admin emails in an array.
	$admin_emails = array_filter( array_map( 'trim', explode( ',', $admin_emails ) ) );

	// Remove array items that are not email addresses.
	if ( ! empty( $admin_emails ) ) {

		foreach ( $admin_emails as $key => $value ) {
			if ( ! is_email( $value ) ) {
                unset( $admin_emails[$key] );
            }
		}

		$admin_emails = array_values( $admin_emails );

	}

	if ( empty( $admin_emails ) ) {
        return;
    }

	// Prepare the email subject and content.
	$email_subject = ( ! empty( $notification_settings['subject'] ) ? sanitize_text_field( $notification_settings['subject'] ) : '' );
	$email_content = ( ! empty( $notification_settings['content'] ) ? $notification_settings['content'] : '' );

	// Get the Affiliate and Payment objects.
	$affiliate = slicewp_get_affiliate( $affiliate_id );
	$payment   = slicewp_get_payment( $payment_id );

	// Replace the tags with data.
	$merge_tags = new SliceWP_Merge_Tags();
	$merge_tags->set_data( 'affiliate', $affiliate );
	$merge_tags->set_data( 'payment', $payment );

	$email_subject = $merge_tags->replace_tags( $email_subject );
	$email_content = $merge_tags->replace_tags( $email_content );

	// Send the email.
	slicewp_wp_email( $admin_emails, $email_subject, $email_content );

}
add_action( 'slicewp_insert_payment', 'slicewp_send_email_notification_admin_payment_request', 20, 2 );


/**
 * Send an email notification to the affiliate when a new "requested" payment is created.
 *
 * @param int   $payment_id
 * @param array $payment_data
 *
 */
function slicewp_send_email_notification_affiliate_payment_request( $payment_id = 0, $payment_data = array() ) {

	if ( empty( $payment_id ) ) {
        return;
    }

    if ( empty( $payment_data['status'] ) || $payment_data['status'] != 'requested' ) {
        return;
    }

	// Get the affiliate ID attached to the payment.
	$affiliate_id = absint( $payment_data['affiliate_id'] );

	// Verify if email notification sending is enabled.
	$notification_settings = slicewp_get_email_notification_settings( 'affiliate_payment_request' );

	if ( empty( $notification_settings['enabled'] ) ) {
        return;
    }
	
	// Verify if the email notification subject and content are filled in.
	if ( empty( $notification_settings['subject'] ) || empty( $notification_settings['content'] ) ) {
        return;
    }
	
	// Get the affiliate email address.
	$affiliate = slicewp_get_affiliate( $affiliate_id );
	$payment   = slicewp_get_payment( $payment_id );

	$user      = get_user_by( 'id', $affiliate->get('user_id') );

	if ( empty( $user->user_email ) ) {
        return;
    }

	// Prepare the email Content
	$email_subject = ( ! empty( $notification_settings['subject'] ) ? sanitize_text_field( $notification_settings['subject'] ) : '' );
	$email_content = ( ! empty( $notification_settings['content'] ) ? $notification_settings['content'] : '' );

	// Replace the tags with data
	$merge_tags = new SliceWP_Merge_Tags();
	$merge_tags->set_data( 'affiliate', $affiliate );
	$merge_tags->set_data( 'payment', $payment );

	$email_subject = $merge_tags->replace_tags( $email_subject );
	$email_content = $merge_tags->replace_tags( $email_content );

	slicewp_wp_email( $user->user_email, $email_subject, $email_content );

}
add_action( 'slicewp_insert_payment', 'slicewp_send_email_notification_affiliate_payment_request', 20, 2 );