<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * AJAX callback that handles a payout request.
 * 
 */
function slicewp_action_ajax_create_payout_request() {

    // Check for nonce.
    if ( empty( $_POST['slicewp_token'] ) || ! wp_verify_nonce( $_POST['slicewp_token'], 'slicewp_create_payout_request' ) ) {
		wp_die( 0 );
	}

    // Check that affiliate is the correct one.
    if ( empty( $_POST['affiliate_id'] ) || absint( $_POST['affiliate_id'] ) != slicewp_get_current_affiliate_id() ) {
        wp_die( 0 );
    }

    // Set the affiliate ID.
    $affiliate_id = absint( $_POST['affiliate_id'] );

    // Get the general settings.
    $affiliate_payout_method = slicewp_get_affiliate_payout_method( $affiliate_id );
    $payout_methods          = slicewp_get_payout_methods();

    $require_invoice = ( ! empty( slicewp_get_setting( 'payout_request_require_invoice', false ) ) && ! empty( $payout_methods[$affiliate_payout_method]['supports'] ) && in_array( 'payout_request_invoice', $payout_methods[$affiliate_payout_method]['supports'] ) );
    $grace_period    = absint( slicewp_get_setting( 'commissions_grace_period', 0 ) );
    $minimum_amount  = absint( slicewp_get_setting( 'payments_minimum_amount', 0 ) );


    // Check if the affiliate has a pending payout request.
    $payments = slicewp_get_payments( array( 'affiliate_id' => $affiliate_id, 'status' => 'requested' ), true );

    if ( ! empty( $payments ) ) {

        wp_send_json_error( array( 'message' => __( 'You already have a pending payout request.', 'slicewp' ) ) );

    }

    // Check for invoice.
    if ( ! empty( $require_invoice ) ) {

        if ( empty( $_FILES['invoice'] ) ) {
            wp_send_json_error( array( 'message' => __( 'Please upload an invoice.', 'slicewp' ) ) );
        }

        if ( empty( $_FILES['invoice']['type'] ) || $_FILES['invoice']['type'] != 'application/pdf' ) {
            wp_send_json_error( array( 'message' => __( 'Please make sure the invoice is a PDF document.', 'slicewp' ) ) );
        }

    }

    // Set up to date.
    $date_max = new DateTime( date( 'Y-m-d' ) . ' 23:59:59' );

    // Take into account the grace period.
	if ( ! empty( $grace_period ) ) {

		$date_grace_end = new DateTime( date( 'Y-m-d' ) . ' 23:59:59');
		$date_grace_end = $date_grace_end->modify( '-' . $grace_period . ' day' );

		if ( $date_max > $date_grace_end ) {

			$date_max = $date_grace_end;

		}

	}

	// Prepare the arguments to read the commissions.
	$commission_args = array(
		'number'	   => -1,
		'status'	   => 'unpaid',
		'date_min' 	   => '',
		'date_max' 	   => ( ! empty( $date_max ) ? get_gmt_from_date( $date_max->format( 'Y-m-d H:i:s' ) ) : '' ),
		'affiliate_id' => slicewp_get_current_affiliate_id(),
		'payment_id'   => 0
	);

    $commissions = slicewp_get_commissions( $commission_args );

    // Bail if the affiliate doesn't have any commissions.
    if ( empty( $commissions ) ) {

        wp_send_json_error( array( 'message' => __( "You don't have any payable commissions.", 'slicewp' ) ) );

    }

    // Check for commissions to be at least the payments minimum.
    if ( array_sum( array_column( array_map( 'slicewp_object_to_array', $commissions ), 'amount' ) ) < $minimum_amount ) {

        wp_send_json_error( array( 'message' => __( "You're not yet eligible for a payout.", 'slicewp' ) ) );

    }

    // Prepare the payment data.
    $payment_data = array(
        'affiliate_id'	 	 => absint( $_POST['affiliate_id'] ),
        'amount'		 	 => slicewp_sanitize_amount( array_sum( array_column( array_map( 'slicewp_object_to_array', $commissions ), 'amount' ) ) ),
        'currency'		 	 => sanitize_text_field( slicewp_get_setting( 'active_currency', 'USD' ) ),
        'payout_method'	 	 => slicewp_get_affiliate_payout_method( absint( $_POST['affiliate_id'] ) ),
        'date_created'   	 => slicewp_mysql_gmdate(),
        'date_modified'  	 => slicewp_mysql_gmdate(),
        'status'		 	 => 'requested',
        'commission_ids' 	 => sanitize_text_field( implode( ',', array_column( array_map( 'slicewp_object_to_array', $commissions ), 'id' ) ) ),
        'payout_id'		 	 => 0,
        'originator_user_id' => get_current_user_id()
    );
    
    $payment_id = slicewp_insert_payment( $payment_data );

    // If the payment could not be created, return an error.
    if ( ! $payment_id ) {

        wp_send_json_error( array( 'message' => __( "Something went wrong. The request could not be made.", 'slicewp' ) ) );

    }

    // Payments is good.
    // Add the payment ID to each commission.
    foreach ( array_column( array_map( 'slicewp_object_to_array', $commissions ), 'id' ) as $commission_id ) {

        slicewp_update_commission( $commission_id, array( 'date_modified' => slicewp_mysql_gmdate(), 'payment_id' => $payment_id ) );

    }

    // Upload invoice if it exists.
    if ( $require_invoice ) {

        // Include needed dependencies.
        require_once( ABSPATH . 'wp-admin/includes/media.php' );
        require_once( ABSPATH . 'wp-admin/includes/file.php' );
        require_once( ABSPATH . 'wp-admin/includes/image.php' );

        $attachment_id = media_handle_sideload( $_FILES['invoice'] );

        if ( ! is_wp_error( $attachment_id ) ) {
            slicewp_add_payment_meta( $payment_id, '_invoice_attachment_id', absint( $attachment_id ) );
        }

    }

    ob_start();

    $payment = slicewp_get_payment( $payment_id );

    include 'views/view-card-payment-requested.php';

    $view = ob_get_contents();

    ob_end_clean();

    // Send success.
    wp_send_json_success( array( 'html' => $view ) );

}
add_action( 'wp_ajax_slicewp_action_ajax_create_payout_request', 'slicewp_action_ajax_create_payout_request' );


/**
 * AJAX callback that handles the cancellation of a payout request.
 * 
 */
function slicewp_action_ajax_cancel_payout_request() {

    // Check for nonce.
    if ( empty( $_POST['slicewp_token'] ) || ! wp_verify_nonce( $_POST['slicewp_token'], 'slicewp_cancel_payout_request' ) ) {
		wp_die( 0 );
	}

    // Check that affiliate is the correct one.
    if ( empty( $_POST['affiliate_id'] ) || absint( $_POST['affiliate_id'] ) != slicewp_get_current_affiliate_id() ) {
        wp_die( 0 );
    }

    $affiliate_id = absint( $_POST['affiliate_id'] );

    // Check for payment.
    if ( empty( $_POST['payment_id'] ) ) {
        wp_die( 0 );
    }

    $payment_id = absint( $_POST['payment_id'] );
    $payment    = slicewp_get_payment( $payment_id );

    if ( is_null( $payment ) || $payment->get( 'affiliate_id' ) != $affiliate_id || $payment->get( 'status' ) != 'requested' ) {
        wp_die( 0 );
    }

    // If all is good by now, delete the payment.
    $deleted = slicewp_delete_payment( $payment_id );

    if ( ! $deleted ) {
        wp_die( 0 );
    }

    // Remove the payment's ID from each commission that has it attached.
	$commission_ids = slicewp_get_commissions( array( 'fields' => 'id', 'number' => -1, 'payment_id' => absint( $payment_id ) ) );

	if ( ! empty( $commission_ids ) ) {

		foreach ( $commission_ids as $commission_id ) {
			
			slicewp_update_commission( $commission_id, array( 'payment_id' => 0 ) );

		}

	}

    // Send success.
    $payment_preview = slicewp_generate_affiliate_payment_request_preview( $affiliate_id );

    if ( empty( $payment_preview['amount'] ) || $payment_preview['amount'] < absint( slicewp_get_setting( 'payments_minimum_amount', 0 ) ) ) {

        wp_send_json_success();

    } else {

        ob_start();

        include 'views/view-card-request-payout.php';

        $view = ob_get_contents();

        ob_end_clean();

        wp_send_json_success( array( 'html' => $view ) );
        
    }

}
add_action( 'wp_ajax_slicewp_action_ajax_cancel_payout_request', 'slicewp_action_ajax_cancel_payout_request' );