<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Outputter for the bonuses shown in the affiliate account.
 * 
 */
class SliceWP_Outputter_Affiliate_Bonuses {

    /**
     * The currently logged-in affiliate.
     * 
     * @access protected
     * @var    int
     * 
     */
    protected $affiliate_id = 0;

    /**
     * The bonuses available to the affiliate.
     * 
     * @access protected
     * @var    array
     * 
     */
    protected $bonuses = array();

    
    /**
     * Constructor.
     * 
     */
    public function __construct() {

        $this->affiliate_id = slicewp_get_current_affiliate_id();

        $this->prepare_bonuses();

    }


    /**
     * Builds the bonuses that are available to the currently logged-in affiliate.
     * 
     */
    protected function prepare_bonuses() {

        $bonuses = array_filter( (array)get_option( 'slicewp_performance_bonuses', array() ) );

        if ( empty( $bonuses ) ) {
            return;
        }

        // Filter out inactive bonuses.
        foreach ( $bonuses as $bonus_id => $bonus ) {

            if ( $bonus['status'] != 'active' ) {
                unset( $bonuses[$bonus_id] );
            }

        }

        // Filter out bonuses that are not meant for this affiliate.
        $affiliate_id       = $this->affiliate_id;
        $affiliate_group_id = slicewp_get_affiliate_group_id( $affiliate_id );

        foreach ( $bonuses as $bonus_id => $bonus ) {

            if ( empty( $bonus['availability_type'] ) ) {
                continue;
            }

            if ( $bonus['availability_type'] == 'exclude' && ( in_array( $affiliate_id, $bonus['availability_affiliate_id'] ) || in_array( $affiliate_group_id, $bonus['availability_affiliate_group_id'] ) ) ) {
                unset( $bonuses[$bonus_id] );
            }

            if ( $bonus['availability_type'] == 'include' && ( ! in_array( $affiliate_id, $bonus['availability_affiliate_id'] ) && ! in_array( $affiliate_group_id, $bonus['availability_affiliate_group_id'] ) ) ) {
                unset( $bonuses[$bonus_id] );
            }

        }

        $this->bonuses = $bonuses;
        
    }


    /**
     * Returns the progress of the given affiliate, for the given bonus.
     * 
     * @param int $bonus_id
     * @param int $affiliate_id
     * 
     * @return null|array
     * 
     */
    protected function get_bonus_progress( $bonus_id, $affiliate_id ) {

        return slicewp_get_affiliate_bonus_progress( $affiliate_id, $bonus_id );

    }


    /**
     * Outputs the bonuses available to the affiliate.
     * 
     */
    public function output() {

        ?>

            <div class="slicewp-grid slicewp-columns-2">

                <?php foreach ( $this->bonuses as $bonus ): ?>

                    <?php $progress = $this->get_bonus_progress( $bonus['id'], $this->affiliate_id ); ?>

                    <div class="slicewp-card slicewp-card-bonus <?php echo ( $progress[$bonus['target_type']]['percentage'] == 100 ? 'slicewp-completed' : '' ); ?>">
                        
                        <div class="slicewp-card-inner">

                            <div class="slicewp-bonus-reward-wrapper">

                                <div class="slicewp-bonus-icon">
                                    <?php

                                        switch ( $bonus['reward_type'] ) {

                                            case 'commission':
                                                $svg = 'outline-cash';
                                                break;

                                            case 'store_credit':
                                                $svg = 'outline-wallet';
                                                break;

                                            case 'coupon':
                                                $svg = 'outline-ticket';
                                                break;

                                            default:
                                                $svg = 'outline-gift';
                                                break;

                                        }

                                        echo slicewp_get_svg( $svg );
                                    
                                    ?>

                                    <?php if ( $progress[$bonus['target_type']]['percentage'] == 100 ): ?>
                                        <div class="slicewp-completed-badge">
                                            <?php echo slicewp_get_svg( 'solid-check-badge' ); ?>
                                        </div>
                                    <?php endif; ?>

                                </div>

                                <div>

                                    <div class="slicewp-bonus-reward">
                                        <?php
                                            switch ( $bonus['reward_type'] ) {

                                                case 'commission':
                                                    echo slicewp_format_amount( $bonus['reward_commission_amount'], slicewp_get_setting( 'active_currency', 'USD' ) );
                                                    break;

                                                case 'store_credit':
                                                    echo slicewp_format_amount( $bonus['reward_store_credit_amount'], slicewp_get_setting( 'active_currency', 'USD' ) );
                                                    break;

                                                default:
                                                    break;

                                            }
                                        ?>
                                    </div>

                                    <div class="slicewp-bonus-reward-type">
                                        <?php
                                            switch ( $bonus['reward_type'] ) {

                                                case 'commission':
                                                    echo __( 'Bonus Commission', 'slicewp' );
                                                    break;

                                                case 'store_credit':
                                                    echo __( 'Store Credit', 'slicewp' );
                                                    break;

                                                case 'coupon':
                                                    echo __( 'Coupon Code', 'slicewp' );
                                                    break;

                                                default:
                                                    break;

                                            }
                                        ?>
                                    </div>
                                </div>

                            </div>

                            <?php
                                
                                switch ( $bonus['reward_type'] ) {

                                    case 'commission':
                                        $reward_string = sprintf( __( 'a %s bonus commission', 'slicewp' ), slicewp_format_amount( $bonus['reward_commission_amount'], slicewp_get_setting( 'active_currency', 'USD' ) ) );
                                        break;

                                    case 'store_credit':
                                        $reward_string = sprintf( __( '%s in store credit', 'slicewp' ), slicewp_format_amount( $bonus['reward_store_credit_amount'], slicewp_get_setting( 'active_currency', 'USD' ) ) );
                                        break;

                                    default:
                                        break;

                                }

                                if ( $bonus['target_type'] == 'referred_orders_count' ) {

                                    echo '<p>' . sprintf( __( 'Refer %s orders and earn %s.', 'slicewp' ), ( $bonus['target'] ), $reward_string ) . '</p>';

                                }

                                if ( $bonus['target_type'] == 'referred_orders_totals' ) {

                                    echo '<p>' . sprintf( __( 'Refer orders totalling %s and earn %s.', 'slicewp' ), slicewp_format_amount( $bonus['target'], slicewp_get_setting( 'active_currency', 'USD' ) ), $reward_string ) . '</p>';

                                }

                                if ( $bonus['target_type'] == 'earned_commissions_totals' ) {

                                    echo '<p>' . sprintf( __( 'Reach a total of %s in earned commissions and get %s.', 'slicewp' ), slicewp_format_amount( $bonus['target'], slicewp_get_setting( 'active_currency', 'USD' ) ), $reward_string ) . '</p>';

                                }

                            ?>
                            
                        </div>

                        <div class="slicewp-card-footer" style="background: transparent;">

                            <div class="slicewp-progressbar"><div class="slicewp-progressbar-fill" style="width: <?php echo $progress[$bonus['target_type']]['percentage']; ?>%;"></div></div>

                            <div class="slicewp-bonus-progress-wrapper">
                                <div class="slicewp-bonus-progress"><?php echo ( $progress[$bonus['target_type']]['percentage'] == 100 ? $progress[$bonus['target_type']]['target'] : $progress[$bonus['target_type']]['current'] ) . ' / ' . $progress[$bonus['target_type']]['target']; ?></div>

                                <?php if ( ! empty( $progress[$bonus['target_type']]['date_end'] ) && $progress[$bonus['target_type']]['percentage'] != 100 ): ?>

                                    <?php // @todo - Dynamic timer: https://www.w3schools.com/howto/howto_js_countdown.asp ?>
                                    <div class="slicewp-bonus-progress-timer">
                                        <svg xmlns="http://www.w3.org/2000/svg" class="h-5 w-5" viewBox="0 0 20 20" fill="currentColor"><path fill-rule="evenodd" d="M10 18a8 8 0 100-16 8 8 0 000 16zm1-12a1 1 0 10-2 0v4a1 1 0 00.293.707l2.828 2.829a1 1 0 101.415-1.415L11 9.586V6z" clip-rule="evenodd" /></svg>
                                        <div>
                                            <?php

                                                $now    = new DateTime( 'now', new DateTimeZone( 'UTC' ) );
                                                $future = new DateTime( $progress[$bonus['target_type']]['date_end'] );

                                                $interval = $future->diff( $now );

                                                echo $interval->format( sprintf( __( "%sd %sh %sm", 'slicewp' ), '<span class="slicewp-date-format-day">%a</span>', '<span class="slicewp-date-format-hour">%h</span>', '<span class="slicewp-date-format-minute">%i</span>' ) );

                                            ?>
                                        </div>
                                    </div>

                                <?php endif; ?>

                                <?php if ( $progress[$bonus['target_type']]['percentage'] == 100 ): ?>

                                    <strong><?php echo __( 'Completed!', 'slicewp' ); ?></strong>

                                <?php endif; ?>

                            </div>

                        </div>

                    </div>

                <?php endforeach; ?>

            </div>

        <?php

    }

}