<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Registers the "recurring" commission type.
 *
 * @param array $commission_types
 *
 * @return array
 *
 */
function slicewp_register_commission_type_recurring( $commission_types ) {

	$commission_types['recurring'] = array(
		'label' 	 => __( 'Recurring', 'slicewp' ),
		'rate_types' => array( 'percentage', 'fixed_amount' )
	);

	return $commission_types;

}
add_filter( 'slicewp_register_commission_types', 'slicewp_register_commission_type_recurring' );


/**
 * Modifies the commission amount based on the group commission rates the affiliate is attached to.
 *
 * @param float $commission_amount
 * @param float $amount
 * @param array $args
 *
 * @return float
 *
 */
function slicewp_calculate_affiliate_group_recurring_commission_amount( $commission_amount, $amount, $args ) {

	if ( empty( $args['affiliate_id'] ) ) {
		return $commission_amount;
	}

	// Check for the recurring type.
    if ( $args['type'] != 'recurring' ) {
		return $commission_amount;
	}

	$affiliate_id = absint( $args['affiliate_id'] );

	// Check if the affiliate is in a group.
	$affiliate_group_id = slicewp_get_affiliate_group_id( $affiliate_id );

	if ( empty( $affiliate_group_id ) ) {
		return $commission_amount;
	}

	// If the affiliate group has custom recurring commissions enabled, overwrite the existing ones.
    if ( 'custom' == slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) ) {

        $rate_type = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_type_recurring', true );
        $rate      = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_recurring', true );

    }

	if ( ! is_numeric( $rate ) || empty( $rate_type ) ) {
        return $commission_amount;
    }

	// Calculate the recurring commission amount.
	$commission_amount = ( $rate_type == 'percentage' ? round( ( $amount * $rate / 100 ), 2 ) : $rate );

	return $commission_amount;

}
add_filter( 'slicewp_calculate_commission_amount', 'slicewp_calculate_affiliate_group_recurring_commission_amount', 35, 3 );


/**
 * Calculates the recurring commission amount for the affiliate.
 * 
 * @param float $commission_amount
 * @param float $amount
 * @param array $args
 * 
 * @return float $commission_amount
 * 
 */
function slicewp_calculate_affiliate_recurring_commission_amount( $commission_amount, $amount, $args ) {

	if ( empty( $args['affiliate_id'] ) ) {
		return $commission_amount;
	}

	// Check for the recurring type.
    if ( $args['type'] != 'recurring' ) {
		return $commission_amount;
	}

	$rate_type = '';
	$rate      = '';

    // If the affiliate has custom recurring commissions enabled, overwrite the existing product ones.
    if ( 'custom' == slicewp_get_affiliate_meta( $args['affiliate_id'], 'recurring_commissions', true ) ) {

        $rate_type = slicewp_get_affiliate_meta( $args['affiliate_id'], 'commission_rate_type_recurring', true );
        $rate      = slicewp_get_affiliate_meta( $args['affiliate_id'], 'commission_rate_recurring', true );

    }

    
    if ( ! is_numeric( $rate ) || empty( $rate_type ) ) {
        return $commission_amount;
    }

	// Calculate the recurring commission amount.
	$commission_amount = ( $rate_type == 'percentage' ? round( ( $amount * $rate / 100 ), 2 ) : $rate );

    // Return the commission amount.
    return $commission_amount;

}
add_filter( 'slicewp_calculate_commission_amount', 'slicewp_calculate_affiliate_recurring_commission_amount', 75, 3 );


/**
 * Returns the affiliate commission rate for an affiliate.
 *
 * @param array $affiliate_commission_rates
 * @param int 	$affiliate_id
 *
 * @return array
 *
 */
function slicewp_rc_get_affiliate_commission_rates( $affiliate_commission_rates, $affiliate_id ) {

	// Set the affiliate recurring rate to the values from settings.
	$affiliate_commission_rates['recurring']['rate']	  = slicewp_get_setting( 'commission_rate_recurring' );
	$affiliate_commission_rates['recurring']['rate_type'] = slicewp_get_setting( 'commission_rate_type_recurring' );

	// Get the recurring commissions settings for affiliate.
	$affiliate_recurring_commissions = slicewp_get_affiliate_meta( $affiliate_id, 'recurring_commissions', true );

	// If recurring commissions are disabled for this affiliate, remove the recurring rate values and return remaining.
	if ( $affiliate_recurring_commissions == 'disabled' ) {

		unset( $affiliate_commission_rates['recurring'] );
		return $affiliate_commission_rates;

	}

	// If the affiliate has a custom recurring rate, set it and return it.
	if ( $affiliate_recurring_commissions == 'custom' ) {

		// Get the affiliate custom recurring rate and rate type.
		$rate 	   = slicewp_get_affiliate_meta( $affiliate_id, 'commission_rate_recurring', true );
		$rate_type = slicewp_get_affiliate_meta( $affiliate_id, 'commission_rate_type_recurring', true );

		if ( is_numeric( $rate ) && ! empty( $rate_type ) ) {

			$affiliate_commission_rates['recurring']['rate']	  = $rate;
			$affiliate_commission_rates['recurring']['rate_type'] = $rate_type;

			return $affiliate_commission_rates;

		}

	}

	// Check for affiliate graoup.
	$affiliate_group_id = slicewp_get_affiliate_group_id( $affiliate_id );

	if ( ! empty( $affiliate_group_id ) ) {
	
		// If there are no affiliate recurring commissions settings, check the ones on the group the affiliate is a part of.
		$group_recurring_commissions = slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true );

		// If recurring commissions are disabled for the group, unset recurring rates and return.
		if ( $group_recurring_commissions == 'disabled' ) {

			if ( isset( $affiliate_commission_rates['recurring'] ) ) {
				unset( $affiliate_commission_rates['recurring'] );
			}
			
			return $affiliate_commission_rates;

		}

		// If there are custom rates set for the group, set them and return them.
		if ( $group_recurring_commissions == 'custom' ) {

			$rate      = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_recurring', true );
			$rate_type = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_type_recurring', true );

			// Save the custom rate and rate type.
			if ( is_numeric( $rate ) && ! empty( $rate_type ) ) {

				$affiliate_commission_rates['recurring']['rate']	  = $rate;
				$affiliate_commission_rates['recurring']['rate_type'] = $rate_type;

				return $affiliate_commission_rates;

			}

		}

	}

	// If recurring commissions are disabled sitewide and the group and affiliate ones are site default, unset the recurring rates.
	if ( empty( slicewp_get_setting( 'recurring_commissions' ) ) && empty( $group_recurring_commissions ) && empty( $affiliate_recurring_commissions ) ) {

		if ( isset( $affiliate_commission_rates['recurring'] ) ) {
			unset( $affiliate_commission_rates['recurring'] );
		}

	}

	// Return the filtered value.
	return $affiliate_commission_rates;

}
add_filter( 'slicewp_affiliate_commission_rates', 'slicewp_rc_get_affiliate_commission_rates', 20, 2 );