<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Recurring commissions Paid Member Subscriptions integration class.
 *
 */
class SliceWP_Integration_Recurring_Commissions_Paid_Member_Subscriptions extends SliceWP_Integration_Recurring_Commissions {

	/**
     * Constructor.
     *
     */
    public function __construct() {

        $this->origin = 'pms';

        // Add the "recurring" commission type to Paid Member Subscriptions supports array.
		add_filter( 'slicewp_integration_supports_pms', array( $this, 'add_integration_supports' ) );

		// Adds a new recurring commission to the database.
        add_action( 'pms_payment_insert', array( $this, 'insert_recurring_commission' ), 10 );
		add_action( 'pms_payment_update', array( $this, 'insert_recurring_commission' ), 10 );

		// Add recurring commission rate settings in the subscription plan page.
		add_action( 'slicewp_pms_metabox_commission_settings_bottom', array( $this, 'add_subscription_plan_recurring_commissions_settings' ), 15 );

		// Save the subscription plan recurring commissions settings.
		add_action( 'pms_save_meta_box_pms-subscription', array( $this, 'save_subscription_plan_recurring_commissions_settings' ) );

    }


    /**
     * Add the "recurring" commission type to Paid Member Subscriptions supports array.
     *
     * @param array $supports
     *
     * @return array
     *
     */
    public function add_integration_supports( $supports ) {

        if ( empty( $supports['commission_types'] ) || ! is_array( $supports['commission_types'] ) ) {
			return $supports;
		}

        $supports['commission_types'][] = 'recurring';

        return $supports;

    }


	/**
     * Adds a new recurring commission to the database.
     *
     * @param int $payment_id
     *
     */
	public function insert_recurring_commission( $payment_id ) {

		$payment = pms_get_payment( $payment_id );

		// Make sure this is a recurring payment for a subscription.
		if ( empty( $payment->type ) ) {
			return;
		}

		if ( ! in_array( $payment->type, array( 'recurring_payment', 'subscription_recurring_payment', 'subscr_payment' ) ) ) {
			return;
		}

		// Make sure the payment's status is "completed".
		if ( empty( $payment->status ) || $payment->status != 'completed' ) {
			return;
		}

		// Make sure there isn't already a commission generated for this payment.
		$commissions_count = slicewp_get_commissions( array( 'type' => 'recurring', 'origin' => 'pms', 'reference' => $payment_id ), true );

		if ( ! empty( $commissions_count ) ) {
			return;
		}

		// Get the first order of the subscription.
		$parent_payment = $this->get_parent_payment( $payment_id );

		if ( is_null( $parent_payment ) ) {
			return;
		}

		// Get the first commission, generated for the first subscription payment.
        $parent_commission = slicewp_get_commissions( array( 'origin' => 'pms', 'reference' => $parent_payment->id, 'order' => 'ASC', 'parent_id' => 0 ) );
        $parent_commission = ( ! empty( $parent_commission ) ? $parent_commission[0] : null );

        // Bail if the commission is missing or is rejected
        if ( is_null( $parent_commission ) || $parent_commission->get( 'status' ) == 'rejected' ) {
            return;
        }

		// Bail if the affiliate doesn't exist or is inactive.
        $affiliate = slicewp_get_affiliate( absint( $parent_commission->get( 'affiliate_id' ) ) );

        if ( is_null( $affiliate ) || $affiliate->get( 'status' ) != 'active' ) {

            slicewp_add_log( "PMS: Recurring commission was not inserted because the affiliate either doesn't exist or is not active." );
            return;

        }

		$recurring_commissions_affiliate = slicewp_get_affiliate_meta( $affiliate->get( 'id' ), 'recurring_commissions', true );

        // Bail if the affiliate has recurring commissions disabled.
        if ( 'disabled' == $recurring_commissions_affiliate ) {

            slicewp_add_log( 'PMS: Recurring commission was not inserted because the affiliate has recurring commissions disabled.' );
            return;

        }

		// Get affiliate group settings.
        $affiliate_group_id          = slicewp_get_affiliate_group_id( $affiliate->get( 'id' ) );
        $recurring_commissions_group = ( ! empty( $affiliate_group_id ) ? slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) : '' );

		$subscription_plan_id = $parent_payment->subscription_id;

        // Verify if commissions are disabled for this product.
        if ( get_post_meta( $subscription_plan_id, 'slicewp_disable_commissions', true ) ) {
            return;
        }

		$recurring_commissions_product = get_post_meta( $subscription_plan_id, 'slicewp_recurring_commissions', true );

        // Verify if product has recurring commissions disabled.
        if ( 'disabled' == $recurring_commissions_product ) {
            
            slicewp_add_log( 'PMS: Recurring commission was not inserted for this subscription plan because it has recurring commissions disabled. Subscription plan: ' . $subscription_plan_id );
            return;

        }

        if ( 'disabled' == $recurring_commissions_group && $recurring_commissions_affiliate != 'custom' && $recurring_commissions_product != 'custom' ) {
            return;
        }

		// Verify if there are custom settings for the product/affiliate.
        // If there aren't and recurring commissions aren't enabled sitewide, we don't generate commissions.
        if ( empty( slicewp_get_setting( 'recurring_commissions' ) ) && ! $this->has_custom_settings( array( 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $subscription_plan_id ) ) ) {
            return;
        }

		// Verify if the recurrence limit has been reached.
        if ( $this->recurrence_limit_reached( array( 'parent_id' => $parent_commission->get( 'id' ), 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $subscription_plan_id ) ) ) {

            slicewp_add_log( 'PMS: Recurring commission was not inserted because the recurrence limit has been reached. Subscription plan: ' . $subscription_plan_id );
            return;

        }

		// Get the currency.
		$payments_settings = get_option( 'pms_payments_settings', array() );
		$currency		   = ( ! empty( $payments_settings['currency'] ) ? $payments_settings['currency'] : 'USD' );

		// Get the order amount.
        $amount = $payment->amount;
        
        // Exclude tax.
        if ( slicewp_get_setting( 'exclude_tax', false ) ) {

			if ( function_exists( 'pms_in_tax_determine_tax_breakdown' ) ) {

				$tax 	= pms_in_tax_determine_tax_breakdown( $payment->id );
				$amount = ( empty( $tax ) ? $amount : $tax['subtotal'] );

			}

        }

        // Calculate the commission amount for the entire order.
        $args = array(
            'origin'       => 'pms',
            'type'         => 'recurring',
            'affiliate_id' => $parent_commission->get( 'affiliate_id' ),
            'product_id'   => $subscription_plan_id
        );

		if ( function_exists( 'slicewp_maybe_convert_amount' ) ) {
            $amount = slicewp_maybe_convert_amount( $amount, $currency, slicewp_get_setting( 'active_currency', 'USD' ) );
        }

		$commission_amount = slicewp_calculate_commission_amount( $amount, $args );

        // Check that the commission amount is not zero.
        if ( $commission_amount == 0 && empty( slicewp_get_setting( 'zero_amount_commissions' ) ) ) {

            slicewp_add_log( 'PMS: Recurring commission was not inserted because the commission amount is zero.' );
            return;

        }

		// Prepare commission data.
        $commission_data = array(
            'affiliate_id'		=> $parent_commission->get( 'affiliate_id' ),
            'parent_id'			=> $parent_commission->get( 'id' ),
            'visit_id'			=> 0,
            'date_created'		=> slicewp_mysql_gmdate(),
            'date_modified'		=> slicewp_mysql_gmdate(),
            'type'				=> 'recurring',
            'status'			=> 'unpaid',
            'reference'			=> $payment->id,
			'reference_amount'	=> slicewp_sanitize_amount( $payment->amount ),
            'origin'			=> 'pms',
            'amount'			=> slicewp_sanitize_amount( $commission_amount ),
            'currency'			=> slicewp_get_setting( 'active_currency', 'USD' ),
            'customer_id'       => ( ! empty( $parent_commission->get( 'customer_id' ) ) ? absint( $parent_commission->get( 'customer_id' ) ) : 0 )
        );

		// Insert the commission.
        $commission_id = slicewp_insert_commission( $commission_data );

        if ( ! empty( $commission_id ) ) {

            slicewp_add_log( sprintf( 'PMS: Recurring commission #%s has been successfully inserted.', $commission_id ) );
            
        } else {

            slicewp_add_log( 'PMS: Recurring commission could not be inserted due to an unexpected error.' );
            
        }

	}


    /**
	 * Adds recurring commission rate settings in the subscription plan page.
	 *
	 * @param int $subscription_plan_id
	 *
	 */
	function add_subscription_plan_recurring_commissions_settings() {

		global $post;

		// Get recurring commissions status.
		$recurring_commissions = get_post_meta( $post->ID, 'slicewp_recurring_commissions', true );

		// Get the recurring rate and rate type.
	    $commission_rate_type = get_post_meta( $post->ID, 'slicewp_commission_rate_type_recurring', true );
	    $commission_rate 	  = get_post_meta( $post->ID, 'slicewp_commission_rate_recurring', true );

	    // Get the recurrence.
	    $recurrence       = get_post_meta( $post->ID, 'slicewp_commissions_recurrence', true );
	    $recurrence_limit = get_post_meta( $post->ID, 'slicewp_commissions_recurrence_limit', true );

		?>

		<div class="slicewp-options-group slicewp-hide-if-disabled-commissions">

			<div class="slicewp-option-field-wrapper pms-meta-box-field-wrapper cozmoslabs-form-field-wrapper">
				<label for="slicewp-recurring-commissions" class="pms-meta-box-field-label cozmoslabs-form-field-label"><?php echo __( 'Recurring Commissions', 'slicewp' ); ?></label>
				<select name="slicewp_recurring_commissions" id="slicewp-recurring-commissions" class="slicewp-option-field-recurring-commissions regular-text">
					<option value=""><?php echo __( 'Site Default', 'slicewp' ); ?></option>
					<option value="custom" <?php selected( $recurring_commissions, 'custom' ); ?>><?php echo __( 'Custom', 'slicewp' ); ?></option>
					<option value="disabled" <?php selected( $recurring_commissions, 'disabled' ); ?>><?php echo __( 'Disabled', 'slicewp' ); ?></option>
				</select>
			</div>

			<div class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate-type slicewp-commission-type-recurring pms-meta-box-field-wrapper cozmoslabs-form-field-wrapper">
				<label for="slicewp-commission-rate-type-recurring" class="pms-meta-box-field-label cozmoslabs-form-field-label"><?php echo __( 'Recurring Rate Type', 'slicewp' ); ?></label>
				<select name="slicewp_commission_rate_type_recurring" id="slicewp-commission-rate-type-recurring">
					<option value=""><?php echo __( 'Site Default', 'slicewp' ); ?></option>
					<?php foreach ( slicewp_get_commission_type_rate_types( 'recurring' ) as $rate_type => $rate_type_label ) : ?>
						<option value="<?php echo esc_attr( $rate_type ); ?>"<?php selected( $commission_rate_type, $rate_type ); ?>><?php echo esc_html( $rate_type_label ); ?></option>
					<?php endforeach; ?>
				</select>
				<p class="cozmoslabs-description cozmoslabs-description-align-right"><?php echo __( 'The selected commission rate type will be used to calculate the commission for this subscription plan.', 'slicewp' ); ?></p>
			</div>

			<div class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate slicewp-commission-type-recurring pms-meta-box-field-wrapper cozmoslabs-form-field-wrapper">
				<label for="slicewp-commission-rate-recurring" class="pms-meta-box-field-label cozmoslabs-form-field-label"><?php echo __( 'Recurring Rate', 'slicewp' ); ?></label>
				<input type="number" step="any" min="0" size="5" name="slicewp_commission_rate_recurring" value="<?php echo esc_attr( $commission_rate ); ?>" id="slicewp-commission-rate-recurring" />
				<p class="cozmoslabs-description cozmoslabs-description-align-right"><?php echo __( 'This rate will be used to calculate the commission for this subscription plan.', 'slicewp' ); ?></p>
			</div>

			<div class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence pms-meta-box-field-wrapper cozmoslabs-form-field-wrapper">
				<label for="slicewp-commissions-recurrence" class="pms-meta-box-field-label cozmoslabs-form-field-label"><?php echo __( 'Commissions Recurrence', 'slicewp' ); ?></label>
				<select name="slicewp_commissions_recurrence" id="slicewp-commissions-recurrence">
					<option value=""><?php echo __( 'Site Default', 'slicewp' ); ?></option>
					<option value="unlimited" <?php selected( $recurrence, 'unlimited' ); ?>><?php echo __( 'Unlimited', 'slicewp' ); ?></option>
					<option value="limited" <?php selected( $recurrence, 'limited' ); ?>><?php echo __( 'Limited', 'slicewp' ); ?></option>
				</select>
				<p class="cozmoslabs-description cozmoslabs-description-align-right"><?php echo __( 'Select whether the affiliate should be rewarded for all recurring payments for this subscription plan or for a limited number of recurring payments.', 'slicewp' ); ?></p>
			</div>

			<div class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence-limit pms-meta-box-field-wrapper cozmoslabs-form-field-wrapper">
		        <label for="slicewp-commissions-recurrence-limit" class="pms-meta-box-field-label cozmoslabs-form-field-label"><?php echo __( 'Recurrence Limit', 'slicewp' ); ?></label>
		        <input type="number" name="slicewp_commissions_recurrence_limit" value="<?php echo esc_attr( $recurrence_limit ); ?>" id="slicewp-commissions-recurrence-limit" style="min-width: 250px;" />
		        <p class="cozmoslabs-description cozmoslabs-description-align-right"><?php echo __( 'Set the maximum number of commissions an affiliate is rewarded for recurring payments made by this subscription plan.', 'slicewp' ); ?></p>
		    </div>

		</div>

		<?php

	}


	/**
	 * Saves the subscription plan recurring commissions settings.
	 *
	 * @param int $post_id - the ID of the subscription plan
	 *
	 */
	function save_subscription_plan_recurring_commissions_settings( $post_id ) {

		if ( wp_is_post_revision( $post_id ) || wp_is_post_autosave( $post_id ) ) {
			return;
		}

	    // Update disabled recurring commissions.
	    if ( ! empty( $_POST['slicewp_recurring_commissions'] ) ) {
			update_post_meta( $post_id, 'slicewp_recurring_commissions', sanitize_text_field( $_POST['slicewp_recurring_commissions'] ) );
		} else {
			delete_post_meta( $post_id, 'slicewp_recurring_commissions' );
		}

	    // Update recurring commission rate type.
	    if ( ! empty( $_POST['slicewp_commission_rate_type_recurring'] ) ) {
			update_post_meta( $post_id, 'slicewp_commission_rate_type_recurring', sanitize_text_field( $_POST['slicewp_commission_rate_type_recurring'] ) );
		} else {
			delete_post_meta( $post_id, 'slicewp_commission_rate_type_recurring' );
		}

	    // Update recurring commission rate.
	    if ( ! empty( $_POST['slicewp_commission_rate_recurring'] ) ) {
			update_post_meta( $post_id, 'slicewp_commission_rate_recurring', sanitize_text_field( $_POST['slicewp_commission_rate_recurring'] ) );
		} else {
			delete_post_meta( $post_id, 'slicewp_commission_rate_recurring' );
		}

	    // Update recurring commissions recurrence.
	    if ( ! empty( $_POST['slicewp_commissions_recurrence'] ) ) {
			update_post_meta( $post_id, 'slicewp_commissions_recurrence', sanitize_text_field( $_POST['slicewp_commissions_recurrence'] ) );
		} else {
			delete_post_meta( $post_id, 'slicewp_commissions_recurrence' );
		}

	    // Update recurring commissions recurrence limit.
	    if ( isset( $_POST['slicewp_commissions_recurrence_limit'] ) ) {
			update_post_meta( $post_id, 'slicewp_commissions_recurrence_limit', absint( $_POST['slicewp_commissions_recurrence_limit'] ) );
		} else {
			delete_post_meta( $post_id, 'slicewp_commissions_recurrence_limit' );
		}

	}


	/**
	 * Returns the initiating payment for the subscription.
	 * 
	 * @param PMS_Payment|int $payment
	 * 
	 * @return PMS_Payment|null
	 * 
	 */
	public function get_parent_payment( $payment ) {

		global $wpdb;

		// Get current payment.
        $payment = ( is_a( $payment, 'PMS_Payment' ) ? $payment : pms_get_payment( $payment ) );

		if ( empty( $payment->id ) ) {
			return null;
		}

		$query = "SELECT p.id FROM {$wpdb->prefix}pms_payments p
			JOIN {$wpdb->prefix}pms_paymentmeta pm
				ON pm.meta_key = 'subscription_id'
				AND p.id = pm.payment_id
			WHERE
				p.type IN ( 'subscription_initial_payment', 'recurring_payment_profile_created', 'expresscheckout' )
				AND pm.meta_value = '" . esc_sql( $payment->member_subscription_id) . "'
			ORDER BY p.id ASC LIMIT 1";

        $parent_payment_id = $wpdb->get_var( $query );

        if ( ! $parent_payment_id ) {
            return null;
        }

		$parent_payment = pms_get_payment( $parent_payment_id );

        return ( ! empty( $parent_payment->id ) ? $parent_payment : null );

	}


	/**
     * Returns the subscription plan's recurrence limit.
     *
     * @param int $product_id - the subscription plan id
     *
     * @return null|false|int - null if the default should be used, false for unlimited usages and int for limit
     *
     */
    protected function get_product_recurrence_limit( $product_id ) {

        $recurrence_limit = null;

        // Bail if recurring commissions settings aren't set for the product.
        if ( 'custom' != get_post_meta( $product_id, 'slicewp_recurring_commissions', true ) ) {
            return $recurrence_limit;
        }

        // Get recurrence limit for the product/variation.
        $recurrence = get_post_meta( $product_id, 'slicewp_commissions_recurrence', true );

        if ( ! empty( $recurrence ) ) {

            $recurrence_limit = ( $recurrence == 'limited' ? absint( get_post_meta( $product_id, 'slicewp_commissions_recurrence_limit', true ) ) : false );

        }

        return $recurrence_limit;

    }


	/**
     * Verifies if the given product and affiliate have custom recurring rate settings enabled.
     *
     * @param array
     *
     * @return bool
     *
     */
    protected function has_custom_settings( $args ) {

        // Check for affiliate.
        if ( 'custom' == slicewp_get_affiliate_meta( $args['affiliate_id'], 'recurring_commissions', true ) ) {
            return true;
        }

        // Check for product.
        if ( 'custom' == get_post_meta( $args['product_id'], 'slicewp_recurring_commissions', true ) ) {
            return true;
        }

        // Check for affiliate group.
        $affiliate_group_id = slicewp_get_affiliate_group_id( absint( $args['affiliate_id'] ) );

        if ( ! empty( $affiliate_group_id ) ) {

            if ( 'custom' == slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) ) {
                return true;
            }

        }

        return false;

    }


	/**
     * Calculates the recurring commission amount for products.
     * 
     * @param float $commission_amount
     * @param float $amount
     * @param array $args
     * 
     * @return float $commission_amount
     * 
     */
    public function calculate_commission_amount( $commission_amount, $amount, $args ) {

        // Check that the commission is from Paid Member Subscriptions.
        if ( $args['origin'] != $this->origin ) {
            return $commission_amount;
        }

        // Check for the recurring type.
        if ( $args['type'] != 'recurring' ) {
            return $commission_amount;
        }

        // Get the product.
        $product_id = absint( $args['product_id'] );

        $rate_type = '';
        $rate      = '';

        // Get the rate type and rate for the product.
        if ( 'custom' == get_post_meta( $product_id, 'slicewp_recurring_commissions', true ) ) {

            $rate_type = get_post_meta( $product_id, 'slicewp_commission_rate_type_recurring', true );
            $rate      = get_post_meta( $product_id, 'slicewp_commission_rate_recurring', true );

        }

        // Calculate the recurring commission amount.
        if ( ! empty( $rate_type ) && is_numeric( $rate ) ) {

            $commission_amount = ( $rate_type == 'percentage' ? round( ( $amount * $rate / 100 ), 2 ) : $rate );

        }

        // Return the commission amount.
        return $commission_amount;

    }

}

new SliceWP_Integration_Recurring_Commissions_Paid_Member_Subscriptions();