<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Recurring commissions Studiocart integration class
 *
 */
class SliceWP_Integration_Recurring_Commissions_Studiocart extends SliceWP_Integration_Recurring_Commissions {

	/**
     * Constructor
     *
     */
    public function __construct() {

        $this->origin = 'stc';

        // Add the "recurring" commission type to Studiocart supports array
        add_filter( 'slicewp_integration_supports_stc', array( $this, 'add_integration_supports' ) );

        // Adds a new recurring commission to the database
        add_action( 'sc_renewal_payment', array( $this, 'insert_recurring_commission' ), -1, 3 );

        // Add recurring commission rate settings in the product page
        add_action( 'slicewp_stc_metabox_commission_settings_bottom', array( $this, 'add_product_recurring_commissions_settings' ) );

        // Save the product recurring commissions settings
        add_action( 'save_post_sc_product', array( $this, 'save_product_recurring_commissions_settings' ) );

        // Calculate the recurring commission amount
        add_filter( 'slicewp_calculate_commission_amount', array( $this, 'calculate_commission_amount' ), 50, 3 );

    }


    /**
     * Add the "recurring" commission type to Studiocart supports array
     *
     * @param array $supports
     *
     * @return array
     *
     */
    public function add_integration_supports( $supports ) {

        if ( empty( $supports['commission_types'] ) || ! is_array( $supports['commission_types'] ) ) {
            return $supports;
        }

        $supports['commission_types'][] = 'recurring';

        return $supports;

    }


    /**
     * Adds a new recurring commission to the database.
     *
     * @param string $status
     * @param array $order_info
     * @param string $order_type
     *
     */
    public function insert_recurring_commission( $status, $order_info, $order_type ) {

		// Check if this is a recurring payment.
		if ( ! isset( $order_info['plan'] ) || 'recurring' != $order_info['plan']->{'type'} ) {
            return;
        }

		// Get the parent commission.
        $parent_commission = $this->get_parent_commission( $order_info );

        // Bail if the commission is missing or is rejected.
        if ( is_null( $parent_commission ) || $parent_commission->get( 'status' ) == 'rejected' ) {
            return;
        }

        // Bail if the affiliate doesn't exist or is inactive.
        $affiliate = slicewp_get_affiliate( absint( $parent_commission->get( 'affiliate_id' ) ) );

        if ( is_null( $affiliate ) || $affiliate->get( 'status' ) != 'active' ) {

            slicewp_add_log( "STC: Recurring commission was not inserted because the affiliate either doesn't exist or is not active." );
            return;

        }

        $recurring_commissions_affiliate = slicewp_get_affiliate_meta( $affiliate->get( 'id' ), 'recurring_commissions', true );

        // Bail if the affiliate has recurring commissions disabled.
        if ( 'disabled' == $recurring_commissions_affiliate ) {

            slicewp_add_log( 'STC: Recurring commission was not inserted because the affiliate has recurring commissions disabled.' );
            return;

        }

        // Get affiliate group settings.
        $affiliate_group_id          = slicewp_get_affiliate_group_id( $affiliate->get( 'id' ) );
        $recurring_commissions_group = ( ! empty( $affiliate_group_id ) ? slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) : '' );

        $product_id = $order_info['product_id'];

        // Verify if commissions are disabled for this product.
        if ( get_post_meta( $product_id, 'slicewp-disable-commissions', true ) ) {
            return;
        }

        $recurring_commissions_product = get_post_meta( $product_id, 'slicewp-recurring-commissions', true );
        
        // Verify if product has recurring commissions disabled.
        if ( 'disabled' == $recurring_commissions_product ) {
            
            slicewp_add_log( 'STC: Recurring commission was not inserted for this product because it has recurring commissions disabled. Product: ' . $product_id );
            return;

        }

        if ( 'disabled' == $recurring_commissions_group && $recurring_commissions_affiliate != 'custom' && $recurring_commissions_product != 'custom' ) {
            return;
        }

        // Verify if there are custom settings for the product/affiliate.
        // If there aren't and recurring commissions aren't enabled sitewide, we don't generate commissions.
        if ( empty( slicewp_get_setting( 'recurring_commissions' ) ) && ! $this->has_custom_settings( array( 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $product_id ) ) ) {
            return;
        }

        // Verify if the recurrence limit has been reached.
        if ( $this->recurrence_limit_reached( array( 'parent_id' => $parent_commission->get( 'id' ), 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $product_id ) ) ) {

            slicewp_add_log( 'STC: Recurring commission was not inserted because the recurrence limit has been reached. Product: ' . $product_id );
            return;

        }

        // Get the order amount.
        $amount = $order_info['amount'];

        // Exclude tax.
		if ( slicewp_get_setting( 'exclude_tax', false ) ) {

            if ( ! empty( $order_info['tax_amount'] ) ) {
                $amount = $amount - $order_info['tax_amount'];
            }

		}
        
        // Calculate the commission amount for the entire order.
        $args = array(
            'origin'       => 'stc',
            'type'         => 'recurring',
            'affiliate_id' => $parent_commission->get( 'affiliate_id' ),
            'product_id'   => $product_id
        );

        if ( function_exists( 'slicewp_maybe_convert_amount' ) ) {
            $amount = slicewp_maybe_convert_amount( $amount, $order_info['currency'], slicewp_get_setting( 'active_currency', 'USD' ) );
        }

        $commission_amount = slicewp_calculate_commission_amount( $amount, $args );

        // Check that the commission amount is not zero
        if ( $commission_amount == 0 && empty( slicewp_get_setting( 'zero_amount_commissions' ) ) ) {

            slicewp_add_log( 'STC: Recurring commission was not inserted because the commission amount is zero.' );
            return;

        }

        // Prepare commission data
        $commission_data = array(
            'affiliate_id'		=> $parent_commission->get( 'affiliate_id' ),
            'parent_id'			=> $parent_commission->get( 'id' ),
            'visit_id'			=> 0,
            'date_created'		=> slicewp_mysql_gmdate(),
            'date_modified'		=> slicewp_mysql_gmdate(),
            'type'				=> 'recurring',
            'status'			=> 'unpaid',
            'reference'			=> $order_info['ID'],
            'reference_amount'	=> slicewp_sanitize_amount( $order_info['amount'] ),
            'origin'			=> 'stc',
            'amount'			=> slicewp_sanitize_amount( $commission_amount ),
            'currency'			=> slicewp_get_setting( 'active_currency', 'USD' ),
            'customer_id'       => ( ! empty( $parent_commission->get( 'customer_id' ) ) ? absint( $parent_commission->get( 'customer_id' ) ) : 0 )
        );

        // Insert the commission
        $commission_id = slicewp_insert_commission( $commission_data );

        if ( ! empty( $commission_id ) ) {

            slicewp_add_log( sprintf( 'STC: Recurring commission #%s has been successfully inserted.', $commission_id ) );
            
        } else {

            slicewp_add_log( 'STC: Recurring commission could not be inserted due to an unexpected error.' );
            
        }

    }


	/**
	 * Adds recurring commission rate settings in the product page
	 *
	 * @param array $fields
	 * 
	 * @return array
	 * 
	 */
	public function add_product_recurring_commissions_settings( $fields ) {

		global $post;
		
		// Get current product ID
		$product_id = ( ! empty( $_REQUEST['edit'] ) ? absint( $_REQUEST['edit'] ) : 0 );

		// Check if is hidden by parent
		$is_hidden_by_parent = false;

		// Get the product categories
		$categories = get_the_terms( $post->ID, 'sc_product_cat' );

		// Get the first category of the product
		if ( ! empty( $categories[0]->term_id ) && get_term_meta( $categories[0]->term_id, 'slicewp_disable_commissions', true ) ) {
            $is_hidden_by_parent = true;
        }

		// Get recurring commissions status
		$recurring_commissions = get_post_meta( $product_id, 'slicewp-recurring-commissions', true );

		// Add the fields into an option group
		$fields['slicewp_options_group_rcr_div'] = array(
			'type'	=> 'html',
			'value' => '<div class="slicewp-options-group slicewp-show-if-subscription slicewp-hide-if-disabled-commissions ' . ( $is_hidden_by_parent ? 'slicewp-hidden-by-parent' : '' ) . '">'
		);

		// Add Recurring Commissions heading
		$fields['slicewp_heading_rcr'] = array(
			'class'	=> 'widefat',
			'type'	=> 'html',
			'value' => '<div id="slicewp_heading_rcr" class="sc-field sc-row"><h4 style="margin: 0;padding: 0 0 5px;border-bottom: 1px solid #d5d5d5;flex-basis: 100%;">' . __( 'Recurring Commissions', 'slicewp' ). '</h4></div>'
		);

		$fields['slicewp_option_field_wrapper_recurring_commissions'] = array(
			'type'	=> 'html',
			'value' => '<div class="slicewp-option-field-wrapper">'
		);

		$fields['slicewp_recurring_commissions'] = array(
			'class' 		=> 'slicewp-option-field-recurring-commissions',
			'id' 			=> 'slicewp-recurring-commissions',
			'label'			=> __( 'Recurring Commissions', 'slicewp' ),
			'type'			=> 'select',
			'selections'    =>
							array(
								''			=> __( 'Site Default', 'slicewp' ),
								'custom'	=> __( 'Custom', 'slicewp' ),
								'disabled'	=> __( 'Disabled', 'slicewp' )
							),
			'value'			=> '' // will be populated by Studiocart, if removed a notice will be shown
		);

		$fields['slicewp_option_field_wrapper_recurring_commissions_div_end'] = array(
			'type'	=> 'html',
			'value' => '</div>'
		);

		$fields['slicewp_option_field_wrapper_rate_type_recurring'] = array(
			'type'	=> 'html',
			'value' => '<div class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate-type slicewp-commission-type-recurring">'
		);

		// Prepare the commission type for select field
		$commissions_select[''] = __( 'Site Default', 'slicewp' );
		
        foreach ( slicewp_get_commission_type_rate_types( 'recurring' ) as $rate_type => $rate_type_label ) {
            $commissions_select[$rate_type] = $rate_type_label;
        }

		$fields['slicewp_commission_rate_type_recurring'] = array(
			'class' 		=> '', // if removed a notice will be shown
			'id' 			=> 'slicewp-commission-rate-type-recurring',
			'label'			=> __( 'Recurring Rate Type', 'slicewp' ),
			'type'			=> 'select',
			'selections'	=> $commissions_select,
			'value'			=> '' // will be populated by Studiocart, if removed a notice will be shown
		);

		$fields['slicewp_option_field_wrapper_rate_type_recurring_div_end'] = array(
			'type'	=> 'html',
			'value' => '</div>'
		);

		$fields['slicewp_option_field_wrapper_rate_recurring'] = array(
			'type'	=> 'html',
			'value' => '<div class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate slicewp-commission-type-recurring">'
		);

		$fields['slicewp_commission_rate_recurring'] = array(
			'class' => '', // if removed a notice will be shown
			'id' 	=> 'slicewp-commission-rate-recurring',
			'label'	=> __( 'Recurring Rate', 'slicewp' ),
			'type'	=> 'text',
			'value'	=> '' // will be populated by Studiocart, if removed a notice will be shown
		);
			
		$fields['slicewp_option_field_wrapper_rate_recurring_div_end'] = array(
			'type'	=> 'html',
			'value' => '</div>'
		);


		$fields['slicewp_option_field_wrapper_commissions_recurrence'] = array(
			'type'	=> 'html',
			'value' => '<div class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence">'
		);

		$fields['slicewp_recurring_commissions_recurrence'] = array(
			'class' 		=> '', // if removed a notice will be shown
			'id' 			=> 'slicewp-commissions-recurrence',
			'label'			=> __( 'Commissions Recurrence', 'slicewp' ),
			'type'			=> 'select',
			'selections'    =>
							array(
								''			=> __( 'Site Default', 'slicewp' ),
								'unlimited'	=> __( 'Unlimited', 'slicewp' ),
								'limited'	=> __( 'Limited', 'slicewp' )
							),
			'value'			=> '' // will be populated by Studiocart, if removed a notice will be shown
		);

		$fields['slicewp_option_field_wrapper_commissions_recurrence_div_end'] = array(
			'type'	=> 'html',
			'value' => '</div>'
		);

		$fields['slicewp_option_field_wrapper_commissions_recurrence_limit'] = array(
			'type'	=> 'html',
			'value' => '<div class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence-limit">'
		);

		$fields['slicewp_commissions_recurrence_limit'] = array(
			'class' => '', // if removed a notice will be shown
			'id' 	=> 'slicewp-commissions-recurrence-limit',
			'label'	=> __( 'Recurrence Limit', 'slicewp' ),
			'type'	=> 'text',
			'value'	=> '' // will be populated by Studiocart, if removed a notice will be shown
		);

		$fields['slicewp_option_field_wrapper_rate_recurring_div_end'] = array(
			'type'	=> 'html',
			'value' => '</div>'
		);

		$fields['slicewp_options_group_rcr_div_end'] = array(
			'type'	=> 'html',
			'value' => '</div>'
		);

		return $fields;

    }


    /**
     * Saves the product recurring commissions settings
     *
     * @param int $product_id - the ID of the product
     *
     */
    function save_product_recurring_commissions_settings( $product_id ) {

        // Update recurring commissions
        if ( ! empty( $_POST['slicewp-recurring-commissions'] ) ) {
            update_post_meta( $product_id, 'slicewp-recurring-commissions', sanitize_text_field( $_POST['slicewp-recurring-commissions'] ) );
        } else {
            delete_post_meta( $product_id, 'slicewp-recurring-commissions' );
        }

        // Update recurring commission rate type
        if ( ! empty( $_POST['slicewp-commission-rate-type-recurring'] ) ) {
            update_post_meta( $product_id, 'slicewp-commission-rate-type-recurring', sanitize_text_field( $_POST['slicewp-commission-rate-type-recurring'] ) );
        } else {
            delete_post_meta( $product_id, 'slicewp-commission-rate-type-recurring' );
        }

        // Update recurring commission rate
        if ( ! empty( $_POST['slicewp-commission-rate-recurring'] ) ) {
            update_post_meta( $product_id, 'slicewp-commission-rate-recurring', sanitize_text_field( $_POST['slicewp-commission-rate-recurring'] ) );
        } else {
            delete_post_meta( $product_id, 'slicewp-commission-rate-recurring' );
        }

        // Update recurring commissions recurrence
        if ( ! empty( $_POST['slicewp-commissions-recurrence'] ) ) {
            update_post_meta( $product_id, 'slicewp-commissions-recurrence', sanitize_text_field( $_POST['slicewp-commissions-recurrence'] ) );
        } else {
            delete_post_meta( $product_id, 'slicewp-commissions-recurrence' );
        }

        // Update recurring commissions recurrence limit
        if ( isset( $_POST['slicewp-commissions-recurrence-limit'] ) ) {
            update_post_meta( $product_id, 'slicewp-commissions-recurrence-limit', absint( $_POST['slicewp-commissions-recurrence-limit'] ) );
        } else {
            delete_post_meta( $product_id, 'slicewp-commissions-recurrence-limit' );
        }

    }


	/**
     * Returns the parent commission for the recurring order/subscription
     *
	 * @param array $order_info
	 * 
     * @return null|int
     *
     */
    public function get_parent_commission( $order_info ) {

		// Get the parent subscription
		$parent_subscription = $this->get_parent_subscription( $order_info );

		// Check if we have a commission registered for the parent subscription
		if ( ! empty( $parent_subscription ) ) {

			$parent_commission = slicewp_get_commissions( array( 'origin' => 'stc', 'reference' => $parent_subscription, 'order' => 'ASC', 'parent_id' => 0 ) );
			$parent_commission = ( ! empty( $parent_commission ) ? $parent_commission[0] : null );
			
			if ( ! empty( $parent_commission ) ) {
                return $parent_commission;
            }

		}

		// Get the parent order
		$parent_order = $this->get_parent_order( $order_info );

		// Check if we have a commission registered for the parent order
		if ( ! empty( $parent_order ) ) {

			$parent_commission = slicewp_get_commissions( array( 'origin' => 'stc', 'reference' => $parent_order, 'order' => 'ASC', 'parent_id' => 0 ) );
			$parent_commission = ( ! empty( $parent_commission ) ? $parent_commission[0] : null );

			if ( ! empty( $parent_commission ) ) {
                return $parent_commission;
            }

		}

		return null;

    }


	/**
	 * Returns the initiating subscription for the renewal order
	 *
	 * @param array $order_info
	 * 
	 * @return null|int
	 *
	 */
	public function get_parent_subscription( $order_info ) {
		
		// Check for subscription ID
		if ( empty( $order_info['subscription_id'] ) ) {
            return null;
        }
		
		// Get the parent subscription for this order
		$parent_subscription = get_post( $order_info['subscription_id'] );

		if ( empty( $parent_subscription ) ) {
            return null;
        }

		return $parent_subscription->ID;

	}


	/**
	 * Returns the initiating order for the renewal order
	 *
	 * @param array $order_info
	 * 
	 * @return null|int
	 *
	 */
	public function get_parent_order( $order_info ) {

		// Check for subscription ID
		if ( empty( $order_info['subscription_id'] ) ) {
            return null;
        }

		// Perpare the args
		$args = array(
			'post_type' 	=> 'sc_order',
			'orderby'		=> 'date',
			'order'			=> 'ASC',
			'meta_query' => array(
				array(
					'key'	=> '_sc_subscription_id',
					'value'	=> $order_info['subscription_id']
				),
			)
		);

		$query = new WP_Query( $args );

		// Check if we have any orders
		if ( ! $query->have_posts() ) {
            return null;
        }
		
		// Get the parent order of the subscription_id
		$parent_order = $query->get_posts()[0];

		return $parent_order->ID;

	}


    /**
     * Returns the product's recurrence limit
     *
     * @param int $product_id - the product id
     *
     * @return null|false|int - null if the default should be used, false for unlimited usages and int for limit
     *
     */
    protected function get_product_recurrence_limit( $product_id ) {

        $recurrence_limit = null;

        // Bail if recurring commissions settings aren't set for the product
        if ( 'custom' != get_post_meta( $product_id, 'slicewp-recurring-commissions', true ) ) {
            return $recurrence_limit;
        }

        // Get recurrence limit for the product/variation
        $recurrence = get_post_meta( $product_id, 'slicewp-commissions-recurrence', true );

        if ( ! empty( $recurrence ) ) {

            $recurrence_limit = ( $recurrence == 'limited' ? absint( get_post_meta( $product_id, 'slicewp-commissions-recurrence-limit', true ) ) : false );

        }

        return $recurrence_limit;

    }


    /**
     * Verifies if the given product and affiliate have custom recurring rate settings enabled.
     *
     * @param array
     *
     * @return bool
     *
     */
    protected function has_custom_settings( $args ) {

        // Check for affiliate.
        if ( 'custom' == slicewp_get_affiliate_meta( $args['affiliate_id'], 'recurring_commissions', true ) ) {
            return true;
        }

        // Check for product.
        if ( 'custom' == get_post_meta( $args['product_id'], 'slicewp-recurring-commissions', true ) ) {
            return true;
        }

        // Check for affiliate group.
        $affiliate_group_id = slicewp_get_affiliate_group_id( absint( $args['affiliate_id'] ) );

        if ( ! empty( $affiliate_group_id ) ) {

            if ( 'custom' == slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) ) {
                return true;
            }

        }

        return false;

    }


    /**
     * Calculates the recurring commission amount for products and variations.
     * 
     * @param float $commission_amount
     * @param float $amount
     * @param array $args
     * 
     * @return float $commission_amount
     * 
     */
    public function calculate_commission_amount( $commission_amount, $amount, $args ) {

        // Check that the commission is from Studiocart
        if ( $args['origin'] != $this->origin ) {
            return $commission_amount;
        }

        // Check for the recurring type
        if ( $args['type'] != 'recurring' ) {
            return $commission_amount;
        }

        // Get the product
        $product_id = absint( $args['product_id'] );

        $rate_type = '';
        $rate      = '';

        // Get the rate type and rate for the product
        if ( 'custom' == get_post_meta( $product_id, 'slicewp-recurring-commissions', true ) ) {

            $rate_type = get_post_meta( $product_id, 'slicewp-commission-rate-type-recurring', true );
            $rate      = get_post_meta( $product_id, 'slicewp-commission-rate-recurring', true );

        }

        // Calculate the recurring commission amount
        if ( ! empty( $rate_type ) && is_numeric( $rate ) ) {

            $commission_amount = ( $rate_type == 'percentage' ? round( ( $amount * $rate / 100 ), 2 ) : $rate );

        }

        // Return the commission amount
        return $commission_amount;

    }

}

new SliceWP_Integration_Recurring_Commissions_Studiocart();