<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Recurring commissions Surecart integration class.
 *
 */
class SliceWP_Integration_Recurring_Commissions_SureCart extends SliceWP_Integration_Recurring_Commissions {

    /**
     * Constructor
     *
     */
    public function __construct() {

        $this->origin = 'surecart';

        // Add the "recurring" commission type to SureCart supports array.
        add_filter( 'slicewp_integration_supports_surecart', array( $this, 'add_integration_supports' ) );

        // // Adds a new recurring commission to the database.
        add_action( 'surecart/subscription_renewed', array( $this, 'insert_recurring_commission' ), 10, 2 );

    }


    /**
     * Add the "recurring" commission type to SureCart supports array.
     *
     * @param array $supports
     *
     * @return array
     *
     */
    public function add_integration_supports( $supports ) {

        if ( empty( $supports['commission_types'] ) || ! is_array( $supports['commission_types'] ) ) {
            return $supports;
        }

        $supports['commission_types'][] = 'recurring';

        return $supports;

    }


    /**
     * Adds a new recurring commission to the database.
     *
     * @param \SureCart\Models\Subscription $subscription
     *
     */
    public function insert_recurring_commission( $subscription ) {

        $purchase = \SureCart\Models\Purchase::with( [ 'initial_order', 'subscription', 'subscription.current_period', 'period.checkout', 'checkout.order' ] )->find( $subscription->purchase );
        
        $checkout = $purchase->subscription->current_period->checkout;
        $order    = $checkout->order;

        if ( empty( $purchase->initial_order ) || empty( $purchase->initial_order->id ) ) {

            slicewp_add_log( 'SureCart: Recurring commission was not inserted because parent commission is either missing or rejected. Subscription: ' . var_export( $subscription, true ) );
            return;

        }

        // Get the first commission, generated for the first subscription payment.
        $parent_commission = slicewp_get_commissions( array( 'origin' => 'surecart', 'reference' => $purchase->initial_order->id, 'order' => 'ASC', 'parent_id' => 0 ) );
        $parent_commission = ( ! empty( $parent_commission ) ? $parent_commission[0] : null );

        // Bail if the commission is missing or is rejected.
        if ( is_null( $parent_commission ) || $parent_commission->get( 'status' ) == 'rejected' ) {

            slicewp_add_log( 'SureCart: Recurring commission was not inserted because parent commission is either missing or rejected. Subscription: ' . var_export( $subscription, true ) );
            return;

        }

        // Bail if the affiliate doesn't exist or is inactive.
        $affiliate = slicewp_get_affiliate( absint( $parent_commission->get( 'affiliate_id' ) ) );

        if ( is_null( $affiliate ) || $affiliate->get( 'status' ) != 'active' ) {

            slicewp_add_log( "SureCart: Recurring commission was not inserted because the affiliate either doesn't exist or is not active." );
            return;

        }

        $recurring_commissions_affiliate = slicewp_get_affiliate_meta( $affiliate->get( 'id' ), 'recurring_commissions', true );

        // Bail if the affiliate has recurring commissions disabled.
        if ( 'disabled' == $recurring_commissions_affiliate ) {

            slicewp_add_log( 'SureCart: Recurring commission was not inserted because the affiliate has recurring commissions disabled.' );
            return;

        }

        // Get affiliate group settings.
        $affiliate_group_id          = slicewp_get_affiliate_group_id( $affiliate->get( 'id' ) );
        $recurring_commissions_group = ( ! empty( $affiliate_group_id ) ? slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) : '' );

        if ( 'disabled' == $recurring_commissions_group && $recurring_commissions_affiliate != 'custom' ) {
            return;
        }

        // Verify if there are custom settings for the product/affiliate.
        // If there aren't and recurring commissions aren't enabled sitewide, we don't generate commissions.
        if ( empty( slicewp_get_setting( 'recurring_commissions' ) ) && ! $this->has_custom_settings( array( 'affiliate_id' => $parent_commission->get( 'affiliate_id' ) ) ) ) {
            return;
        }

        if ( \SureCart\Support\Currency::isZeroDecimal( $checkout->currency ) ) {
            $order_amount = $checkout->amount_due;
        } else {
            $order_amount = round( $checkout->amount_due / 100, 2 );
        }

        // Calculate commission amount.
        $args = array(
            'origin'       => 'surecart',
            'type'         => 'recurring',
            'affiliate_id' => $parent_commission->get( 'affiliate_id' )
        );

        if ( function_exists( 'slicewp_maybe_convert_amount' ) ) {
            $order_amount = slicewp_maybe_convert_amount( $order_amount, strtoupper( $checkout->currency ), slicewp_get_setting( 'active_currency', 'USD' ) );
        }

        $commission_amount = slicewp_calculate_commission_amount( $order_amount, $args );

        // Check that the commission amount is not zero.
        if ( ( $commission_amount == 0 ) && empty( slicewp_get_setting( 'zero_amount_commissions' ) ) ) {

            slicewp_add_log( 'SureCart: Commission was not inserted because the commission amount is zero. Order: ' . sanitize_text_field( $order->id ) );
            return;

        }

        // Prepare commission data.
        $commission_data = array(
            'affiliate_id'		=> absint( $parent_commission->get( 'affiliate_id' ) ),
            'parent_id'			=> absint( $parent_commission->get( 'id' ) ),
            'visit_id'			=> 0,
            'date_created'		=> slicewp_mysql_gmdate(),
            'date_modified'		=> slicewp_mysql_gmdate(),
            'type'				=> 'recurring',
            'status'			=> 'unpaid',
            'reference'			=> sanitize_text_field( $order->id ),
			'reference_amount'	=> slicewp_sanitize_amount( $order_amount ),
            'origin'			=> 'surecart',
            'amount'			=> slicewp_sanitize_amount( $commission_amount ),
            'currency'			=> slicewp_get_setting( 'active_currency', 'USD' ),
            'customer_id'       => ( ! empty( $parent_commission->get( 'customer_id' ) ) ? absint( $parent_commission->get( 'customer_id' ) ) : 0 )
        );

        // Insert the commission.
        $commission_id = slicewp_insert_commission( $commission_data );

        if ( ! empty( $commission_id ) ) {

            slicewp_add_log( sprintf( 'SureCart: Recurring commission #%s has been successfully inserted.', $commission_id ) );
            
        } else {

            slicewp_add_log( 'SureCart: Recurring commission could not be inserted due to an unexpected error.' );
            
        }

    }


    /**
     * Verifies if the given product and affiliate have custom recurring rate settings enabled.
     *
     * @param array
     *
     * @return bool
     *
     */
    protected function has_custom_settings( $args ) {

        // Check for affiliate.
        if ( 'custom' == slicewp_get_affiliate_meta( $args['affiliate_id'], 'recurring_commissions', true ) ) {
            return true;
        }

        // Check for affiliate group.
        $affiliate_group_id = slicewp_get_affiliate_group_id( absint( $args['affiliate_id'] ) );

        if ( ! empty( $affiliate_group_id ) ) {

            if ( 'custom' == slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) ) {
                return true;
            }

        }

        return false;

    }

}

new SliceWP_Integration_Recurring_Commissions_SureCart();