<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Recurring commissions WooCommerce integration class.
 *
 */
class SliceWP_Integration_Recurring_Commissions_WooCommerce extends SliceWP_Integration_Recurring_Commissions {

    /**
     * Constructor
     *
     */
    public function __construct() {

        $this->origin = 'woo';

        // Add the "recurring" commission type to WooCommerce supports array.
        add_filter( 'slicewp_integration_supports_woo', array( $this, 'add_integration_supports' ) );

        // Adds a new recurring commission to the database.
        add_action( 'woocommerce_subscription_renewal_payment_complete', array( $this, 'insert_recurring_commission' ), 10, 2 );

        // Add recurring commission rate settings in product page.
        add_action( 'slicewp_woo_product_data_panel_bottom', array( $this, 'add_product_recurring_commissions_settings' ), 20 );

        // Add variation recurring commission rate settings in product page.
        add_action( 'slicewp_woo_variation_product_data_panel_bottom', array( $this, 'add_product_variation_recurring_commissions_settings' ), 10, 3 );

        // Save the product commission rate settings.
        add_action( 'save_post_product', array( $this, 'save_product_recurring_commissions_settings' ), 15, 2 );

        // Save the product variation commission rate settings.
        add_action( 'woocommerce_ajax_save_product_variations', array( $this, 'save_variation_product_recurring_commissions_settings' ), 15, 1 );

        // Calculate the recurring commission amount.
        add_filter( 'slicewp_calculate_commission_amount', array( $this, 'calculate_commission_amount' ), 50, 3 );

    }


    /**
     * Add the "recurring" commission type to WooCommerce supports array.
     *
     * @param array $supports
     *
     * @return array
     *
     */
    public function add_integration_supports( $supports ) {

        if ( empty( $supports['commission_types'] ) || ! is_array( $supports['commission_types'] ) ) {
            return $supports;
        }

        $supports['commission_types'][] = 'recurring';

        return $supports;

    }


    /**
     * Adds a new recurring commission to the database.
     *
     * @param WC_Subscription $subscription
     * @param WC_Order        $last_order
     *
     */
    public function insert_recurring_commission( $subscription, $last_order ) {

        $first_order_id = ( method_exists( $subscription, 'get_parent_id' ) ? $subscription->get_parent_id() : $subscription->order->id );

        // Get the first commission, generated for the first subscription payment
        $parent_commission = slicewp_get_commissions( array( 'origin' => 'woo', 'reference' => $first_order_id, 'order' => 'ASC', 'parent_id' => 0 ) );
        $parent_commission = ( ! empty( $parent_commission ) ? $parent_commission[0] : null );

        // Bail if the commission is missing or is rejected
        if ( is_null( $parent_commission ) || $parent_commission->get( 'status' ) == 'rejected' ) {

            slicewp_add_log( 'WOO: Recurring commission was not inserted because parent commission is either missing or rejected. Subscription: ' . var_export( $subscription, true ) );
            return;

        }

        // Bail if the affiliate doesn't exist or is inactive.
        $affiliate = slicewp_get_affiliate( absint( $parent_commission->get( 'affiliate_id' ) ) );

        if ( is_null( $affiliate ) || $affiliate->get( 'status' ) != 'active' ) {

            slicewp_add_log( "WOO: Recurring commission was not inserted because the affiliate either doesn't exist or is not active." );
            return;

        }

        $recurring_commissions_affiliate = slicewp_get_affiliate_meta( $affiliate->get( 'id' ), 'recurring_commissions', true );

        // Bail if the affiliate has recurring commissions disabled.
        if ( 'disabled' == $recurring_commissions_affiliate ) {

            slicewp_add_log( 'WOO: Recurring commission was not inserted because the affiliate has recurring commissions disabled.' );
            return;

        }

        // Get affiliate group settings.
        $affiliate_group_id          = slicewp_get_affiliate_group_id( $affiliate->get( 'id' ) );
        $recurring_commissions_group = ( ! empty( $affiliate_group_id ) ? slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) : '' );

        // Get all cart items
        $cart_items = $last_order->get_items();

        // Set cart shipping
        $cart_shipping = $last_order->get_shipping_total( 'edit' );

        if ( ! slicewp_get_setting( 'exclude_tax', false ) ) {
            $cart_shipping = $cart_shipping + $last_order->get_shipping_tax( 'edit' );
        }

        $commission_amount = 0;

        // Go through each item and calculate the commission amount
        foreach ( $cart_items as $cart_item ) {

            /**
             * @todo - If all items from cart are disabled or for all items the recurrence limit has been reached,
             *         a zero amount commission will be inserted. Ideally, in this case, no commission should be generated.
             */

            // Get the product categories
            $categories = get_the_terms( $cart_item->get_product_id(), 'product_cat' );

            // Verify if commissions are disabled for this product category
            if ( ! empty( $categories[0]->term_id ) && get_term_meta( $categories[0]->term_id, 'slicewp_disable_commissions', true ) ) {
                continue;
            }

            // Verify if commissions are disabled for this product
            if ( get_post_meta( $cart_item->get_product_id(), 'slicewp_disable_commissions', true ) ) {
                continue;
            }

            // Verify if commissions are disabled for this product variation
            if ( ! empty( $cart_item->get_variation_id() ) && get_post_meta( $cart_item->get_variation_id(), 'slicewp_disable_commissions', true ) ) {
                continue;
            }

            $recurring_commissions_variation = ( ! empty( $cart_item->get_variation_id( 'edit' ) ) ? get_post_meta( $cart_item->get_variation_id( 'edit' ), 'slicewp_recurring_commissions', true ) : '' );
            $recurring_commissions_product   = get_post_meta( $cart_item->get_product_id( 'edit' ), 'slicewp_recurring_commissions', true );

            // Verify if product variation has recurring commissions disabled.
            if ( ! empty( $cart_item->get_variation_id( 'edit' ) ) && 'disabled' == $recurring_commissions_variation ) {
                
                slicewp_add_log( 'WOO: Recurring commission was not calculated for this variation because product variation has recurring commissions disabled. Product: ' . $cart_item->get_product_id() . '. Variation: ' . $cart_item->get_variation_id() );
                continue;

            }

            // Verify if product has recurring commissions disabled.
            if ( 'disabled' == $recurring_commissions_product ) {
                
                slicewp_add_log( 'WOO: Recurring commission was not calculated for this product because product has recurring commissions disabled. Product: ' . $cart_item->get_product_id() );
                continue;

            }

            if ( 'disabled' == $recurring_commissions_group && $recurring_commissions_affiliate != 'custom' && $recurring_commissions_variation != 'custom' && $recurring_commissions_product != 'custom' ) {
                continue;
            }

            // Set product ID
            $variation_id = $cart_item->get_variation_id( 'edit' );
            $product_id   = ( ! empty( $variation_id ) ? $variation_id : $cart_item->get_product_id( 'edit' ) );

            // Verify if there are custom settings for the product/affiliate
            // If there aren't and recurring commissions aren't enabled sitewide, we don't generate commissions
            if ( empty( slicewp_get_setting( 'recurring_commissions' ) ) && ! $this->has_custom_settings( array( 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $product_id ) ) ) {
                continue;
            }

            // Verify if the recurrence limit has been reached
            if ( $this->recurrence_limit_reached( array( 'parent_id' => $parent_commission->get( 'id' ), 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $product_id ) ) ) {

                slicewp_add_log( 'WOO: Recurring commission was not calculated for this product because the recurrence limit has been reached. Product: ' . $cart_item->get_product_id() . ( ! empty( $cart_item->get_variation_id() ) ? '. Variation: ' . $cart_item->get_variation_id() : '' ) );
                continue;

            }

            $amount = $cart_item->get_total( 'edit' );

            // Include tax
            if ( ! slicewp_get_setting( 'exclude_tax', false ) ) {
                $amount = $amount + $cart_item->get_total_tax( 'edit' );
            }

            // Include shipping
            if ( ! slicewp_get_setting( 'exclude_shipping', false ) && $cart_shipping > 0 ) {
                $amount = $amount + $cart_shipping / count( $cart_items );
            }

            // Calculate commission amount
            $args = array(
                'origin'       => 'woo',
                'type'         => 'recurring',
                'affiliate_id' => $parent_commission->get( 'affiliate_id' ),
                'product_id'   => $product_id
            );

            if ( function_exists( 'slicewp_maybe_convert_amount' ) ) {
                $amount = slicewp_maybe_convert_amount( $amount, $last_order->get_currency( 'edit' ), slicewp_get_setting( 'active_currency', 'USD' ) );
            }

            $commission_amount += slicewp_calculate_commission_amount( $amount, $args );

        }

        // Verify if there are custom settings for the product/affiliate
        // If there aren't and recurring commissions aren't enabled sitewide, we don't generate commissions
        if ( $commission_amount == 0 && empty( slicewp_get_setting( 'recurring_commissions' ) ) ) {

            slicewp_add_log( 'WOO: Recurring commission was not inserted because sitewide recurring commissions are disabled.' );
            return;

        }

        // Check that the commission amount is not zero
        if ( $commission_amount == 0 && empty( slicewp_get_setting( 'zero_amount_commissions' ) ) ) {

            slicewp_add_log( 'WOO: Recurring commission was not inserted because the commission amount is zero.' );
            return;

        }

        // Prepare commission data
        $commission_data = array(
            'affiliate_id'		=> $parent_commission->get( 'affiliate_id' ),
            'parent_id'			=> $parent_commission->get( 'id' ),
            'visit_id'			=> 0,
            'date_created'		=> slicewp_mysql_gmdate(),
            'date_modified'		=> slicewp_mysql_gmdate(),
            'type'				=> 'recurring',
            'status'			=> 'unpaid',
            'reference'			=> ( method_exists( $last_order, 'get_id' ) ? $last_order->get_id() : $last_order->id ),
			'reference_amount'	=> slicewp_sanitize_amount( $last_order->get_total() ),
            'origin'			=> 'woo',
            'amount'			=> slicewp_sanitize_amount( $commission_amount ),
            'currency'			=> slicewp_get_setting( 'active_currency', 'USD' ),
            'customer_id'       => ( ! empty( $parent_commission->get( 'customer_id' ) ) ? absint( $parent_commission->get( 'customer_id' ) ) : 0 )
        );

        // Insert the commission
        $commission_id = slicewp_insert_commission( $commission_data );

        if ( ! empty( $commission_id ) ) {

            slicewp_add_log( sprintf( 'WOO: Recurring commission #%s has been successfully inserted.', $commission_id ) );
            
        } else {

            slicewp_add_log( 'WOO: Recurring commission could not be inserted due to an unexpected error.' );
            
        }

    }


    /**
     * Adds the product commission rate settings in the SliceWP product tab.
     *
     */
    public function add_product_recurring_commissions_settings() {

        global $post;

        // Check if is hidden by parent
        $is_hidden_by_parent = false;

        // Get the product categories
        $categories = get_the_terms( $post->ID, 'product_cat' );

        // Get the first category of the product
        if ( ! empty( $categories[0]->term_id ) && get_term_meta( $categories[0]->term_id, 'slicewp_disable_commissions', true ) ) {
            $is_hidden_by_parent = true;
        }

        echo '<div class="slicewp-options-group options_group slicewp-show-if-subscription slicewp-hide-if-disabled-commissions ' . ( $is_hidden_by_parent ? 'slicewp-hidden-by-parent' : '' ) . '">';

            woocommerce_wp_select( array(
                'id'          => 'slicewp_product_recurring_commissions',
                'label'       => __( 'Recurring Commissions', 'slicewp' ),
                'value'       => get_post_meta( $post->ID, 'slicewp_recurring_commissions', true ),
                'options'     => array( '' => __( 'Site default', 'slicewp' ), 'custom' => __( 'Custom', 'slicewp' ), 'disabled' => __( 'Disabled', 'slicewp' ) ),
                'desc_tip'    => true,
                'class'       => 'slicewp-option-field-recurring-commissions select short',
                'description' => __( 'Select whether the affiliate should be rewarded for all recurring payments for this subscription product or for a limited number of recurring payments.', 'slicewp' ),
                'wrapper_class' => 'slicewp-option-field-wrapper'
            ));

            // Add the commission rate type field
            woocommerce_wp_select( array(
                'id'          => 'slicewp_product_commission_rate_type_recurring',
                'label'       => __( 'Recurring Rate Type', 'slicewp' ),
                'value'       => get_post_meta( $post->ID, 'slicewp_commission_rate_type_recurring', true ),
                'options'     => array_merge( array( '' => __( 'Site default', 'slicewp' ) ), slicewp_get_commission_type_rate_types( 'recurring' ) ),
                'desc_tip'    => true,
                'description' => __( 'The selected commission rate type will be used to calculate the commission for this product.', 'slicewp' ),
                'wrapper_class' => 'slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate-type slicewp-commission-type-recurring'
            ));

            // Add the comission rate field
            woocommerce_wp_text_input( array(
                'id'          => 'slicewp_product_commission_rate_recurring',
                'label'       => __( 'Recurring Rate', 'slicewp' ),
                'value'       => get_post_meta( $post->ID, 'slicewp_commission_rate_recurring', true ),
                'desc_tip'    => true,
                'description' => __( 'This rate will be used to calculate the commission for this product.', 'slicewp' ),
                'wrapper_class' => 'slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate slicewp-commission-type-recurring'
            ));

            // Add the recurrence for recurring commissions
            woocommerce_wp_select( array(
                'id'          => 'slicewp_product_commissions_recurrence',
                'label'       => __( 'Commissions Recurrence', 'slicewp' ),
                'value'       => get_post_meta( $post->ID, 'slicewp_commissions_recurrence', true ),
                'options'     => array( '' => __( 'Site default', 'slicewp' ), 'unlimited' => __( 'Unlimited', 'slicewp' ), 'limited' => __( 'Limited', 'slicewp' ) ),
                'desc_tip'    => true,
                'description' => __( 'Select whether the affiliate should be rewarded for all recurring payments for this subscription product or for a limited number of recurring payments.', 'slicewp' ),
                'wrapper_class' => 'slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence'
            ));

            // Add the recurrence limit
            woocommerce_wp_text_input( array(
                'id'          => 'slicewp_product_commissions_recurrence_limit',
                'label'       => __( 'Recurrence Limit', 'slicewp' ),
                'value'       => get_post_meta( $post->ID, 'slicewp_commissions_recurrence_limit', true ),
                'desc_tip'    => true,
                'description' => __( 'Set the maximum number of commissions an affiliate is rewarded for recurring payments made by this subscription product.', 'slicewp' ),
                'wrapper_class' => 'slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence-limit'
            ));

        echo '</div>';

    }


    /**
     * Adds the product variation recurring commission rate settings in the variations tab.
     * 
     * @param int     $loop           - position in the loop
     * @param array   $variation_data - variation data
     * @param WP_Post $variation      - post data
     * 
     */
    public function add_product_variation_recurring_commissions_settings( $loop, $variation_data, $variation ) {

        // Check if the recurring fields are hidden by a parent category or product
        $is_hidden_by_parent = false;

        // Get the product categories
        $categories = get_the_terms( $variation->post_parent, 'product_cat' );

        // Verify if commissions are disabled for category
        if ( ! empty( $categories[0]->term_id ) && get_term_meta( $categories[0]->term_id, 'slicewp_disable_commissions', true ) ) {
            $is_hidden_by_parent = true;
        }

        // Verify if the commissions are disabled for the parent product
        if ( get_post_meta( $variation->post_parent, 'slicewp_disable_commissions', true ) ) {
            $is_hidden_by_parent = true;
        }

        // Get recurring commissions status
        $recurring_commissions = get_post_meta( $variation->ID, 'slicewp_recurring_commissions', true );

        // Get the current product variation settings
        $commission_rate_type = get_post_meta( $variation->ID, 'slicewp_commission_rate_type_recurring', true );
        $commission_rate      = get_post_meta( $variation->ID, 'slicewp_commission_rate_recurring', true );

        // Get the recurrence
        $recurrence       = get_post_meta( $variation->ID, 'slicewp_commissions_recurrence', true );
        $recurrence_limit = get_post_meta( $variation->ID, 'slicewp_commissions_recurrence_limit', true );

        ?>

        <div class="slicewp-options-group slicewp-show-if-subscription slicewp-hide-if-disabled-commissions <?php echo ( $is_hidden_by_parent ? 'slicewp-hidden-by-parent' : '' ); ?>">

            <p class="slicewp-option-field-wrapper form-row form-row-full">
                <label for="slicewp_product_variation_recurring_commissions[<?php echo $variation->ID; ?>]"><?php echo __( 'Recurring Commissions', 'slicewp' ); ?></label>
                <select name="slicewp_product_variation_recurring_commissions[<?php echo $variation->ID; ?>]" id="slicewp_product_variation_recurring_commissions[<?php echo $variation->ID; ?>]" class="slicewp-option-field-recurring-commissions">
                    <option value=""><?php echo __( 'Site default', 'slicewp' ); ?></option>
                    <option value="custom" <?php selected( $recurring_commissions, 'custom' ); ?>><?php echo __( 'Custom', 'slicewp' ); ?></option>
                    <option value="disabled" <?php selected( $recurring_commissions, 'disabled' ); ?>><?php echo __( 'Disabled', 'slicewp' ); ?></option>
                </select>
            </p>

            <p class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate-type slicewp-commission-type-recurring form-row form-row-full">
                <label for="slicewp_product_variation_commission_rate_type_recurring[<?php echo $variation->ID; ?>]"><?php echo __( 'Recurring Rate Type', 'slicewp' ); ?></label>
                <select name="slicewp_product_variation_commission_rate_type_recurring[<?php echo $variation->ID; ?>]" id="slicewp_product_variation_commission_rate_type_recurring[<?php echo $variation->ID; ?>]">
                    <option value=""><?php echo __( 'Site Default', 'slicewp' ); ?></option>
                    <?php foreach( slicewp_get_commission_type_rate_types( 'recurring' ) as $rate_type => $rate_type_label ) : ?>
                        <option value="<?php echo esc_attr( $rate_type ); ?>"<?php selected( $commission_rate_type, $rate_type ); ?>><?php echo esc_html( $rate_type_label ); ?></option>
                    <?php endforeach; ?>
                </select>
            </p>

            <p class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate slicewp-commission-type-recurring form-row form-row-full">
                <label for="slicewp_product_variation_commission_rate_recurring[<?php echo $variation->ID; ?>]"><?php echo __( 'Recurring Rate', 'slicewp' ); ?></label>
                <input type="text" size="5" name="slicewp_product_variation_commission_rate_recurring[<?php echo $variation->ID; ?>]" value="<?php echo esc_attr( $commission_rate ); ?>" class="wc_input_price" id="slicewp_product_variation_commission_rate_recurring[<?php echo $variation->ID; ?>]" />
            </p>

            <p class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence form-row form-row-full">
                <label for="slicewp_product_variation_commissions_recurrence[<?php echo $variation->ID; ?>]"><?php echo __( 'Commissions Recurrence', 'slicewp' ); ?></label>
                <select name="slicewp_product_variation_commissions_recurrence[<?php echo $variation->ID; ?>]" id="slicewp_product_variation_commissions_recurrence[<?php echo $variation->ID; ?>]">
                    <option value=""><?php echo __( 'Site default', 'slicewp' ); ?></option>
                    <option value="unlimited" <?php selected( $recurrence, 'unlimited' ); ?>><?php echo __( 'Unlimited', 'slicewp' ); ?></option>
                    <option value="limited" <?php selected( $recurrence, 'limited' ); ?>><?php echo __( 'Limited', 'slicewp' ); ?></option>
                </select>
            </p>

            <p class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence-limit form-row form-row-full">
                <label for="slicewp_product_variation_commissions_recurrence_limit[<?php echo $variation->ID; ?>]"><?php echo __( 'Recurrence Limit', 'slicewp' ); ?></label>
                <input type="number" name="slicewp_product_variation_commissions_recurrence_limit[<?php echo $variation->ID; ?>]" value="<?php echo esc_attr( $recurrence_limit ); ?>" id="slicewp_product_variation_commissions_recurrence_limit[<?php echo $variation->ID; ?>]" />
            </p>

        </div>

        <?php
        
    }


    /**
     * Saves the product recurring commissions data into the product meta.
     * 
     * @param int     $post_id
     * @param WP_Post $post
     * 
     */
    public function save_product_recurring_commissions_settings( $post_id, $post ) {

        // Verify for nonce
        if ( empty( $_POST['slicewp_token'] ) || ! wp_verify_nonce( $_POST['slicewp_token'], 'slicewp_save_meta' ) ) {
            return $post_id;
        }

        // Verify for autosave
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
            return $post_id;
        }

        if ( wp_is_post_revision( $post_id ) || wp_is_post_autosave( $post_id ) ) {
            return $post_id;
        }

        // Update recurring commissions.
        if ( ! empty( $_POST['slicewp_product_recurring_commissions'] ) ) {
            update_post_meta( $post_id, 'slicewp_recurring_commissions', sanitize_text_field( $_POST['slicewp_product_recurring_commissions'] ) );
        } else {
            delete_post_meta( $post_id, 'slicewp_recurring_commissions' );
        }

        // Update recurring commission rate type.
        if ( ! empty( $_POST['slicewp_product_commission_rate_type_recurring'] ) ) {
            update_post_meta( $post_id, 'slicewp_commission_rate_type_recurring', sanitize_text_field( $_POST['slicewp_product_commission_rate_type_recurring'] ) );
        } else {
            delete_post_meta( $post_id, 'slicewp_commission_rate_type_recurring' );
        }

        // Update recurring commission rate.
        if ( ! empty( $_POST['slicewp_product_commission_rate_recurring'] ) ) {
            update_post_meta( $post_id, 'slicewp_commission_rate_recurring', sanitize_text_field( $_POST['slicewp_product_commission_rate_recurring'] ) );
        } else {
            delete_post_meta( $post_id, 'slicewp_commission_rate_recurring' );
        }

        // Update recurring commissions recurrence.
        if ( ! empty( $_POST['slicewp_product_commissions_recurrence'] ) ) {
            update_post_meta( $post_id, 'slicewp_commissions_recurrence', sanitize_text_field( $_POST['slicewp_product_commissions_recurrence'] ) );
        } else {
            delete_post_meta( $post_id, 'slicewp_commissions_recurrence' );
        }

        // Update recurring commissions recurrence limit.
        if ( isset( $_POST['slicewp_product_commissions_recurrence_limit'] ) ) {
            update_post_meta( $post_id, 'slicewp_commissions_recurrence_limit', absint( $_POST['slicewp_product_commissions_recurrence_limit'] ) );
        } else {
            delete_post_meta( $post_id, 'slicewp_commissions_recurrence_limit' );
        }

    }


    /**
     * Saves the product variation commission settings into the product meta.
     * 
     * @param int $product_id
     * 
     */
    public function save_variation_product_recurring_commissions_settings( $product_id = 0 ) {

        // Check for variations
        if ( empty( $_POST['variable_post_id'] ) ) {
            return;
        }

        // Parse all the variations
        foreach ( $_POST['variable_post_id'] as $variation_id ) {
            
            $variation_id = absint( $variation_id );
            
            // Update disabled recurring commissions.
            if ( ! empty( $_POST['slicewp_product_variation_recurring_commissions'][$variation_id] ) ) {
                update_post_meta( $variation_id, 'slicewp_recurring_commissions', sanitize_text_field( $_POST['slicewp_product_variation_recurring_commissions'][$variation_id] ) );
            } else {
                delete_post_meta( $variation_id, 'slicewp_recurring_commissions' );
            }

            // Update recurring commission rate type.
            if ( ! empty( $_POST['slicewp_product_variation_commission_rate_type_recurring'][$variation_id] ) ) {
                update_post_meta( $variation_id, 'slicewp_commission_rate_type_recurring', sanitize_text_field( $_POST['slicewp_product_variation_commission_rate_type_recurring'][$variation_id] ) );
            } else {
                delete_post_meta( $variation_id, 'slicewp_commission_rate_type_recurring' );
            }

            // Update recurring commission rate.
            if ( ! empty( $_POST['slicewp_product_variation_commission_rate_recurring'][$variation_id] ) ) {
                update_post_meta( $variation_id, 'slicewp_commission_rate_recurring', sanitize_text_field( $_POST['slicewp_product_variation_commission_rate_recurring'][$variation_id] ) );
            } else {
                delete_post_meta( $variation_id, 'slicewp_commission_rate_recurring' );
            }

            // Update recurring commissions recurrence.
            if ( ! empty( $_POST['slicewp_product_variation_commissions_recurrence'][$variation_id] ) ) {
                update_post_meta( $variation_id, 'slicewp_commissions_recurrence', sanitize_text_field( $_POST['slicewp_product_variation_commissions_recurrence'][$variation_id] ) );
            } else {
                delete_post_meta( $variation_id, 'slicewp_commissions_recurrence' );
            }

            // Update recurring commissions recurrence limit.
            if ( isset( $_POST['slicewp_product_variation_commissions_recurrence_limit'][$variation_id] ) ) {
                update_post_meta( $variation_id, 'slicewp_commissions_recurrence_limit', absint( $_POST['slicewp_product_variation_commissions_recurrence_limit'][$variation_id] ) );
            } else {
                delete_post_meta( $variation_id, 'slicewp_commissions_recurrence_limit' );
            }

        }

    }


    /**
     * Returns the product's recurrence limit.
     *
     * @param int $product_id
     *
     * @return null|false|int - null if the default should be used, false for unlimited usages and int for limit
     *
     */
    protected function get_product_recurrence_limit( $product_id ) {

        $recurrence_limit = null;

        // Get the product
        $product = wc_get_product( $product_id );

        // If this is a variation, save the parent product's ID
        $parent_id = ( $product->is_type( 'variation' ) ? $product->get_parent_id() : 0 );

        // Get recurrence limit for the product/variation
        if ( 'custom' == get_post_meta( $product_id, 'slicewp_recurring_commissions', true ) ) {

            $recurrence = get_post_meta( $product_id, 'slicewp_commissions_recurrence', true );

            if ( ! empty( $recurrence ) ) {

                $recurrence_limit = ( $recurrence == 'limited' ? absint( get_post_meta( $product_id, 'slicewp_commissions_recurrence_limit', true ) ) : false );

            }

        }

        // If the product is actually a variation, get the recurrence limit from the parent product if it's not explicitly set for the variation
        if ( ! empty( $parent_id ) && is_null( $recurrence_limit ) ) {

            if ( 'custom' == get_post_meta( $parent_id, 'slicewp_recurring_commissions', true ) ) {

                $recurrence = get_post_meta( $parent_id, 'slicewp_commissions_recurrence', true );

                if ( ! empty( $recurrence ) ) {

                    $recurrence_limit = ( $recurrence == 'limited' ? absint( get_post_meta( $parent_id, 'slicewp_commissions_recurrence_limit', true ) ) : false );

                }

            }

        }

        return $recurrence_limit;

    }


    /**
     * Verifies if the given product and affiliate have custom recurring rate settings enabled.
     *
     * @param array
     *
     * @return bool
     *
     */
    protected function has_custom_settings( $args ) {

        // Check for affiliate.
        if ( 'custom' == slicewp_get_affiliate_meta( $args['affiliate_id'], 'recurring_commissions', true ) ) {
            return true;
        }

        // Check for product.
        if ( 'custom' == get_post_meta( $args['product_id'], 'slicewp_recurring_commissions', true ) ) {
            return true;
        }

        // Check for affiliate group.
        $affiliate_group_id = slicewp_get_affiliate_group_id( absint( $args['affiliate_id'] ) );

        if ( ! empty( $affiliate_group_id ) ) {

            if ( 'custom' == slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) ) {
                return true;
            }

        }

        return false;

    }


    /**
     * Calculates the recurring commission amount for products and variations.
     * 
     * @param float $commission_amount
     * @param float $amount
     * @param array $args
     * 
     * @return float $commission_amount
     * 
     */
    public function calculate_commission_amount( $commission_amount, $amount, $args ) {

        // Check that the commission is from WooCommerce
        if ( $args['origin'] != $this->origin ) {
            return $commission_amount;
        }

        // Check for the recurring type
        if ( $args['type'] != 'recurring' ) {
            return $commission_amount;
        }

        // Get the product
        $product_id = absint( $args['product_id'] );
        $product    = wc_get_product( $product_id );

        // If this is a variation, save the parent product's ID
        $parent_id = ( $product->is_type( 'variation' ) ? $product->get_parent_id() : 0 );

        $rate_type = '';
        $rate      = '';

        // Get the rate type and rate for the product/variation
        if ( 'custom' == get_post_meta( $product_id, 'slicewp_recurring_commissions', true ) ) {

            $rate_type = get_post_meta( $product_id, 'slicewp_commission_rate_type_recurring', true );
            $rate      = get_post_meta( $product_id, 'slicewp_commission_rate_recurring', true );

        }

        // If the product is actually a variation and the rates haven't been set for the variation,
        // try to get them from the parent product
        if ( ! empty( $parent_id ) && ( empty( $rate_type ) || ! is_numeric( $rate ) ) ) {

            if ( 'custom' == get_post_meta( $parent_id, 'slicewp_recurring_commissions', true ) ) {

                $rate_type = get_post_meta( $parent_id, 'slicewp_commission_rate_type_recurring', true );
                $rate      = get_post_meta( $parent_id, 'slicewp_commission_rate_recurring', true );

            }

        }

        // Calculate the recurring commission amount
        if ( ! empty( $rate_type ) && is_numeric( $rate ) ) {

            $commission_amount = ( $rate_type == 'percentage' ? round( ( $amount * $rate / 100 ), 2 ) : $rate );

        }

        // Return the commission amount
        return $commission_amount;

    }

}

new SliceWP_Integration_Recurring_Commissions_WooCommerce();