<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * REST API controller for "customers".
 * 
 */
class SliceWP_REST_API_Controller_Customers extends SliceWP_REST_API_Controller {

    /**
	 * Route base.
	 *
	 * @var    string
     * @access protected
     * 
	 */
    protected $rest_base = 'customers';


    /**
     * Registers the routes.
     * 
     */
    public function register_routes() {

        // Get all customers.
        register_rest_route( $this->namespace, '/' . $this->rest_base, array(
			'methods' 			  => WP_REST_Server::READABLE,
			'callback'			  => array( $this, 'get_items' ),
			'permission_callback' => array( $this, 'permissions_check' ),
            'args'                => $this->get_collection_params()
		));

        // Get single customers.
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/(?P<id>\d+)', array(
			'methods'			  => WP_REST_Server::READABLE,
			'callback'			  => array( $this, 'get_item' ),
			'permission_callback' => array( $this, 'permissions_check' )
		));

        // Create single customer.
        register_rest_route( $this->namespace, '/' . $this->rest_base . '/', array(
            'methods'             => WP_REST_Server::CREATABLE,
            'callback'			  => array( $this, 'create_item' ),
            'permission_callback' => array( $this, 'permissions_check' ),
            'args'                => array(
                'user_id' => array(
                    'description'       => __( 'The ID of the user that is attached to the customer.', 'slicewp' ),
                    'type'              => 'integer',
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'email' => array(
                    'description'       => __( 'The email of the user that is attached to the customer.', 'slicewp' ),
                    'type'              => 'string',
                    'format'            => 'email',
                    'required'          => true,
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'affiliate_id' => array(
                    'description'       => __( 'The ID of the affiliate that referred the customer.', 'slicewp' ),
                    'type'              => 'integer',
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'first_name' => array(
                    'description'       => __( "The customer's first name.", 'slicewp' ),
                    'type'              => 'string',
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'last_name' => array(
                    'description'       => __( "The customer's last name.", 'slicewp' ),
                    'type'              => 'string',
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'meta_data' => array(
                    'description'       => __( 'The meta data to add to the customer', 'slicewp' ),
                    'type'              => array( 'array', 'object' ),
                    'validate_callback' => 'rest_validate_request_arg'
                )
            )
        ));

        // Update single customer.
        register_rest_route( $this->namespace, '/' . $this->rest_base . '/(?P<id>\d+)', array(
            'methods'             => WP_REST_Server::EDITABLE,
            'callback'			  => array( $this, 'update_item' ),
            'permission_callback' => array( $this, 'permissions_check' ),
            'args'                => array(
                'user_id' => array(
                    'description'       => __( 'The ID of the user that is attached to the customer.', 'slicewp' ),
                    'type'              => 'integer',
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'email' => array(
                    'description'       => __( 'The email of the user that is attached to the customer.', 'slicewp' ),
                    'type'              => 'string',
                    'format'            => 'email',
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'first_name' => array(
                    'description'       => __( "The customer's first name.", 'slicewp' ),
                    'type'              => 'string',
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'last_name' => array(
                    'description'       => __( "The customer's last name.", 'slicewp' ),
                    'type'              => 'string',
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'meta_data' => array(
                    'description'       => __( 'The meta data to add to the customer', 'slicewp' ),
                    'type'              => array( 'array', 'object' ),
                    'validate_callback' => 'rest_validate_request_arg'
                )
            )
        ));

        // Delete single customer.
        register_rest_route( $this->namespace, '/' . $this->rest_base . '/(?P<id>\d+)', array(
            'methods'             => WP_REST_Server::DELETABLE,
            'callback'			  => array( $this, 'delete_item' ),
            'permission_callback' => array( $this, 'permissions_check' )
        ));

    }


    /**
     * Returns an array of customers data as a rest response.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function get_items( $request ) {

        // Prepare query args.
        $args = array();

        foreach ( $request->get_params() as $key => $value ) {

            if ( in_array( $key, array_keys( $this->get_collection_params() ) ) ) {
                $args[$key] = $value;
            }

        }

        // Get customers.
        $customers = slicewp_get_customers( $args );

        foreach ( $customers as $key => $customer ) {

            $customer = (object)$customer->to_array();

            // Add customer meta.
            if ( ! is_null( $request->get_param( 'meta' ) ) && filter_var( $request->get_param( 'meta' ), FILTER_VALIDATE_BOOLEAN ) ) {

                $customer->meta = slicewp_get_customer_meta( $customer->id );

            }

            $customers[$key] = $customer;

        }

        return rest_ensure_response( $customers );

    }


    /**
     * Returns the customer with the given ID as a rest response.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function get_item( $request ) {

        $customer = slicewp_get_customer( absint( $request->get_param( 'id' ) ) );

        if ( is_null( $customer ) ) {
            return new WP_Error( 'invalid_customer_id', 'Customer does not exist.', array( 'status' => 404 ) );
        }

        $customer = (object)$customer->to_array();

        // Add customer meta.
        if ( ! is_null( $request->get_param( 'meta' ) ) && filter_var( $request->get_param( 'meta' ), FILTER_VALIDATE_BOOLEAN ) ) {

            $customer->meta = slicewp_get_customer_meta( $customer->id );

        }

        return rest_ensure_response( $customer );

    }


    /**
     * Creates a new customer.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function create_item( $request ) {

        // Prepare item data.
        $customer_data = $this->prepare_item_data_for_database( $request );

        // If there's no data to create object, return.
        if ( empty( $customer_data ) ) {
            return new WP_Error( 'customer_create_false', 'Did not create customer. No writable data provided.', array( 'status' => 400 ) );
        }

        // Check if a customer with the same email exists.
        if ( ! is_null( slicewp_get_customer_by_email( $customer_data['email'] ) ) ) {
            return new WP_Error( 'customer_create_false', 'Did not create customer. A customer with the same email already exists.', array( 'status' => 400 ) );
        }

        // Check if a customer with the same user ID exists.
        if ( ! empty( $customer_data['user_id'] ) ) {

            if ( ! is_null( slicewp_get_customer_by_user_id( $customer_data['user_id'] ) ) ) {
                return new WP_Error( 'customer_create_false', 'Did not create customer. A customer with the same user ID already exists.', array( 'status' => 400 ) );
            }
            
        }

        // Add dates.
        $customer_data['date_created']  = slicewp_mysql_gmdate();
        $customer_data['date_modified'] = slicewp_mysql_gmdate();

        // Insert the item.
        $inserted = slicewp_insert_customer( $customer_data );

        // If inserted, return item.
        if ( $inserted ) {

            $customer = slicewp_get_customer( $inserted );
            $customer = (object)$customer->to_array();

            // Add meta data.
            if ( ! empty( $request->get_param( 'meta_data' ) ) ) {

                $this->update_item_meta( $inserted, $request->get_param( 'meta_data' ) );

                $customer->meta = $this->prepare_item_meta_for_response( slicewp_get_customer_meta( $inserted ) );

            }

            return rest_ensure_response( $customer );

        }

        return new WP_Error( 'customer_create_false', 'Something went wrong. Could not create customer.', array( 'status' => 500 ) );

    }


    /**
     * Updates the customer with the given ID.
     * 
     * @todo Return errors when providing invalid properties or when blocking the request from happening, for example, having two 
     *       customers with the same user ID.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function update_item( $request ) {
        
        $customer = slicewp_get_customer( absint( $request->get_param( 'id' ) ) );

        if ( is_null( $customer ) ) {
            return new WP_Error( 'invalid_customer_id', 'Customer does not exist.', array( 'status' => 404 ) );
        }

        // Prepare item data.
        $customer_data = $this->prepare_item_data_for_database( $request );

        // If there's no data to update, return.
        if ( empty( $customer_data ) && empty( $request->get_param( 'meta_data' ) ) ) {
            return new WP_Error( 'customer_update_false', 'Did not update customer. No writable data provided.', array( 'status' => 400 ) );
        }

        // Update item.
        if ( ! empty( $customer_data ) ) {

            // Add date modified before updating.
            $customer_data['date_modified'] = slicewp_mysql_gmdate();

            $updated = slicewp_update_customer( $customer->get( 'id' ), $customer_data );
            
        }

        // Update item meta.
        if ( ! empty( $request->get_param( 'meta_data' ) ) ) {

            $this->update_item_meta( $customer->get( 'id' ), $request->get_param( 'meta_data' ) );

            $updated = true;

        }

        // If updated, return item.
        if ( $updated ) {

            $customer = slicewp_get_customer( absint( $request->get_param( 'id' ) ) );
            $customer = (object)$customer->to_array();

            if ( ! empty( $request->get_param( 'meta_data' ) ) ) {

                $customer->meta = $this->prepare_item_meta_for_response( slicewp_get_customer_meta( absint( $customer->id ) ) );

            }

            return rest_ensure_response( $customer );

        }

        return new WP_Error( 'customer_update_false', 'Something went wrong. Could not update customer.', array( 'status' => 500 ) );

    }


    /**
     * Delete the customer with the given ID.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function delete_item( $request ) {

        $customer = slicewp_get_customer( absint( $request->get_param( 'id' ) ) );

        if ( is_null( $customer ) ) {
            return new WP_Error( 'invalid_customer_id', 'Customer does not exist.', array( 'status' => 404 ) );
        }

        // Add metadata to customer.
        $customer       = (object)$customer->to_array();
        $customer->meta = slicewp_get_customer_meta( $customer->id );

        // Delete customer.
        $deleted = slicewp_delete_customer( $customer->id );

        if ( $deleted ) {
            return rest_ensure_response( $customer );
        }

        return new WP_Error( 'customer_delete_false', 'Something went wrong. Could not delete customer.', array( 'status' => 500 ) );

    }


    /**
     * Filters and sanitizes the request data to prepare it for the database.
     * 
     * @param WP_REST_Request $request
     * 
     * @return array
     * 
     */
    protected function prepare_item_data_for_database( $request ) {

        $schema    = $this->get_item_schema();
        $data_keys = array_keys( array_filter( $schema['properties'], array( $this, 'filter_writable_properties' ) ) );

        $data = array();

        foreach ( $data_keys as $key ) {

            $value = $request->get_param( $key );

            if ( is_null( $value ) ) {
                continue;
            }

            if ( in_array( $key, array( 'affiliate_id', 'user_id' ) ) ) {
                $data[$key] = absint( $value );
            }

            if ( $key == 'email' ) {
                $data[$key] = sanitize_email( $value );
            }

            if ( in_array( $key, array( 'first_name', 'last_name' ) ) ) {
                $data[$key] = sanitize_text_field( $value );
            }

        }

        return $data;

    }


    /**
     * Get the customer's schema, conforming to JSON Schema.
     * 
     * @return array
     * 
     */
    public function get_item_schema() {

        $schema = array(
            '$schema'    => 'http://json-schema.org/draft-04/schema#',
            'title'      => 'customer',
            'type'       => 'object',
            'properties' => array(
                'id' => array(
                    'description' => esc_html( __( 'Unique identifier for the object.', 'slicewp' ) ),
                    'type'        => 'integer',
                    'readonly'    => true
                ),
                'affiliate_id' => array(
                    'description' => esc_html( __( 'The unique identifier of the affiliate that referred this customer.', 'slicewp' ) ),
                    'type'        => 'integer'
                ),
                'user_id' => array(
                    'description' => esc_html( __( 'The unique identifier of the user this customer is attached to.', 'slicewp' ) ),
                    'type'        => 'integer'
                ),
                'email' => array(
                    'description' => esc_html( __( "The customer's email address.", 'slicewp' ) ),
                    'type'        => 'string',
                ),
                'first_name' => array(
                    'description' => esc_html( __( "The customer's first name.", 'slicewp' ) ),
                    'type'        => 'string',
                ),
                'last_name' => array(
                    'description' => esc_html( __( "The customer's last name.", 'slicewp' ) ),
                    'type'        => 'string',
                ),
                'date_created' => array(
                    'description' => esc_html( __( 'The date at which the item was created, in the GMT timezone.', 'slicewp' ) ),
                    'type'        => 'date-time',
                    'readonly'    => true
                ),
                'date_modified' => array(
                    'description' => esc_html( __( 'The date at which the item was last modified, in the GMT timezone.', 'slicewp' ) ),
                    'type'        => 'date-time',
                    'readonly'    => true
                )
            )
        );

        return $schema;

    }


    /**
     * Get the query params for collections.
     *
     * @return array
     * 
     */
    public function get_collection_params() {

        return array(
            'number' => array(
                'description'       => __( 'The number of items to query for. Use -1 to query for all.', 'slicewp' ),
                'type'              => 'integer',
                'sanitize_callback' => function( $param ) {
                    return (int)$param;
                },
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'offset' => array(
                'description'       => __( 'The number of items to offset the query by.', 'slicewp' ),
                'type'              => 'integer',
                'sanitize_callback' => 'absint',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'affiliate_id' => array(
                'description'       => __( 'The affiliate ID to query by.', 'slicewp' ),
                'type'              => 'integer',
                'sanitize_callback' => 'absint',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'date_min' => array(
                'description'       => __( 'The minimum creation date of the customer to query by.', 'slicewp' ),
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'date_max' => array(
                'description'       => __( 'The maximum creation date of the customer to query by.', 'slicewp' ),
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'order' => array(
                'description'       => __( 'How to order the results. Use ASC for ascending order or DESC for descending order.', 'slicewp' ),
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field',
                'validate_callback' => 'rest_validate_request_arg'
            )
        );

    }


    /**
     * Updates the meta data for the given item.
     * 
     * @param int   $item_id
     * @param array $meta_data
     * 
     */
    protected function update_item_meta( $item_id, $meta_data ) {

        // Sanitize array.
        $meta_data = _slicewp_array_wp_kses_post( $meta_data );

        foreach ( $meta_data as $key => $value ) {

            if ( ! is_string( $key ) ) {
                continue;
            }

            slicewp_update_customer_meta( absint( $item_id ), $key, $value );

        }

    }

}

new SliceWP_REST_API_Controller_Customers();