<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * REST API controller for "payouts".
 * 
 */
class SliceWP_REST_API_Controller_Payouts extends SliceWP_REST_API_Controller {

    /**
	 * Route base.
	 *
	 * @var    string
     * @access protected
     * 
	 */
    protected $rest_base = 'payouts';


    /**
     * Registers the routes.
     * 
     */
    public function register_routes() {

        // Get all payouts.
        register_rest_route( $this->namespace, '/' . $this->rest_base, array(
			'methods' 			  => WP_REST_Server::READABLE,
			'callback'			  => array( $this, 'get_items' ),
			'permission_callback' => array( $this, 'permissions_check' ),
            'args'                => $this->get_collection_params()
		));

        // Get single payout.
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/(?P<id>\d+)', array(
			'methods'			  => WP_REST_Server::READABLE,
			'callback'			  => array( $this, 'get_item' ),
			'permission_callback' => array( $this, 'permissions_check' )
		));

        // Create single payout.
        register_rest_route( $this->namespace, '/' . $this->rest_base . '/', array(
            'methods'             => WP_REST_Server::CREATABLE,
            'callback'			  => array( $this, 'create_item' ),
            'permission_callback' => array( $this, 'permissions_check' ),
            'args'                => array(
                'originator_user_id' => array(
                    'description'       => __( 'The ID of the user that generated the payout.', 'slicewp' ),
                    'type'              => 'integer',
                    'required'          => true,
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'amount' => array(
                    'description'       => __( "The payout's amount.", 'slicewp' ),
                    'type'              => 'number',
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'meta_data' => array(
                    'description'       => __( 'The meta data to add to the payout', 'slicewp' ),
                    'type'              => array( 'array', 'object' ),
                    'validate_callback' => 'rest_validate_request_arg'
                )
            )
        ));

        // Update single payout.
        register_rest_route( $this->namespace, '/' . $this->rest_base . '/(?P<id>\d+)', array(
            'methods'             => WP_REST_Server::EDITABLE,
            'callback'			  => array( $this, 'update_item' ),
            'permission_callback' => array( $this, 'permissions_check' ),
            'args'                => array(
                'originator_user_id' => array(
                    'description'       => __( 'The ID of the user that generated the payout.', 'slicewp' ),
                    'type'              => 'integer',
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'amount' => array(
                    'description'       => __( "The payout's amount.", 'slicewp' ),
                    'type'              => 'number',
                    'validate_callback' => 'rest_validate_request_arg'
                ),
                'meta_data' => array(
                    'description'       => __( 'The meta data to add to the payout', 'slicewp' ),
                    'type'              => array( 'array', 'object' ),
                    'validate_callback' => 'rest_validate_request_arg'
                )
            )
        ));

        // Delete single payout.
        register_rest_route( $this->namespace, '/' . $this->rest_base . '/(?P<id>\d+)', array(
            'methods'             => WP_REST_Server::DELETABLE,
            'callback'			  => array( $this, 'delete_item' ),
            'permission_callback' => array( $this, 'permissions_check' )
        ));

    }


    /**
     * Returns an array of payouts data as a rest response.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function get_items( $request ) {

        // Prepare query args.
        $args = array();

        foreach ( $request->get_params() as $key => $value ) {

            if ( in_array( $key, array_keys( $this->get_collection_params() ) ) ) {
                $args[$key] = $value;
            }

        }

        // Get payouts.
        $payouts = slicewp_get_payouts( $args );

        foreach ( $payouts as $key => $payout ) {

            $payout = (object)$payout->to_array();

            // Add payout meta.
            if ( ! is_null( $request->get_param( 'meta' ) ) && filter_var( $request->get_param( 'meta' ), FILTER_VALIDATE_BOOLEAN ) ) {

                $payout->meta = slicewp_get_payout_meta( $payout->id );

            }
            
            $payouts[$key] = $payout;

        }

        return rest_ensure_response( $payouts );

    }


    /**
     * Returns the payout with the given ID as a rest response.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function get_item( $request ) {

        $payout = slicewp_get_payout( absint( $request->get_param( 'id' ) ) );

        if ( is_null( $payout ) ) {
            return new WP_Error( 'invalid_payout_id', 'Payout does not exist.', array( 'status' => 404 ) );
        }

        $payout = (object)$payout->to_array();

        // Add payout meta.
        if ( ! is_null( $request->get_param( 'meta' ) ) && filter_var( $request->get_param( 'meta' ), FILTER_VALIDATE_BOOLEAN ) ) {

            $payout->meta = slicewp_get_payout_meta( $payout->id );

        }

        return rest_ensure_response( $payout );

    }


    /**
     * Creates a new payout.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function create_item( $request ) {

        // Prepare item data.
        $payout_data = $this->prepare_item_data_for_database( $request );

        // If there's no data to create object, return.
        if ( empty( $payout_data ) ) {
            return new WP_Error( 'payout_create_false', 'Did not create payout. No writable data provided.', array( 'status' => 400 ) );
        }

        // Add dates.
        $payout_data['date_created']  = slicewp_mysql_gmdate();
        $payout_data['date_modified'] = slicewp_mysql_gmdate();

        // Insert the item.
        $inserted = slicewp_insert_payout( $payout_data );

        // If inserted, return item.
        if ( $inserted ) {

            $payout = slicewp_get_payout( $inserted );
            $payout = (object)$payout->to_array();

            // Add meta data.
            if ( ! empty( $request->get_param( 'meta_data' ) ) ) {

                $this->update_item_meta( $inserted, $request->get_param( 'meta_data' ) );

                $payout->meta = $this->prepare_item_meta_for_response( slicewp_get_payout_meta( $inserted ) );

            }

            return rest_ensure_response( $payout );

        }

        return new WP_Error( 'payout_create_false', 'Something went wrong. Could not create payout.', array( 'status' => 500 ) );

    }


    /**
     * Updates the payout with the given ID.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function update_item( $request ) {
        
        $payout = slicewp_get_payout( absint( $request->get_param( 'id' ) ) );

        if ( is_null( $payout ) ) {
            return new WP_Error( 'invalid_payout_id', 'Payout does not exist.', array( 'status' => 404 ) );
        }

        // Prepare item data.
        $payout_data = $this->prepare_item_data_for_database( $request );

        // If there's no data to update, return.
        if ( empty( $payout_data ) && empty( $request->get_param( 'meta_data' ) ) ) {
            return new WP_Error( 'payout_update_false', 'Did not update payout. No writable data provided.', array( 'status' => 400 ) );
        }

        // Update item.
        if ( ! empty( $payout_data ) ) {

            // Add date modified before updating.
            $payout_data['date_modified'] = slicewp_mysql_gmdate();

            $updated = slicewp_update_payout( $payout->get( 'id' ), $payout_data );
            
        }

        // Update item meta.
        if ( ! empty( $request->get_param( 'meta_data' ) ) ) {

            $this->update_item_meta( $payout->get( 'id' ), $request->get_param( 'meta_data' ) );

            $updated = true;

        }

        // If updated, return item.
        if ( $updated ) {

            $payout = slicewp_get_payout( absint( $request->get_param( 'id' ) ) );
            $payout = (object)$payout->to_array();

            if ( ! empty( $request->get_param( 'meta_data' ) ) ) {

                $payout->meta = $this->prepare_item_meta_for_response( slicewp_get_payout_meta( absint( $payout->id ) ) );

            }

            return rest_ensure_response( $payout );

        }

        return new WP_Error( 'payout_update_false', 'Something went wrong. Could not update payout.', array( 'status' => 500 ) );

    }


    /**
     * Delete the payout with the given ID.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function delete_item( $request ) {

        $payout = slicewp_get_payout( absint( $request->get_param( 'id' ) ) );

        if ( is_null( $payout ) ) {
            return new WP_Error( 'invalid_payout_id', 'Payout does not exist.', array( 'status' => 404 ) );
        }

        // Add metadata to payout.
        $payout       = (object)$payout->to_array();
        $payout->meta = slicewp_get_payout_meta( $payout->id );

        // Delete payout.
        $deleted = slicewp_delete_payout( $payout->id );

        if ( $deleted ) {
            return rest_ensure_response( $payout );
        }

        return new WP_Error( 'payout_delete_false', 'Something went wrong. Could not delete payout.', array( 'status' => 500 ) );

    }


    /**
     * Filters and sanitizes the request data to prepare it for the database.
     * 
     * @param WP_REST_Request $request
     * 
     * @return array
     * 
     */
    protected function prepare_item_data_for_database( $request ) {

        $schema    = $this->get_item_schema();
        $data_keys = array_keys( array_filter( $schema['properties'], array( $this, 'filter_writable_properties' ) ) );

        $data = array();

        foreach ( $data_keys as $key ) {

            $value = $request->get_param( $key );

            if ( is_null( $value ) ) {
                continue;
            }

            if ( $key == 'originator_user_id' ) {
                $data[$key] = absint( $value );
            }

            if ( $key == 'amount' ) {
                $data[$key] = slicewp_sanitize_amount( $value );
            }

        }

        return $data;

    }


    /**
     * Get the payout's schema, conforming to JSON Schema.
     * 
     * @return array
     * 
     */
    public function get_item_schema() {

        $schema = array(
            '$schema'    => 'http://json-schema.org/draft-04/schema#',
            'title'      => 'payout',
            'type'       => 'object',
            'properties' => array(
                'id' => array(
                    'description' => esc_html( __( 'Unique identifier for the object.', 'slicewp' ) ),
                    'type'        => 'integer',
                    'readonly'    => true
                ),
                'originator_user_id' => array(
                    'description' => esc_html( __( 'The unique identifier of the user that generated the payout.', 'slicewp' ) ),
                    'type'        => 'integer'
                ),
                'amount' => array(
                    'description' => esc_html( __( "The payout's amount.", 'slicewp' ) ),
                    'type'        => 'string'
                ),
                'date_created' => array(
                    'description' => esc_html( __( 'The date at which the item was created, in the GMT timezone.', 'slicewp' ) ),
                    'type'        => 'date-time',
                    'readonly'    => true
                ),
                'date_modified' => array(
                    'description' => esc_html( __( 'The date at which the item was last modified, in the GMT timezone.', 'slicewp' ) ),
                    'type'        => 'date-time',
                    'readonly'    => true
                )
            )
        );

        return $schema;

    }


    /**
     * Get the query params for collections.
     *
     * @return array
     * 
     */
    public function get_collection_params() {

        return array(
            'number' => array(
                'description'       => __( 'The number of items to query for. Use -1 to query for all.', 'slicewp' ),
                'type'              => 'integer',
                'sanitize_callback' => function( $param ) {
                    return (int)$param;
                },
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'offset' => array(
                'description'       => __( 'The number of items to offset the query by.', 'slicewp' ),
                'type'              => 'integer',
                'sanitize_callback' => 'absint',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'date_min' => array(
                'description'       => __( 'The minimum creation date of the payout to query by.', 'slicewp' ),
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'date_max' => array(
                'description'       => __( 'The maximum creation date of the payout to query by.', 'slicewp' ),
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'order' => array(
                'description'       => __( 'How to order the results. Use ASC for ascending order or DESC for descending order.', 'slicewp' ),
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field',
                'validate_callback' => 'rest_validate_request_arg'
            )
        );

    }

    /**
     * Updates the meta data for the given item.
     * 
     * @param int   $item_id
     * @param array $meta_data
     * 
     */
    protected function update_item_meta( $item_id, $meta_data ) {

        // Sanitize array.
        $meta_data = _slicewp_array_wp_kses_post( $meta_data );

        foreach ( $meta_data as $key => $value ) {

            if ( ! is_string( $key ) ) {
                continue;
            }

            slicewp_update_payout_meta( absint( $item_id ), $key, $value );

        }

    }

}

new SliceWP_REST_API_Controller_Payouts();