<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Class to authenticate the API user and request.
 * 
 */
class SliceWP_API_Auth {

    /**
     * Constructor.
     * 
     */
    public function __construct() {

        add_filter( 'determine_current_user', array( $this, 'authenticate_user' ), 25 );

    }


    /**
     * Authenticates the user.
     * 
     */
    public function authenticate_user( $user_id ) {

        if ( ! empty( $user_id ) ) {
            return $user_id;
        }

        // Check if the call is for our endpoints.
        if ( ! $this->is_request_to_our_api() ) {
            return $user_id;
        }
    
        // Attempt to get the consumer keys.
        if ( ! empty( $_GET['consumer_key'] ) && ! empty( $_GET['consumer_secret'] ) ) {
            $consumer_key    = sanitize_text_field( $_GET['consumer_key'] );
            $consumer_secret = sanitize_text_field( $_GET['consumer_secret'] );
        }
    
        if ( empty( $consumer_key ) && ! empty( $_SERVER['PHP_AUTH_USER'] ) && ! empty( $_SERVER['PHP_AUTH_PW'] ) ) {
            $consumer_key    = sanitize_text_field( $_SERVER['PHP_AUTH_USER'] );
            $consumer_secret = sanitize_text_field( $_SERVER['PHP_AUTH_PW'] );
        }
    
        // Bail if we don't have any key.
        if ( empty( $consumer_key ) || empty( $consumer_secret ) ) {
            return $user_id;
        }
    
        $api_key = slicewp_get_api_key_by_consumer_key( $consumer_key );
    
        // Bail if we don't find an API key or if the key isn't active.
        if ( is_null( $api_key ) || $api_key->get( 'status' ) != 'active' ) {
            return $user_id;
        }
    
        // Bail if the API key does not have the correct permissions.
        if ( ! $this->has_request_permissions( $api_key ) ) {
            return $user_id;
        }
    
        // Bail if the consumer secret isn't correct.
        if ( hash_hmac( 'sha256', $consumer_secret, 'slicewp-api' ) != $api_key->get( 'consumer_secret' ) ) {
            return $user_id;
        }
    
        return absint( $api_key->get( 'user_id' ) );

    }


    /**
     * Check if the request is for our API's endpoints.
     * 
     * @return bool
     * 
     */
    private function is_request_to_our_api() {

        if ( empty( $_SERVER['REQUEST_URI'] ) ) {
            return false;
        }
    
        $rest_prefix = trailingslashit( rest_get_url_prefix() );
        $request_uri = esc_url_raw( wp_unslash( $_SERVER['REQUEST_URI'] ) );
    
        if ( ( false === strpos( $request_uri, $rest_prefix . 'slicewp/' ) ) && ( false === strpos( $request_uri, $rest_prefix . 'slicewp-' ) ) ) {
            return false;
        }

        return true;

    }


    /**
     * Check if the API key has the needed permissions to process the current request.
     * 
     * @param SliceWP_API_Key
     * 
     * @return bool
     * 
     */
    private function has_request_permissions( $api_key ) {

        $method = sanitize_text_field( $_SERVER['REQUEST_METHOD'] );

        if ( 'read_write' == $api_key->get( 'permissions' ) ) {
            return true;
        }

        if ( 'read' == $api_key->get( 'permissions' ) ) {

            if ( $method == WP_REST_Server::READABLE ) {
                return true;
            }

        }

        if ( 'write' == $api_key->get( 'permissions' ) ) {

            $write_methods = array_unique( array_merge( 
                array_map( 'trim', explode( ',', WP_REST_Server::CREATABLE ) ),
                array_map( 'trim', explode( ',', WP_REST_Server::EDITABLE ) ),
                array_map( 'trim', explode( ',', WP_REST_Server::DELETABLE ) )
            ));

            if ( in_array( $method, $write_methods ) ) {
                return true;
            }

        }

        return false;

    }

}

new SliceWP_API_Auth();