<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Validates and handles the adding of the new affiliate group in the database.
 *
 */
function slicewp_admin_action_add_affiliate_group() {

	// Verify for nonce.
	if ( empty( $_POST['slicewp_token'] ) || ! wp_verify_nonce( $_POST['slicewp_token'], 'slicewp_add_affiliate_group' ) ) {
        return;
    }

	// Verify for group name.
	if ( empty( $_POST['group_name'] ) ) {

		slicewp_admin_notices()->register_notice( 'affiliate_group_name_missing', '<p>' . __( 'Please set a name for your new affiliate group.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'affiliate_group_name_missing' );

		return;

	}

    // Verify that the name isn't already taken.
    $affiliate_groups_count = slicewp_get_collections( array( 'object_context' => 'affiliate', 'type' => 'group', 'name' => sanitize_text_field( $_POST['group_name'] ) ), true );

    if ( ! empty( $affiliate_groups_count ) ) {

        slicewp_admin_notices()->register_notice( 'affiliate_group_name_already_exists', '<p>' . __( 'The name for your new affiliate group already exists. Please set a different name.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'affiliate_group_name_already_exists' );

		return;

    }

	$_POST = stripslashes_deep( $_POST );

	// Prepare affiliate group data to be inserted.
	$affiliate_group_data = array(
        'object_context' => 'affiliate',
        'type'           => 'group',
        'name'           => sanitize_text_field( $_POST['group_name'] ),
		'date_created'   => slicewp_mysql_gmdate(),
		'date_modified'  => slicewp_mysql_gmdate(),
	);

	// Insert affiliate group into the database.
	$affiliate_group_id = slicewp_insert_collection( $affiliate_group_data );

	// If the affiliate group could not be inserted show a message to the user.
	if ( ! $affiliate_group_id ) {

		slicewp_admin_notices()->register_notice( 'affiliate_group_insert_false', '<p>' . __( 'Something went wrong. Could not add the affiliate group. Please try again.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'affiliate_group_insert_false' );

		return;

	}

	// Set the description.
	if ( ! empty( $_POST['group_description'] ) ) {

		slicewp_update_collection_meta( $affiliate_group_id, '_description', sanitize_textarea_field( $_POST['group_description'] ) );

	}

    // Save the commission rates.
    $commission_types = slicewp_get_commission_types();

    foreach ( $commission_types as $type => $details ) {

        if ( $type == 'recurring' || $type == 'lifetime_sale' ) {
            continue;
        }

        if ( isset( $_POST['commission_rate_' . $type] ) && is_numeric( $_POST['commission_rate_' . $type] ) ) {
            slicewp_update_collection_meta( $affiliate_group_id, '_commission_rate_' . sanitize_text_field( $type ), sanitize_text_field( $_POST['commission_rate_' . $type] ) );
        } else {
            slicewp_delete_collection_meta( $affiliate_group_id, '_commission_rate_' . sanitize_text_field( $type ) );
        }

        if ( ! empty( $_POST['commission_rate_type_' . $type] ) && in_array( $_POST['commission_rate_type_' . $type], $details['rate_types'] ) ) {
            slicewp_update_collection_meta( $affiliate_group_id, '_commission_rate_type_' . sanitize_text_field( $type ), sanitize_text_field( $_POST['commission_rate_type_' . $type] ) );
        } else {
            slicewp_delete_collection_meta( $affiliate_group_id, '_commission_rate_type_' . sanitize_text_field( $type ) );
        }

    }

    // Save sale commission rates per product/order.
    if ( ! empty( $_POST['commission_rate_type_sale'] ) && $_POST['commission_rate_type_sale'] == 'fixed_amount' ) {

        if ( ! empty( $_POST['commission_fixed_amount_rate_basis'] ) ) {
            slicewp_update_collection_meta( $affiliate_group_id, '_commission_fixed_amount_rate_basis', sanitize_text_field( $_POST['commission_fixed_amount_rate_basis'] ) );
        } else {
            slicewp_delete_collection_meta( $affiliate_group_id, '_commission_fixed_amount_rate_basis' );
        }

    }

	// Redirect to the groups page.
	wp_redirect( add_query_arg( array( 'page' => 'slicewp-affiliate-groups', 'slicewp_message' => 'affiliate_group_insert_success' ), admin_url( 'admin.php' ) ) );
	exit;

}
add_action( 'slicewp_admin_action_add_affiliate_group', 'slicewp_admin_action_add_affiliate_group', 50 );


/**
 * Validates and handles the updating of an affiliate group in the database.
 *
 */
function slicewp_admin_action_update_affiliate_group() {

	// Verify for nonce.
	if ( empty( $_POST['slicewp_token'] ) || ! wp_verify_nonce( $_POST['slicewp_token'], 'slicewp_update_affiliate_group' ) ) {
		return;
	}

	// Verify for affiliate group ID.
	if ( empty( $_POST['affiliate_group_id'] ) ) {

		slicewp_admin_notices()->register_notice( 'affiliate_group_id_missing', '<p>' . __( 'Something went wrong. Could not update the affiliate group. Please try again.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'affiliate_group_id_missing' );

		return;

	}

	// Verify for affiliate group's existance.
	$affiliate_group_id = absint( $_POST['affiliate_group_id'] );
	$affiliate_group 	= slicewp_get_collection( $affiliate_group_id );

	if ( is_null( $affiliate_group ) ) {

		slicewp_admin_notices()->register_notice( 'affiliate_group_not_exists', '<p>' . __( 'Something went wrong. Could not update the affiliate group. Please try again.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'affiliate_group_not_exists' );

		return;

	}

	// Verify for affiliate group name.
	if ( empty( $_POST['group_name'] ) ) {

		slicewp_admin_notices()->register_notice( 'affiliate_group_name_missing', '<p>' . __( 'Please set a name for your affiliate group', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'affiliate_group_name_missing' );

		return;

	}

    // Verify that this name isn't already taken by another affiliate group.
    $affiliate_groups_ids = slicewp_get_collections( array( 'number' => -1, 'fields' => 'id', 'object_context' => 'affiliate', 'type' => 'group', 'name' => sanitize_text_field( $_POST['group_name'] ) ) );

    if ( ( $key = array_search( $affiliate_group_id, $affiliate_groups_ids ) ) !== false ) {
        unset( $affiliate_groups_ids[$key] );
    }

    if ( ! empty( $affiliate_groups_ids ) ) {

        slicewp_admin_notices()->register_notice( 'affiliate_group_name_already_exists', '<p>' . __( 'The name of this affiliate group is taken by another group. Please set a different name.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'affiliate_group_name_already_exists' );

		return;

    }

	$_POST = stripslashes_deep( $_POST );

	// Prepare affiliate group data to be updated.
	$affiliate_group_data = array(
		'name'			=> sanitize_text_field( $_POST['group_name'] ), 
		'date_modified' => slicewp_mysql_gmdate()
	);

	// Update affiliate group into the database.
	$updated = slicewp_update_collection( $affiliate_group_id, $affiliate_group_data );

	// If the affiliate group could not be updated, show a message to the user.
	if ( ! $updated ) {

		slicewp_admin_notices()->register_notice( 'affiliate_group_update_false', '<p>' . __( 'Something went wrong. Could not update the affiliate group. Please try again.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'affiliate_group_update_false' );

		return;

	}

    // Update the description.
    if ( ! empty( $_POST['group_description'] ) ) {
        slicewp_update_collection_meta( $affiliate_group_id, '_description', sanitize_textarea_field( $_POST['group_description'] ) );
    } else {
        slicewp_delete_collection_meta( $affiliate_group_id, '_description' );
    }

    // Save the commission rates.
    $commission_types = slicewp_get_commission_types();

    foreach ( $commission_types as $type => $details ) {

        if ( $type == 'recurring' || $type == 'lifetime_sale' ) {
            continue;
        }

        if ( isset( $_POST['commission_rate_' . $type] ) && is_numeric( $_POST['commission_rate_' . $type] ) ) {
            slicewp_update_collection_meta( $affiliate_group_id, '_commission_rate_' . sanitize_text_field( $type ), sanitize_text_field( $_POST['commission_rate_' . $type] ) );
        } else {
            slicewp_delete_collection_meta( $affiliate_group_id, '_commission_rate_' . sanitize_text_field( $type ) );
        }

        if ( ! empty( $_POST['commission_rate_type_' . $type] ) && in_array( $_POST['commission_rate_type_' . $type], $details['rate_types'] ) ) {
            slicewp_update_collection_meta( $affiliate_group_id, '_commission_rate_type_' . sanitize_text_field( $type ), sanitize_text_field( $_POST['commission_rate_type_' . $type] ) );
        } else {
            slicewp_delete_collection_meta( $affiliate_group_id, '_commission_rate_type_' . sanitize_text_field( $type ) );
        }

    }

    // Save sale commission rates per product/order.
    if ( ! empty( $_POST['commission_rate_type_sale'] ) && $_POST['commission_rate_type_sale'] == 'fixed_amount' ) {

        if ( ! empty( $_POST['commission_fixed_amount_rate_basis'] ) ) {
            slicewp_update_collection_meta( $affiliate_group_id, '_commission_fixed_amount_rate_basis', sanitize_text_field( $_POST['commission_fixed_amount_rate_basis'] ) );
        } else {
            slicewp_delete_collection_meta( $affiliate_group_id, '_commission_fixed_amount_rate_basis' );
        }

    }

	// Redirect to the edit page of the affiliate group with a success message.
	wp_redirect( add_query_arg( array( 'page' => 'slicewp-affiliate-groups', 'subpage' => 'edit-affiliate-group', 'affiliate_group_id' => $affiliate_group_id, 'slicewp_message' => 'affiliate_group_update_success' ), admin_url( 'admin.php' ) ) );
	exit;

}
add_action( 'slicewp_admin_action_update_affiliate_group', 'slicewp_admin_action_update_affiliate_group', 50 );


/**
 * Validates and handles the deleting of an affiliate group from the database.
 *
 */
function slicewp_admin_action_delete_affiliate_group() {

	// Verify for nonce.
	if ( empty( $_GET['slicewp_token'] ) || ! wp_verify_nonce( $_GET['slicewp_token'], 'slicewp_delete_affiliate_group' ) ) {
        return;
    }

	// Verify for affiliate group ID.
	if ( empty( $_GET['affiliate_group_id'] ) ) {
        return;
    }

	// Delete the affiliate group (collection).
	$deleted = slicewp_delete_collection( absint( $_GET['affiliate_group_id'] ) );

	if ( ! $deleted ) {

		slicewp_admin_notices()->register_notice( 'affiliate_group_delete_false', '<p>' . __( 'Something went wrong. Could not delete the affiliate group. Please try again.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'affiliate_group_delete_false' );

		return;

	}

	// Redirect to the current page.
	wp_redirect( add_query_arg( array( 'page' => 'slicewp-affiliate-groups', 'slicewp_message' => 'affiliate_group_delete_success' ), admin_url( 'admin.php' ) ) );
	exit;

}
add_action( 'slicewp_admin_action_delete_affiliate_group', 'slicewp_admin_action_delete_affiliate_group', 50 );


/**
 * Validates and handles the bulk deleting of affiliate groups from the database.
 * 
 */
function slicewp_admin_action_bulk_action_affiliate_groups_delete() {

	if ( empty( $_REQUEST['action'] ) || $_REQUEST['action'] != 'delete' ) {
		return;
	}

	if ( empty( $_REQUEST['slicewp_token'] ) || ! wp_verify_nonce( $_REQUEST['slicewp_token'], 'slicewp_bulk_action_affiliate_groups' ) ) {
		return;
	}

	if ( empty( $_REQUEST['affiliate_group_ids'] ) ) {
		return;
	}

	$affiliate_group_ids = array_map( 'absint', $_REQUEST['affiliate_group_ids'] );
	$deleted_ids  	     = 0;

	foreach ( $affiliate_group_ids as $group_id ) {

		// Delete affiliate group.
        $deleted = slicewp_delete_collection( $group_id );

		if ( $deleted ) {
			$deleted_ids++;
		}

	}

	// Redirect to the current page.
	wp_redirect( add_query_arg( array( 'page' => 'slicewp-affiliate-groups', 'slicewp_message' => 'bulk_action_affiliate_groups_delete_success', 'updated' => absint( $deleted_ids ) ), admin_url( 'admin.php' ) ) );
	exit;

}
add_action( 'admin_init', 'slicewp_admin_action_bulk_action_affiliate_groups_delete' );


/**
 * Validates and handles the deleting of an affiliate group from the database.
 *
 */
function slicewp_admin_action_make_affiliate_group_default() {

	// Verify for nonce.
	if ( empty( $_GET['slicewp_token'] ) || ! wp_verify_nonce( $_GET['slicewp_token'], 'slicewp_make_affiliate_group_default' ) ) {
        return;
    }

	// Verify for affiliate group ID.
	if ( empty( $_GET['affiliate_group_id'] ) ) {
        return;
    }

    $affiliate_group = slicewp_get_collection( absint( $_GET['affiliate_group_id'] ) );

    // Make sure collection is valid.
    if ( is_null( $affiliate_group ) || $affiliate_group->get( 'object_context' ) != 'affiliate' || $affiliate_group->get( 'type' ) != 'group' ) {

        slicewp_admin_notices()->register_notice( 'affiliate_group_invalid', '<p>' . __( 'The affiliate group is not valid. Could not set it as default.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'affiliate_group_invalid' );

        return;

    }

    // Update the default group.
	$settings = slicewp_get_option( 'settings' );

    $settings['default_affiliate_group'] = absint( $affiliate_group->get( 'id' ) );

    slicewp_update_option( 'settings', $settings );

	// Redirect to the current page.
	wp_redirect( add_query_arg( array( 'page' => 'slicewp-affiliate-groups', 'slicewp_message' => 'affiliate_group_make_default_success', 'updated' => absint( $affiliate_group->get( 'id' ) ) ), admin_url( 'admin.php' ) ) );
	exit;

}
add_action( 'slicewp_admin_action_make_affiliate_group_default', 'slicewp_admin_action_make_affiliate_group_default', 50 );


/**
 * Validates and handles the "add affiliates to group" action from the single/bulk modal action.
 * 
 */
function slicewp_admin_action_add_affiliates_to_group() {

	// Verify for nonce.
	if ( empty( $_POST['slicewp_token'] ) || ! wp_verify_nonce( $_POST['slicewp_token'], 'slicewp_add_affiliates_to_group' ) ) {
        return;
    }

	// Verify for expected data.
	if ( empty( $_POST['affiliate_ids'] ) || empty( $_POST['affiliate_group_id'] ) ) {
		return;
	}

	$affiliate_ids 		= array_filter( array_map( 'absint', explode( ',', $_POST['affiliate_ids'] ) ) );
	$affiliate_group_id = absint( $_POST['affiliate_group_id'] );

	if ( empty( $affiliate_ids ) || empty( $affiliate_group_id ) ) {
		return;
	}

	$affiliate_group = slicewp_get_collection( $affiliate_group_id );

	if ( is_null( $affiliate_group ) ) {
		return;
	}

	foreach ( $affiliate_ids as $affiliate_id ) {

		slicewp_set_object_collections( $affiliate_id, 'affiliate', 'group', $affiliate_group_id );

	}

	$redirect_url = ( ! empty( $_POST['_wp_http_referer'] ) ? $_POST['_wp_http_referer'] : '' );

	wp_redirect( add_query_arg( array( 'slicewp_message' => 'add_affiliates_to_group_success', 'updated' => urlencode( json_encode( array( 'affiliate_ids' => $affiliate_ids, 'affiliate_group_id' => $affiliate_group_id ) ) ) ), $redirect_url ) );
	exit;

}
add_action( 'slicewp_admin_action_add_affiliates_to_group', 'slicewp_admin_action_add_affiliates_to_group' );