<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Abstract base class to build charts.
 *
 */
abstract class SliceWP_Chart {

	/**
	 * The chart's unique id.
	 *
	 * @access protected
	 * @var    string
	 *
	 */	
	protected $id = '';

	/**
	 * The label to be displayed in the card's header.
	 *
	 * @access protected
	 * @var    string
	 *
	 */
	protected $label = '';

	/**
	 * The array of filters to display on the chart.
	 *
	 * @access protected
	 * @var    array
	 *
	 */
	protected $filters = array();

	/**
	 * The raw data array from which we build the datasets.
	 *
	 * @access protected
	 * @var   array
	 *
	 */
	protected $raw_data;

	/**
	 * Array with arguments for the chart.
	 *
	 * @access protected
	 * @var    array
	 *
	 */
	protected $args;

	/**
	 * Array containing dataset arrays.
	 *
	 * A dataset array can have the following elements:
	 * - label : the label showing in the tooltip
	 * - data  : the data of the dataset
	 *
	 * @access protected
	 * @var    array
	 *
	 */
	protected $datasets = array();


	/**
	 * Constructor.
	 *
	 */
	public function __construct( $args = array() ) {

		// Set the attributes from the given args.
		foreach ( $args as $key => $value ) {

			if ( ! property_exists( $this, $key ) ) {
				continue;
			}

			$this->$key = $value;

		}

	}


	/**
	 * Generates the base data for the dataset, for each time unit.
	 *
	 * @return array
	 *
	 */
	public function generate_dataset_base_data() {

		$data = array();

		// Build the base data for the "day" time unit
		$date_period = new DatePeriod( new DateTime( $this->args['date_min'] ), new DateInterval('P1D'), new DateTime( $this->args['date_max'] ) );

        foreach ( $date_period as $key => $value ) {

            $data['day'][$value->format( 'Y-m-d' )] = 0;

        }

        // Build the base data for the "week" time unit
        if ( 15 < count( $data['day'] ) ) {

        	$first = true;

        	foreach ( $data['day'] as $date => $value ) {

        		if ( $first || 1 == date( 'N', strtotime( $date ) ) ) {
					$data['week'][$date] = 0;
				}

        		$first = false;

	        }

        }

        // Build the base data for the "month" time unit
        if ( 60 < count( $data['day'] ) ) {

        	$data['month'] = array();

        	foreach ( $data['day'] as $date => $value ) {

        		$data['month'][date( 'Y-m', strtotime( $date ) )] = 0;

        	}

        }

		return $data;

	}


	/**
	 * Generates the data for a dataset based on the given raw array of object data.
	 *
	 * @param array  $raw_data
	 * @param string $sum_attribute
	 *
	 * @return array
	 *
	 */
	public function generate_dataset_data( $raw_data, $sum_attribute = '' ) {

		$data = $this->generate_dataset_base_data();

		// Populate the "day" time unit
		foreach ( $raw_data as $object ) {

			$date = date( 'Y-m-d', strtotime( $object->get( 'date_created' ) ) );

			if ( ! isset( $data['day'][$date] ) ) {
				continue;
			}
			
			if ( empty( $sum_attribute ) ) {
				$data['day'][$date]++;
			} else {
				$data['day'][$date] += $object->get( $sum_attribute );
			}

		}

		// Populate the "week" time unit
		if ( isset( $data['week'] ) ) {

			$date_week = false;

			foreach ( $data['day'] as $date => $value ) {

				if ( isset( $data['week'][$date] ) ) {
					$date_week = $date;
				}

				if ( $date_week ) {
					$data['week'][$date_week] += $value;
				}

			}

		}

		// Populate the "month" time unit
		if ( isset( $data['month'] ) ) {

			foreach ( $data['day'] as $date => $value ) {

	    		$date_month = date( 'Y-m', strtotime( $date ) );

	    		if ( ! isset( $data['month'][$date_month] ) ) {
					continue;
				}

	    		$data['month'][$date_month] += $value;

	    	}

		}

		return $data;

	}


	/**
	 * Outputs all the filters.
	 *
	 */
	public function output_filters() {

		if ( empty( $this->filters ) ) {
			return;
		}

		echo '<div class="slicewp-chart-filters">';

			foreach ( $this->filters as $filter_name ) {

				$method_name = 'output_filter_' . $filter_name;

				if ( method_exists( $this, $method_name ) ) {
					$this->{$method_name}();
				}

			}

		echo '</div>';

	}


	/**
	 * Outputs the chart type switcher.
	 *
	 */
	public function output_filter_type_switch() {

		?>

			<div class="slicewp-field-wrapper slicewp-field-wrapper-chart-type">
                <a href="#" class="slicewp-active" data-chart-type="line">
                    <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><rect x="0" fill="none" width="24" height="24"/><g><path d="M3 19h18v2H3zm3-3c1.1 0 2-.9 2-2 0-.5-.2-1-.5-1.3L8.8 10H9c.5 0 1-.2 1.3-.5l2.7 1.4v.1c0 1.1.9 2 2 2s2-.9 2-2c0-.5-.2-.9-.5-1.3L17.8 7h.2c1.1 0 2-.9 2-2s-.9-2-2-2-2 .9-2 2c0 .5.2 1 .5 1.3L15.2 9H15c-.5 0-1 .2-1.3.5L11 8.2V8c0-1.1-.9-2-2-2s-2 .9-2 2c0 .5.2 1 .5 1.3L6.2 12H6c-1.1 0-2 .9-2 2s.9 2 2 2z"/></g></svg>
                    <span><?php echo __( 'Line', 'slicewp' ); ?></span>
                </a>
                <a href="#" data-chart-type="bar">
                    <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><rect x="0" fill="none" width="24" height="24"/><g><path d="M21 21H3v-2h18v2zM8 10H4v7h4v-7zm6-4h-4v11h4V6zm6-3h-4v14h4V3z"/></g></svg>
                    <span><?php echo __( 'Bar', 'slicewp' ); ?></span>
                </a>
            </div>

		<?php

	}


	/**
	 * Outputs the chart time unit selector.
	 *
	 */
	public function output_filter_time_unit_select() {

		?>

			<div class="slicewp-field-wrapper slicewp-field-wrapper-chart-time-unit">
                <select name="chart_filter_time_unit" class="slicewp-select2 slicewp-chart-filter-time-unit">
                    <option value="day"><?php echo __( 'By day', 'slicewp' ); ?></option>
                    <option value="week"><?php echo __( 'By week', 'slicewp' ); ?></option>
                    <option value="month"><?php echo __( 'By month', 'slicewp' ); ?></option>
                </select>
            </div>

        <?php

	}


	/**
	 * Outputs the entire card chart.
	 *
	 */
	public function output() {

		?>

			<div id="slicewp-card-chart-<?php echo esc_attr( $this->id ); ?>" class="slicewp-card slicewp-card-chart">

				<div class="slicewp-card-header">

					<strong><?php echo ( ! empty( $this->label ) ? esc_html( $this->label ) : '&nbsp' ); ?></strong>

					<?php $this->output_filters(); ?>

				</div>

				<div class="slicewp-chart-canvas-wrapper" style="padding: 15px 0;">
					<div class="slicewp-chart-legend"><!-- --></div>
		            <canvas width="400" height="300" data-id="<?php echo esc_attr( $this->id ); ?>" data-datasets="<?php echo esc_attr( json_encode( $this->datasets ) ); ?>"></canvas>
		        </div>

			</div>

		<?php

	}

}