<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Callback for the AJAX register custom conversion call
 *
 */
function slicewp_action_ajax_register_custom_conversion() {

	// Return if the nonce token does not exist or is invalid
	if ( empty( $_POST['slicewp_token'] ) || ! wp_verify_nonce( $_POST['slicewp_token'], 'slicewp_custom_conversion' ) ) {

		slicewp_add_log( 'Custom Conversion: Commission was not created because the nonce token is either missing or could not be validated.' );
		wp_die( json_encode( array( 'success' => -1 ) ) );

	}
	
	// Get and check to see if referrer exists
	$affiliate_id = slicewp_get_referrer_affiliate_id();
	$visit_id	  = slicewp_get_referrer_visit_id();

	// Return if the referrer affiliate or visit don't exist
	if ( empty( $affiliate_id ) || empty( $visit_id ) ) {

		slicewp_add_log( 'Custom Conversion: Commission was not created because the referrer affiliate_id or visit_id is missing.' );
		wp_die( json_encode( array( 'success' => -2 ) ) );

	}

	/**
	 * Filter to block the registration of the custom conversion.
	 * 
	 * @param bool  $blocked		- Whether the custom conversion is blocked or not, false by default.
	 * @param int   $affiliate_id	- The referrer affiliate ID.
	 * @param int   $visit_id		- The referrer visit ID.
	 * @param array $_POST 			- The post data sent through the AJAX call.
	 * 
	 */
	$blocked = apply_filters( 'slicewp_block_custom_conversion', false, $affiliate_id, $visit_id, $_POST );

	if ( $blocked ) {

		slicewp_add_log( 'Custom Conversion: Commission was not created because the slicewp_block_custom_conversion filter returned true.' );
		wp_die( json_encode( array( 'success' => -3 ) ) );

	}

	// Verify if the affiliate is valid
	if ( ! slicewp_is_affiliate_valid( $affiliate_id ) ) {

		slicewp_add_log( 'Custom Conversion: Commission was not created because the affiliate is not valid.' );
		wp_die( json_encode( array( 'success' => -4 ) ) );

	}

	// Return if an origin was not provided
	if ( empty( $_POST['origin'] ) ) {

		slicewp_add_log( 'Custom Conversion: Commission was not created because the origin parameter was not provided.' );
		wp_die( json_encode( array( 'success' => -5 ) ) );

	}

	// Return if a reference was not provided
	if ( empty( $_POST['reference'] ) ) {

		slicewp_add_log( 'Custom Conversion: Commission was not created because the reference parameter was not provided.' );
		wp_die( json_encode( array( 'success' => -6 ) ) );

	}

	// Return if the type was not provided
	if ( empty( $_POST['type'] ) ) {

		slicewp_add_log( 'Custom Conversion: Commission was not created because the type parameter was not provided.' );
		wp_die( json_encode( array( 'success' => -7 ) ) );

	}

	// Return if the status was not provided
	if ( empty( $_POST['status'] ) ) {

		slicewp_add_log( 'Custom Conversion: Commission was not created because the status parameter was not provided.' );
		wp_die( json_encode( array( 'success' => -8 ) ) );

	}

	// Check to see if the visit has been marked as converted.
	$visit = slicewp_get_visit( $visit_id );

	if ( ! empty( $visit->get( 'commission_id' ) ) ) {

		slicewp_add_log( 'Custom Conversion: Commission was not created because the referrer visit is already marked as converted.' );
		wp_die( json_encode( array( 'success' => -9 ) ) );

	}

	// Set variables
	$amount    = slicewp_sanitize_amount( $_POST['amount'] );
	$origin    = sanitize_text_field( $_POST['origin'] );
	$reference = sanitize_text_field( $_POST['reference'] );
	$type 	   = sanitize_text_field( $_POST['type'] );
	$status    = sanitize_text_field( $_POST['status'] );

	// Check to see if a commission for this payment has been registered
	$commissions = slicewp_get_commissions( array( 'reference' => $reference, 'origin' => $origin ) );

	if ( ! empty( $commissions ) ) {

		slicewp_add_log( 'Custom Conversion: Commission was not created because another commission for the reference and origin already exists.' );
		wp_die( json_encode( array( 'success' => -10 ) ) );

	}


	// Calculate commission amount
	$args = array(
		'origin' 	   => $origin,
		'type'	 	   => $type,
		'affiliate_id' => $affiliate_id
	);

	$commission_amount = slicewp_calculate_commission_amount( $amount, $args );

	// Prepare commission data
	$commission_data = array(
		'affiliate_id'		=> $affiliate_id,
		'visit_id'			=> $visit_id,
		'date_created'		=> slicewp_mysql_gmdate(),
		'date_modified'		=> slicewp_mysql_gmdate(),
		'type'				=> $type,
		'status'			=> 'pending',
		'reference'			=> $reference,
		'reference_amount'	=> slicewp_sanitize_amount( $amount ),
		'origin'			=> $origin,
		'amount'			=> slicewp_sanitize_amount( $commission_amount ),
		'currency'			=> slicewp_get_setting( 'active_currency', 'USD' )
	);

	// Insert the commission
	$commission_id = slicewp_insert_commission( $commission_data );

	if ( ! empty( $commission_id ) ) {

		// Update the commission's status if it's different than pending
		if ( $status != 'pending' ) {

			slicewp_update_commission( $commission_id, array( 'status' => $status ) );

		}

		// Update the visit with the newly inserted commission_id
		slicewp_update_visit( $visit_id, array( 'date_modified' => slicewp_mysql_gmdate(), 'commission_id' => $commission_id ) );

		// Exit with success
		slicewp_add_log( sprintf( 'Custom Conversion: Commission #%s has been successfully inserted.', $commission_id ) );
		wp_die( json_encode( array( 'success' => 1 ) ) );

	} else {

		slicewp_add_log( 'Custom Conversion: Commission could not be inserted due to an unexpected error.' );
		wp_die( json_encode( array( 'success' => -11 ) ) );

	}

}
add_action( 'wp_ajax_nopriv_slicewp_action_ajax_register_custom_conversion', 'slicewp_action_ajax_register_custom_conversion' );
add_action( 'wp_ajax_slicewp_action_ajax_register_custom_conversion', 'slicewp_action_ajax_register_custom_conversion' );